/* BBBalloon - a BB4W plugin to provide balloon tooltips
   Copyright (C) 2005 kana <nicht AT s8 DOT xrea DOT con>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "BBBalloon.h"




/* Constants */

static const char BBB_NAME[] = "BBBalloon";
static const char BBB_VERSION[] = "0.0.1";
static const char BBB_AUTHOR[] = "kana";
static const char BBB_RELEASE[] = "2005-02-06/11";
static const char BBB_LINK[] = "http://nicht.s8.xrea.com/2005/02/BBBalloon";
static const char BBB_EMAIL[] = "nicht AT s8 DOT xrea DOT con";

static const char BBB_CLASS_NAME[] = "BBBalloon";
static const char BBB_RC_NAME[] = "BBBalloon.rc";
static const UINT BBB_MESSAGES[] = {BB_BROADCAST, BB_RECONFIGURE, 0};

#define IDT_TIMEOUT 1

#define BBBALLOONHOOK_CALLBACK (WM_APP+1)




/* Global variables */

static HWND g_PluginWindow = NULL;
static HMODULE g_PluginModule = NULL;
static HWND g_TooltipWindow = NULL;
static int g_DefaultTimeout = 10*1000;
static int g_MinimumTimeout = 10*1000;
static int g_MaximumTimeout = 30*1000;
static BOOL g_UseDefaultTimeout = FALSE;
static WPARAM g_DefaultIcon = NIIF_NONE;
static BOOL g_UseDefaultIcon = FALSE;
static WNDPROC g_OriginalTooltipWindowProcedure = NULL;
static HMODULE g_HookDLLModule = NULL;
static struct {
	HWND hWnd;
	UINT uID;
	UINT uCallbackMessage;
} g_LastActiveTrayIcon;








/* Misc. functions */

PRINTF_LIKE(1, 2)
static void
notice(const char* format, ...)
{
	char buf[80 * 25];
	va_list va;

	va_start(va, format);
	vsnprintf(buf, NUMBER_OF(buf), format, va);
	buf[NUMBER_OF(buf) - 1] = '\0';
	va_end(va);

	MessageBox( g_PluginWindow, buf, BBB_NAME,
	            MB_OK | MB_SETFOREGROUND | MB_TOPMOST );
}




static int
calc_timeout(int timeout)
{
	if (g_UseDefaultTimeout)
		return g_DefaultTimeout;
	else if (timeout < g_MinimumTimeout)
		return g_MinimumTimeout;
	else if (g_MaximumTimeout < timeout)
		return g_MaximumTimeout;
	else
		return timeout;
}




static WPARAM
icon_from_token(const Token* icon)
{
	if (!my_strncmp("INFO", icon->string, icon->length))
		return NIIF_INFO;
	else if(!my_strncmp("WARNING", icon->string, icon->length))
		return NIIF_WARNING;
	else if (!my_strncmp("ERROR", icon->string, icon->length))
		return NIIF_ERROR;
	else /* if (!my_strncmp("NONE", icon->string, icon->length)) */
		return NIIF_NONE;
}




static BOOL
read_settings(const char* filename, HMODULE hmodule)
{
	char rc[MAX_PATH];
	FILE* file;
	char line[256];
	char* nl;

	if (!get_file_path(rc, NUMBER_OF(rc), filename, hmodule))
		return FALSE;

	file = fopen(rc, "rt");
	if (file == NULL) 
		return TRUE;

	while (fgets(line, NUMBER_OF(line), file) != NULL) {
		if (line[0] != '@')
			continue;

		nl = strchr(line, '\n');
		if (nl != NULL)
			*nl = '\0';

		SendMessage(GetBBWnd(), BB_BROADCAST, 0, (LPARAM)line);
	}

	fclose(file);
	return TRUE;
}




PRINTF_LIKE(2, 3)
static void
broam_error(const char* broam, const char* format, ...)
{
	char buf[80 * 25];
	va_list va;

	va_start(va, format);
	vsnprintf(buf, NUMBER_OF(buf), format, va);
	buf[NUMBER_OF(buf) - 1] = '\0';
	va_end(va);

	notice("%s\n\n%s", broam, buf);
}


static BOOL
tokenize_check( int min, int max, BOOL allow_rest,
                int token_num, const char* rest,
                const char* broam )
{
	if ((min == max) && (token_num != min)) {
		broam_error( broam, "%d arg%s required (%d given)",
		             min, (1 < min ? "s" : ""), token_num );
		return FALSE;
	}

	if ((token_num < min) || (max < token_num)) {
		broam_error( broam, "%d or %d arg%s required (%d given)",
		             min, max, (1 < max ? "s" : ""), token_num );
		return FALSE;
	}

	if ((!allow_rest) && (rest[0] != '\0')) {
		broam_error(broam, "Trailing characters");
		return FALSE;
	}

	return TRUE;
}




static void
LastActiveTrayIcon_Remember(NOTIFYICONDATA* nid)
{
	g_LastActiveTrayIcon.hWnd = nid->hWnd;
	g_LastActiveTrayIcon.uID = nid->uID;
	g_LastActiveTrayIcon.uCallbackMessage = nid->uCallbackMessage;
}


static void
LastActiveTrayIcon_Forget(void)
{
	g_LastActiveTrayIcon.hWnd = NULL;
	/* g_LastActiveTrayIcon.uID = ... */
	/* g_LastActiveTrayIcon.uCallbackMessage = ... */
}




static void
TOOLINFO_Initialize(TOOLINFO* pti)
{
	ZeroMemory(pti, sizeof(*pti));
	pti->cbSize = sizeof(*pti);
	pti->hwnd = g_PluginWindow;
	/* pti->uId = 0; */
	/* pti->hinst = NULL; */
}


static LRESULT CALLBACK Tooltip_WindowProcedure(HWND, UINT, WPARAM, LPARAM);

static HWND
Tooltip_New(void)
{
	HWND self;
	TOOLINFO ti;

	self = CreateWindowEx( WS_EX_TOOLWINDOW | WS_EX_TOPMOST,
	                       TOOLTIPS_CLASS,
	                       NULL,
	                       WS_POPUP | TTS_NOPREFIX | TTS_BALLOON,
	                       0, 0, 0, 0,
	                       NULL,
	                       NULL,
	                       g_PluginModule,
	                       NULL );
	if (self == NULL)
		goto E_CreateWindowEx;

	g_OriginalTooltipWindowProcedure
	  = (WNDPROC)SetWindowLongPtr( self, GWLP_WNDPROC,
	                               (LONG_PTR)Tooltip_WindowProcedure );
	if (g_OriginalTooltipWindowProcedure == NULL)
		goto E_Subclassing;

	TOOLINFO_Initialize(&ti);
	ti.uFlags = TTF_TRACK;
	ti.lpszText = NULL;
	if (!SendMessage(self, TTM_ADDTOOL, 0, (LPARAM)&ti))
		goto E_AddTool;

	MakeSticky(self);

	return self;

E_AddTool:
E_Subclassing:
	DestroyWindow(self);
E_CreateWindowEx:
	return NULL;
}


static void Tooltip_Hide(HWND, LPARAM);

static void
Tooltip_Del(HWND self)
{
	TOOLINFO ti;

	Tooltip_Hide(g_TooltipWindow, NIN_BALLOONHIDE);

	RemoveSticky(self);

	TOOLINFO_Initialize(&ti);
	SendMessage(self, TTM_DELTOOL, 0, (LPARAM)&ti);

	DestroyWindow(self);
}


static void
Tooltip_Show( HWND self, const char* text, const char* title,
              int timeout, WPARAM icon )
{
	TOOLINFO ti;

	TOOLINFO_Initialize(&ti);
	ti.lpszText = (LPTSTR)text;

		/* BUGS: Should we use NIN_BALLOONTIMEOUT? */
	Tooltip_Hide(self, NIN_BALLOONTIMEOUT);

	SendMessage(self, TTM_UPDATETIPTEXT, 0, (LPARAM)&ti);
	SendMessage(self, TTM_SETTITLE, icon, (LPARAM)title);
	SendMessage(self, TTM_TRACKACTIVATE, TRUE, (LPARAM)&ti);

	SetTimer(g_PluginWindow, IDT_TIMEOUT, timeout, NULL);
}


/* BUGS: Assumes that NOTIFYICONDATA is the one of version 5. */
static void
Tooltip_ShowByNotifyIconData(HWND self, NOTIFYICONDATA* nid)
{
	if (nid->cbSize < sizeof(NOTIFYICONDATAA))
		return;

	if (sizeof(NOTIFYICONDATAW) <= nid->cbSize) {
		char text[NUMBER_OF(nid->szInfo) * 4];
		char title[NUMBER_OF(nid->szInfoTitle) * 4];
		NOTIFYICONDATAW* nidw = (NOTIFYICONDATAW*)nid;

		translate_wcs_to_acs(text, NUMBER_OF(text), nidw->szInfo);
		translate_wcs_to_acs(title,NUMBER_OF(title),nidw->szInfoTitle);
		Tooltip_Show( self, text, title, calc_timeout(nidw->uTimeout),
		              ( g_UseDefaultIcon ? g_DefaultIcon
		                                 : nidw->dwInfoFlags & 0x3 ) );
	} else {
		Tooltip_Show( self, nid->szInfo, nid->szInfoTitle,
		              calc_timeout(nid->uTimeout),
		              ( g_UseDefaultIcon ? g_DefaultIcon
		                                 : nid->dwInfoFlags & 0x3 ) );
	}

	LastActiveTrayIcon_Remember(nid);
	PostMessage( g_LastActiveTrayIcon.hWnd,
	             g_LastActiveTrayIcon.uCallbackMessage,
	             g_LastActiveTrayIcon.uID,
	             NIN_BALLOONSHOW );
}


static void
Tooltip_Hide(HWND self, LPARAM reason)
{
	TOOLINFO ti;

	TOOLINFO_Initialize(&ti);
	SendMessage(self, TTM_TRACKACTIVATE, FALSE, (LPARAM)&ti);

	if (g_LastActiveTrayIcon.hWnd != NULL) {
		PostMessage( g_LastActiveTrayIcon.hWnd,
		             g_LastActiveTrayIcon.uCallbackMessage,
		             g_LastActiveTrayIcon.uID,
		             reason );
		LastActiveTrayIcon_Forget();
	}
}


static void
Tooltip_SetPotition(HWND self, int x, int y)
{
	SendMessage(self, TTM_TRACKPOSITION, 0, MAKELONG(x, y));
}


static void
Tooltip_SetWidth(HWND self, int width)
{
	SendMessage(self, TTM_SETMAXTIPWIDTH, 0, width);
}




static void
AutoCheck_Begin(void)
{
	char path[MAX_PATH];
	FARPROC end;
	FARPROC begin;
	HWND system_tray;

	system_tray = FindWindow("Shell_TrayWnd", NULL);
	if (system_tray == NULL) {
		notice("System tray not found");
		goto E_FindWindow;
	}

	if (!get_file_path( path, NUMBER_OF(path), "BBBalloonHook.dll",
	                    g_PluginModule) )
	{
		notice("Failed to load BBBalloonHook.dll: Too long path");
		goto E_get_file_path;
	}

	g_HookDLLModule = LoadLibrary(path);
	if (g_HookDLLModule == NULL) {
		notice("Failed to load BBBalloonHook.dll");
		goto E_LoadLibrary;
	}

	begin = GetProcAddress(g_HookDLLModule, "BBBalloonHook_Begin");
	end = GetProcAddress(g_HookDLLModule, "BBBalloonHook_End");
	if ((begin == NULL) || (end == NULL)) {
		notice("Invalid BBBalloonHook.dll");
		goto E_GetProcAddress;
	}

	if ( !(*((BOOL (*)(HWND, HWND, UINT))begin))
		(system_tray, g_PluginWindow, BBBALLOONHOOK_CALLBACK) )
	{
		notice("Failed to hook");
		goto E_BBBalloonHook_Begin;
	}

	return;

E_BBBalloonHook_Begin:
E_GetProcAddress:
	FreeLibrary(g_HookDLLModule);
	g_HookDLLModule = NULL;
E_LoadLibrary:
E_get_file_path:
E_FindWindow:
	return;
}


static void
AutoCheck_End(void)
{
	FARPROC end;
	BOOL result;

	end = GetProcAddress(g_HookDLLModule, "BBBalloonHook_End");
	if (end == NULL) {
		notice("Failed to unhook (#1)");
		return;
	}

	result = (*((BOOL (*)(void))end))();

	FreeLibrary(g_HookDLLModule);
	g_HookDLLModule = NULL;

	if (!result)
		notice("Failed to unhook (#2)");
	return;
}








/* Main functions */

static void
interpret_broam(const char* broam)
{
	Token token[8];
	int token_num;
	const char* rest;
	Token cmd;

	/* 0:@BBBalloon, 1:CMD, rest:ARG(S) */
	tokenize(broam, 2, token, &token_num, &rest);
	if (!tokenize_check(1, 2, TRUE, token_num, rest, broam))
		return;
	if (my_strncmp("@BBBalloon", token[0].string, token[0].length))
		return;

	cmd = token[1];
	if (!my_strncmp("Show", cmd.string, cmd.length)) {
		char* text;
		char* title;
		int timeout;
		WPARAM icon;

		tokenize(rest, 4, token, &token_num, &rest);
		if (!tokenize_check(1, 4, FALSE, token_num, rest, broam))
			return;

		text = my_strndup(token[0].string, token[0].length);
		if ((text != NULL) && (token[0].string[-1] == '"'))
			descape(text);

		if (2 <= token_num) {
			title = my_strndup(token[1].string, token[1].length);
			if ((title != NULL) && (token[1].string[-1] == '"'))
				descape(title);
		} else {
			title = NULL;
		}

		if ( (3 <= token_num)
		     && my_strncmp("-", token[2].string, token[2].length) )
		{
			timeout = my_atoi(token[2].string, token[2].length);
			if (timeout <= 0)
				timeout = 1;
		} else {
			timeout = g_DefaultTimeout;
		}

		if ( (4 <= token_num)
		     && my_strncmp("-", token[3].string, token[3].length) )
		{
			icon = icon_from_token(&(token[3]));
		} else {
			icon = g_DefaultIcon;
		}

		Tooltip_Show(g_TooltipWindow, text, title, timeout, icon);

		free(title);
		free(text);
	} else if (!my_strncmp("Hide", cmd.string, cmd.length)) {
		LPARAM reason;

		tokenize(rest, 1, token, &token_num, &rest);
		if (!tokenize_check(0, 1, FALSE, token_num, rest, broam))
			return;

		if (token_num < 1)
			reason = NIN_BALLOONTIMEOUT;
		else if (!my_strncmp("click", token[0].string,token[0].length))
			reason = NIN_BALLOONUSERCLICK;
		else if(!my_strncmp("timeout",token[0].string,token[0].length))
			reason = NIN_BALLOONTIMEOUT;
		else if (!my_strncmp("hide", token[0].string, token[0].length))
			reason = NIN_BALLOONHIDE;
		else
			reason = NIN_BALLOONTIMEOUT;
		Tooltip_Hide(g_TooltipWindow, reason);
	} else if (!my_strncmp("Position", cmd.string, cmd.length)) {
		tokenize(rest, 2, token, &token_num, &rest);
		if (!tokenize_check(2, 2, FALSE, token_num, rest, broam))
			return;

		Tooltip_SetPotition(
		  g_TooltipWindow,
		  my_atoi(token[0].string, token[0].length),
		  my_atoi(token[1].string, token[1].length)
		);
	} else if (!my_strncmp("Width", cmd.string, cmd.length)) {
		tokenize(rest, 1, token, &token_num, &rest);
		if (!tokenize_check(1, 1, FALSE, token_num, rest, broam))
			return;

		Tooltip_SetWidth( g_TooltipWindow,
		                  my_atoi(token[0].string, token[0].length) );
	} else if (!my_strncmp("Auto", cmd.string, cmd.length)) {
		tokenize(rest, 1, token, &token_num, &rest);
		if (!tokenize_check(1, 1, FALSE, token_num, rest, broam))
			return;

		if (!my_strncmp("true", token[0].string, token[0].length)) {
			if (g_HookDLLModule == NULL)
				AutoCheck_Begin();
		} else {
			if (g_HookDLLModule != NULL)
				AutoCheck_End();
		}
	} else if (!my_strncmp("Timeout", cmd.string, cmd.length)) {
		tokenize(rest, 1, token, &token_num, &rest);
		if (!tokenize_check(1, 1, FALSE, token_num, rest, broam))
			return;

		g_DefaultTimeout = my_atoi(token[0].string, token[0].length);
		if (g_DefaultTimeout <= 0)
			g_DefaultTimeout = 1;
	} else if (!my_strncmp("MinimumTimeout", cmd.string, cmd.length)) {
		tokenize(rest, 1, token, &token_num, &rest);
		if (!tokenize_check(1, 1, FALSE, token_num, rest, broam))
			return;

		g_MinimumTimeout = my_atoi(token[0].string, token[0].length);
		if (g_MinimumTimeout <= 0)
			g_MinimumTimeout = 1;
	} else if (!my_strncmp("MaximumTimeout", cmd.string, cmd.length)) {
		tokenize(rest, 1, token, &token_num, &rest);
		if (!tokenize_check(1, 1, FALSE, token_num, rest, broam))
			return;

		g_MaximumTimeout = my_atoi(token[0].string, token[0].length);
		if (g_MaximumTimeout <= 0)
			g_MaximumTimeout = 1;
	} else if (!my_strncmp("UseDefaultTimeout", cmd.string, cmd.length)) {
		tokenize(rest, 1, token, &token_num, &rest);
		if (!tokenize_check(1, 1, FALSE, token_num, rest, broam))
			return;

		if (!my_strncmp("true", token[0].string, token[0].length))
			g_UseDefaultTimeout = TRUE;
		else
			g_UseDefaultTimeout = FALSE;
	} else if (!my_strncmp("Icon", cmd.string, cmd.length)) {
		tokenize(rest, 1, token, &token_num, &rest);
		if (!tokenize_check(1, 1, FALSE, token_num, rest, broam))
			return;

		g_DefaultIcon = icon_from_token(&(token[0]));
	} else if (!my_strncmp("UseDefaultIcon", cmd.string, cmd.length)) {
		tokenize(rest, 1, token, &token_num, &rest);
		if (!tokenize_check(1, 1, FALSE, token_num, rest, broam))
			return;

		if (!my_strncmp("true", token[0].string, token[0].length))
			g_UseDefaultIcon = TRUE;
		else
			g_UseDefaultIcon = FALSE;
	} else {
		broam_error(broam, "Unknown bro@m");
	}
}




static LRESULT CALLBACK
Tooltip_WindowProcedure(HWND hwnd, UINT msg, WPARAM wp, LPARAM lp)
{
	switch (msg) {
	default:
		break;

	case WM_LBUTTONUP:
	case WM_RBUTTONUP:
	case WM_MBUTTONUP:
	case WM_XBUTTONUP:
		Tooltip_Hide(hwnd, NIN_BALLOONUSERCLICK);
		break;
	}

	return CallWindowProc( g_OriginalTooltipWindowProcedure,
	                       hwnd, msg, wp, lp );
}




static LRESULT CALLBACK
BBB_WindowProcedure(HWND hwnd, UINT msg, WPARAM wp, LPARAM lp)
{
	switch (msg) {
	default:
		break;


	case BB_BROADCAST:
		interpret_broam((const char*)lp);
		break;

	case BB_RECONFIGURE:
		if (!read_settings(BBB_RC_NAME, g_PluginModule))
			notice("Failed to load settings: too long path");
		break;


	case BBBALLOONHOOK_CALLBACK: {
		NOTIFYICONDATA* nid = (NOTIFYICONDATA*)lp;

		if ((wp == NIM_ADD) || (wp == NIM_MODIFY)) {
		  if (nid->uFlags & NIF_INFO) {
		    Tooltip_ShowByNotifyIconData(g_TooltipWindow, nid);
		  }
		}
		} break;


	case WM_CREATE:
		g_PluginWindow = hwnd;

		g_TooltipWindow = Tooltip_New();
		if (g_TooltipWindow == NULL) {
			notice("Failed to create tooltip");
			return 1;
		}

		SendMessage( GetBBWnd(), BB_REGISTERMESSAGE,
		             (WPARAM)hwnd, (LPARAM)BBB_MESSAGES );

		SendMessage(hwnd, BB_RECONFIGURE, 0, 0);
		break;

	case WM_DESTROY:
		KillTimer(hwnd, IDT_TIMEOUT);

		SendMessage( GetBBWnd(), BB_UNREGISTERMESSAGE,
		             (WPARAM)hwnd, (LPARAM)BBB_MESSAGES );

		Tooltip_Del(g_TooltipWindow);
		break;

	case WM_TIMER:
		if (wp == IDT_TIMEOUT) {
			KillTimer(hwnd, IDT_TIMEOUT);
			Tooltip_Hide(g_TooltipWindow, NIN_BALLOONTIMEOUT);
		}
		break;
	}

	return DefWindowProc(hwnd, msg, wp, lp);
}








/* Public functions */

DLL_EXPORT int
beginPlugin(HINSTANCE hplugininstance)
{
	WNDCLASS wc;

	g_PluginModule = (HMODULE)hplugininstance;

	ZeroMemory(&wc, sizeof(wc));
	wc.lpfnWndProc = BBB_WindowProcedure;
	wc.hInstance = hplugininstance;
	wc.lpszClassName = BBB_CLASS_NAME;
	if (!RegisterClass(&wc)) {
		notice("Failed RegisterClass()");
		goto E_RegisterClass;
	}

	g_PluginWindow = CreateWindowEx(
	                   WS_EX_TOOLWINDOW,
	                   BBB_CLASS_NAME,
	                   NULL,
	                   0,
	                   0, 0, 0, 0,
	                   (is_2k_or_later() ? HWND_MESSAGE : NULL),
	                   NULL,
	                   hplugininstance,
	                   NULL
	                 );
	if (g_PluginWindow == NULL) {
		notice("Failed CreateWindowEx() (Plugin Window)");
		goto E_CreateWindowEx;
	}

	return 0;


E_CreateWindowEx:
	UnregisterClass(BBB_CLASS_NAME, hplugininstance);
E_RegisterClass:
	return !0;
}




DLL_EXPORT void
endPlugin(HINSTANCE hplugininstance)
{
	if (g_HookDLLModule != NULL)
		AutoCheck_End();

	DestroyWindow(g_PluginWindow);  g_PluginWindow = NULL;
	UnregisterClass(BBB_CLASS_NAME, hplugininstance);
}




DLL_EXPORT LPCSTR
pluginInfo(int field)
{
	switch (field) {
	default: /* FALLTHRU */
	case PLUGIN_NAME: return BBB_NAME;
	case PLUGIN_VERSION: return BBB_VERSION;
	case PLUGIN_AUTHOR: return BBB_AUTHOR;
	case PLUGIN_RELEASE: return BBB_RELEASE;
	case PLUGIN_LINK: return BBB_LINK;
	case PLUGIN_EMAIL: return BBB_EMAIL;
	}
}




/* __END__ */
