/* Misc. functions */

#include "BBBalloon.h"

#include <ctype.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>








BOOL
is_2k_or_later(void)
{
	OSVERSIONINFO os;

	os.dwOSVersionInfoSize = sizeof(os);
	GetVersionEx(&os);

	return 5 <= os.dwMajorVersion;
}




static BOOL
get_module_dir(char* buf, int size, HMODULE hmodule)
{
	char* bs;

	if (!GetModuleFileName(hmodule, buf, size))
		return FALSE;
	buf[size - 1] = '\0';

	bs = strrchr(buf, '\\');
	if (bs != NULL)
		bs[1] = '\0';

	return TRUE;
}


BOOL
get_file_path(char* buf, int size, const char* filename, HMODULE hmodule)
{
	char dir[MAX_PATH];

	if (!get_module_dir(dir, NUMBER_OF(dir), hmodule))
		return FALSE;

	if (size < strlen(dir) + strlen(filename) + 1)
		return FALSE;

	strcpy(buf, dir);
	strcat(buf, filename);

	return TRUE;
}




char*
my_strncpy(char* dest, const char* src, int n)
{
	if (0 < n) {
		strncpy(dest, src, n-1);
		dest[n - 1] = '\0';
	}

	return dest;
}


char*
my_strndup(const char* s, int n)
{
	char* p;

	if (n < 0)
		return NULL;

	p = malloc(sizeof(char) * (n+1));
	if (p != NULL)
		my_strncpy(p, s, n+1);

	return p;
}


int
my_strncmp(const char* sz, const char* nsz, int n)
{
	int r;

	if ((r = strncmp(sz, nsz, n)))
		return r;
	else
		return (sz[n] == '\0' ? 0 : 1);
}


void
tokenize( const char* source,
          int max,
          Token token[],
          int* token_num,
          const char** rest )
{
	const char* WS = " \t";  /* WhiteSpace */
	const char* head;
	const char* tail;
	int n;

	head = source + strspn(source, WS);
	n = 0;
	while ((*head != '\0') && (n < max)) {
		head += strspn(head, WS);

		if (*head == '\'') {
			head++;
			tail = head + strcspn(head, "'");
		} else if (*head == '"') {
			head++;
			tail = head;
			while ((*tail != '\0') && (*tail != '"')) {
				if ((tail[0] == '\\') && (tail[1] != '\0'))
					tail++;
				tail++;
			}
		} else {
			tail = head + strcspn(head, WS);
		}

		token[n].string = head;
		token[n].length = (tail - head) / sizeof(char);

		head = tail + ((*tail == '\'') || (*tail == '"') ? 1 : 0);
		n++;
	}

	*token_num = n;
	*rest = head + strspn(head, WS);
}


void
descape(char* s)
{
	char* i;
	char* o;

	i = o = s;
	while (*i != '\0') {
		if (*i == '\\') {
			i++;
			if (*i == '\0') {
				*o = '\\';
				break;
			} else if (*i == 'n') {
				*i = '\n';
			}
		}

		*o = *i;
		o++;
		i++;
	}

	*o = '\0';
}


int
my_atoi(const char* s, int n)
{
	int result;
	int i;

	if (!(0 < n))
		return 0;

	result = 0;
	i = (s[0] == '-' ? 1 : 0);
	while ((i < n) && isdigit(s[i])) {
		result = result*10 + (s[i] - '0');
		i++;
	}

	return (s[0] == '-' ? -result : result);
}


void
translate_wcs_to_acs(char* dest, int dest_size, LPCWSTR src)
{
	dest[0] = '\0';
	WideCharToMultiByte(CP_ACP, 0, src, -1, dest, dest_size, NULL, NULL);
	dest[dest_size - 1] = '\0';
}




/* __END__ */
