/* BBLifeGame */

#include <stdlib.h>

#define WIN32_LEAN_AND_MEAN
#include <windows.h>

#include <BBApi.h>




#define NUMBER_OF(a) (sizeof(a) / sizeof((a)[0]))
#define NOP()
#define G_FIELD(x,y) (G.field[(y) * G.x_num + (x)])
#define G_FIELD_TMP(x,y) (G.field_tmp[(1+(y)) * (1+G.x_num+1) + (1+(x))])




static const char BBLG_NAME[] = "BBLifeGame";
static const char BBLG_VERSION[] = "0.0.0";
static const char BBLG_AUTHOR[] = "kana";
static const char BBLG_RELEASE[] = "2004-07-24/11-06";
static const char BBLG_LINK[]
                      = "http://nicht.s8.xrea.com/2004/07/24/BBLifeGame";
static const char BBLG_EMAIL[] = "nicht AT s8 DOT xrea DOT con";

static const UINT BBLG_MSGS[] = {
	BB_BROADCAST,
	BB_RECONFIGURE,
	0
};

static const char STATE_LIFE = '#';
static const char STATE_NONE = '.';

#define TIMER_ID_STEP 1




static struct {
	HWND window;
	char rc_path[MAX_PATH];
	char* field;  /* x_num * y_num */
	char* field_tmp;  /* (1 + x_num + 1) * (1 + y_num + 1) */

	int x;
	int y;
	int x_padding;
	int y_padding;
	int x_num;
	int y_num;
	int cell_width;
	int cell_height;
	int step_interval;
	int snap_to_edge;
	bool always_on_top;
	int base_style;
	int life_style;
	char init[80];
	BOOL survived_number[10];
	BOOL born_number[10];
} G;








static void
error(const char* format, ...)
{
	char buf[80*25];
	va_list va;

	va_start(va, format);
	vsnprintf(buf, NUMBER_OF(buf), format, va);
	va_end(va);

	buf[NUMBER_OF(buf) - 1] = '\0';

	MessageBox( G.window, buf, BBLG_NAME,
	            MB_OK | MB_ICONERROR
	              | MB_SETFOREGROUND | MB_TASKMODAL | MB_TOPMOST );
}


static void
SNPRINTF(char* buf, int size, const char* format, ...)
{
	va_list va;

	va_start(va, format);
	vsnprintf(buf, size, format, va);
	va_end(va);

	buf[size - 1] = '\0';
}


static void
make_always_on_top(HWND hwnd, bool flag)
{
	SetWindowPos( hwnd,
	              (flag ? HWND_TOPMOST : HWND_NOTOPMOST),
	              0x4C, 0x69, 0x74, 0x79,
	              SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOOWNERZORDER
	                | SWP_NOSIZE );
}


static void
set_visibility(HWND hwnd, bool flag)
{
	ShowWindow(hwnd, (flag ? SW_SHOWNA : SW_HIDE));
}








static const struct {const char* name; int number;} STYLE_TABLE[] = {
	{"toolbar", SN_TOOLBAR},
	{"toolbar.button", SN_TOOLBARBUTTON},
	{"toolbar.button.pressed", SN_TOOLBARBUTTONP},
	{"toolbar.label", SN_TOOLBARLABEL},
	{"toolbar.windowlabel", SN_TOOLBARWINDOWLABEL},
	{"toolbar.clock", SN_TOOLBARCLOCK},
	{"menu.title", SN_MENUTITLE},
	{"menu.frame", SN_MENUFRAME},
	{"menu.hilite", SN_MENUHILITE}
};

static const char*
style_name_from_number(int n)
{
	int i;

	for (i = 0; i < (int)NUMBER_OF(STYLE_TABLE); i++)
		if (n == STYLE_TABLE[i].number)
			return STYLE_TABLE[i].name;

	return "toolbar";
}

static int
style_number_from_name(const char* s)
{
	int i;

	for (i = 0; i < (int)NUMBER_OF(STYLE_TABLE); i++)
		if (!stricmp(s, STYLE_TABLE[i].name))
			return STYLE_TABLE[i].number;

	return SN_TOOLBAR;
}




#define RC_KEY(name) "bblifegame." name ":"
#define RC_KEY_X RC_KEY("x")
#define RC_KEY_Y RC_KEY("y")
#define RC_KEY_X_PADDING RC_KEY("x_padding")
#define RC_KEY_Y_PADDING RC_KEY("y_padding")
#define RC_KEY_X_NUM RC_KEY("x_num")
#define RC_KEY_Y_NUM RC_KEY("y_num")
#define RC_KEY_CELL_WIDTH RC_KEY("cell_width")
#define RC_KEY_CELL_HEIGHT RC_KEY("cell_height")
#define RC_KEY_STEP_INTERVAL RC_KEY("step_interval")
#define RC_KEY_SNAP_TO_EDGE RC_KEY("snap_to_edge")
#define RC_KEY_ALWAYS_ON_TOP RC_KEY("always_on_top")
#define RC_KEY_BASE_STYLE RC_KEY("base_style")
#define RC_KEY_LIFE_STYLE RC_KEY("life_style")
#define RC_KEY_INIT RC_KEY("init")
#define RC_KEY_VARIATION RC_KEY("variation")


static void
read_settings(const char* rc_path)
{
	const char* s;
	BOOL* b;

	G.x = ReadInt(rc_path, RC_KEY_X, 0);
	G.y = ReadInt(rc_path, RC_KEY_Y, 64);
	G.x_padding = ReadInt(rc_path, RC_KEY_X_PADDING, 4);
	G.y_padding = ReadInt(rc_path, RC_KEY_Y_PADDING, 4);
	G.x_num = ReadInt(rc_path, RC_KEY_X_NUM, 16);
	G.y_num = ReadInt(rc_path, RC_KEY_Y_NUM, 16);
	G.cell_width = ReadInt(rc_path, RC_KEY_CELL_WIDTH, 4);
	G.cell_height = ReadInt(rc_path, RC_KEY_CELL_HEIGHT, 4);
	G.step_interval = ReadInt(rc_path, RC_KEY_STEP_INTERVAL, 1000);
	G.snap_to_edge = ReadInt(rc_path, RC_KEY_SNAP_TO_EDGE, 10);
	G.always_on_top = ReadBool(rc_path, RC_KEY_ALWAYS_ON_TOP, true);

	s = ReadString(rc_path, RC_KEY_BASE_STYLE, "toolbar");
	G.base_style = style_number_from_name(s);
	s = ReadString(rc_path, RC_KEY_LIFE_STYLE, "menu.title");
	G.life_style = style_number_from_name(s);
	s = ReadString(rc_path, RC_KEY_INIT, "random(30/100)");
	SNPRINTF(G.init, NUMBER_OF(G.init), "%s", s);

	s = ReadString(rc_path, RC_KEY_VARIATION, "23/3");
	memset(G.survived_number, 0x00, sizeof(G.survived_number));
	memset(G.born_number, 0x00, sizeof(G.born_number));
	b = G.survived_number;
	while (*s != '\0') {
		switch (*s) {
		default:
			break;

		case '0': case '1': case '2': case '3': case '4':
		case '5': case '6': case '7': case '8': case '9':
			b[*s - '0'] = TRUE;
			break;

		case '/':
			b = G.born_number;
			break;
		}

		s++;
	}
}


static void
write_settings(const char* rc_path)
{
	const char* s;

	WriteInt(rc_path, RC_KEY_X, G.x);
	WriteInt(rc_path, RC_KEY_Y, G.y);
	WriteInt(rc_path, RC_KEY_X_PADDING, G.x_padding);
	WriteInt(rc_path, RC_KEY_Y_PADDING, G.y_padding);
	WriteInt(rc_path, RC_KEY_X_NUM, G.x_num);
	WriteInt(rc_path, RC_KEY_Y_NUM, G.y_num);
	WriteInt(rc_path, RC_KEY_CELL_WIDTH, G.cell_width);
	WriteInt(rc_path, RC_KEY_CELL_HEIGHT, G.cell_height);
	WriteInt(rc_path, RC_KEY_STEP_INTERVAL, G.step_interval);
	WriteInt(rc_path, RC_KEY_SNAP_TO_EDGE, G.snap_to_edge);
	WriteBool(rc_path, RC_KEY_ALWAYS_ON_TOP, G.always_on_top);

	s = style_name_from_number(G.base_style);
	WriteString(rc_path, RC_KEY_BASE_STYLE, s);
	s = style_name_from_number(G.life_style);
	WriteString(rc_path, RC_KEY_LIFE_STYLE, s);
	WriteString(rc_path, RC_KEY_INIT, G.init);
}


#undef RC_KEY_INIT
#undef RC_KEY_LIFE_STYLE
#undef RC_KEY_BASE_STYLE
#undef RC_KEY_ALWAYS_ON_TOP
#undef RC_KEY_SNAP_TO_EDGE
#undef RC_KEY_STEP_INTERVAL
#undef RC_KEY_CELL_HEIGHT
#undef RC_KEY_CELL_WIDTH
#undef RC_KEY_Y_NUM
#undef RC_KEY_X_NUM
#undef RC_KEY_Y_PADDING
#undef RC_KEY_X_PADDING
#undef RC_KEY_Y
#undef RC_KEY_X
#undef RC_KEY




static void
init_field(const char* init_command)
{
	char command[NUMBER_OF(G.init)];
	char type[NUMBER_OF(command)];
	char arg[NUMBER_OF(command)];
	int n;
	int x;
	int y;

	SNPRINTF(command, NUMBER_OF(command), "%s", init_command);
	n = sscanf(command, "%[^()](%[^()])", type, arg);

	memset(G.field, STATE_NONE, sizeof(G.field[0]) * (G.x_num * G.y_num));

	if (!(1 <= n)) {
		NOP();
	} else if (!strcmp(type, "none")) {
		NOP();
	} else if (!strcmp(type, "random") && n == 2) {
		int max;
		int rate;

		if (2 != sscanf(arg, "%d/%d", &rate, &max) || max < 0) {
			rate = 10;
			max = 1000;
		}

		for (y = 0; y < G.y_num; y++) {
			for (x = 0; x < G.x_num; x++) {
				if ((rand() % max) + 1 <= rate) {
					G_FIELD(x, y) = STATE_LIFE;
				}
			}
		}
	} else if (!strcmp(type, "file") && n == 2) {
		FILE* f;
		char dir[MAX_PATH];
		char path[MAX_PATH];
		char* t;
		int x_num;
		int y_num;
		int c;

		SNPRINTF(dir, NUMBER_OF(dir), "%s", G.rc_path);
		t = strrchr(dir, '\\');
		if (t != NULL) t[1] = '\0';
		SNPRINTF(path, NUMBER_OF(path), "%s%s", dir, arg);

		f = fopen(path, "rt");
		if (f == NULL) return;

		if (2 != fscanf(f, "%d %d\n", &x_num, &y_num)) goto E_NUM;
		if (G.x_num < x_num || G.y_num < y_num) goto E_NUM;

		x = 0;
		y = 0;
		while ((c = fgetc(f)) != EOF) {
			switch (c) {
			case '#':
				G_FIELD( x + G.x_num/2 - x_num/2,
				         y + G.y_num/2 - y_num/2 ) =STATE_LIFE;
			default:
			case '.':
				x++;
				if (!(x < G.x_num)) x = G.x_num - 1;
				break;
			case '\n':
				x = 0;
				y++;
				if (!(y < G.y_num)) y = G.y_num - 1;
				break;
			}
		}

E_NUM:
		fclose(f);
	} else {
		NOP();
	}

	InvalidateRect(G.window, NULL, TRUE);
}




static void
step(void)
{
	int x;
	int y;

	memset( G.field_tmp, 0,
	        sizeof(G.field_tmp[0]) * ((1+G.x_num+1) * (1+G.y_num+1)) );

	for (y = 0; y < G.y_num; y++) {
		for (x = 0; x < G.x_num; x++) {
			if (G_FIELD(x, y) == STATE_LIFE) {
				G_FIELD_TMP(x - 1, y - 1)++;
				G_FIELD_TMP(x    , y - 1)++;
				G_FIELD_TMP(x + 1, y - 1)++;

				G_FIELD_TMP(x - 1, y    )++;

				G_FIELD_TMP(x + 1, y    )++;

				G_FIELD_TMP(x - 1, y + 1)++;
				G_FIELD_TMP(x    , y + 1)++;
				G_FIELD_TMP(x + 1, y + 1)++;
			}
		}
	}

	for (y = 0-1; y < G.y_num+1; y++) {
		G_FIELD_TMP(G.x_num - 1, y) += G_FIELD_TMP(0 - 1, y);
		G_FIELD_TMP(0, y) += G_FIELD_TMP(G.x_num - 1 + 1, y);
	}

	for (x = 0-1; x < G.x_num+1; x++) {
		G_FIELD_TMP(x, G.y_num - 1) += G_FIELD_TMP(x, 0 - 1);
		G_FIELD_TMP(x, 0) += G_FIELD_TMP(x, G.y_num - 1 + 1);
	}

	for (y = 0; y < G.y_num; y++) {
	  for (x = 0; x < G.x_num; x++) {
	    int neighbours;

	    neighbours = G_FIELD_TMP(x, y);
	    if (!(0 <= neighbours && neighbours <= 9))
	      continue;

	    if (G_FIELD(x, y) == STATE_LIFE) {
	      G_FIELD(x, y) = ( G.survived_number[neighbours]
	                        ? STATE_LIFE : STATE_NONE );
	    } else {  /* STATE_NONE */
	      G_FIELD(x, y) = ( G.born_number[neighbours]
	                        ? STATE_LIFE : STATE_NONE );
	    }
	  }
	}

	InvalidateRect(G.window, NULL, TRUE);
}




static void
interpret_broam(const char* broam)
{
	if (!strcmp(broam, "@BBShowPlugins"))
		{set_visibility(G.window, true);  return;}
	if (!strcmp(broam, "@BBHidePlugins"))
		{set_visibility(G.window, false);  return;}

	if (!!strncmp(broam, "@BBLifeGame.", 12)) return;
	broam += 12;

	if (!strcmp(broam, "reset")) {
		init_field(G.init);
	} else if (!strncmp(broam, "init ", 5)) {
		init_field(broam + 5);
	} else {
		NOP();
	}
}




static LRESULT CALLBACK
window_procedure(HWND hwnd, UINT msg, WPARAM wp, LPARAM lp)
{
	switch (msg) {
	default:
		break;


	case BB_BROADCAST:
		interpret_broam((const char*)lp);
		break;

	case BB_RECONFIGURE:
		init_field(G.init);
		break;


	case WM_CREATE:
		make_always_on_top(hwnd, G.always_on_top);

		MakeSticky(hwnd);
		SendMessage( GetBBWnd(), BB_REGISTERMESSAGE,
		             (WPARAM)hwnd, (LPARAM)BBLG_MSGS );
		SetTimer(hwnd, TIMER_ID_STEP, G.step_interval, NULL);
		break;

	case WM_DESTROY:
		KillTimer(hwnd, TIMER_ID_STEP);
		SendMessage( GetBBWnd(), BB_UNREGISTERMESSAGE,
		             (WPARAM)hwnd, (LPARAM)BBLG_MSGS );
		RemoveSticky(hwnd);
		break;

	case WM_LBUTTONUP:
	case WM_NCLBUTTONUP:
		step();
		break;

	case WM_NCHITTEST:
		return HTCAPTION;

	case WM_WINDOWPOSCHANGING:
		if (1 <= G.snap_to_edge)
			SnapWindowToEdge((WINDOWPOS*)lp, G.snap_to_edge, true);
		break;

	case WM_TIMER:
		switch (wp) {
		default:
			break;

		case TIMER_ID_STEP:
			step();
			break;
		}
		return 0;

	case WM_PAINT: {
		HDC hdc;
		PAINTSTRUCT ps;
		RECT rect;
		StyleItem* base_style;
		StyleItem* life_style;
		int bevel_and_border_width;
		RECT cell;
		int x;
		int y;

		GetClientRect(hwnd, &rect);
		base_style = (StyleItem*)GetSettingPtr(G.base_style);
		life_style = (StyleItem*)GetSettingPtr(G.life_style);
		bevel_and_border_width = *(int*)GetSettingPtr(SN_BEVELWIDTH)
		                         +*(int*)GetSettingPtr(SN_BORDERWIDTH);

		hdc = BeginPaint(hwnd, &ps);

		MakeStyleGradient(hdc, &rect, base_style, true);

		for (y = 0; y < G.y_num; y++) {
		  for (x = 0; x < G.x_num; x++) {
		    if (G_FIELD(x, y) == STATE_LIFE) {
		      cell.left = G.x_padding + x * G.cell_width;
		      cell.top = G.y_padding + y * G.cell_height;
		      cell.right = G.x_padding + (x+1) * G.cell_width;
		      cell.bottom = G.y_padding + (y+1) * G.cell_height;
		      MakeStyleGradient(hdc, &cell, life_style, false);
		    }
		  }
		}

		EndPaint(hwnd, &ps);
		} break;
	}

	return DefWindowProc(hwnd, msg, wp, lp);
}








extern "C" DLL_EXPORT int
beginPlugin(HINSTANCE plugin_instance)
{
	WNDCLASS wc;
	char* t;
	char dir[MAX_PATH];

	GetModuleFileName(plugin_instance, dir, NUMBER_OF(dir));
	dir[NUMBER_OF(dir) - 1] = '\0';
	t = strrchr(dir, '\\');
	if (t != NULL) t[1] = '\0';
	SNPRINTF(G.rc_path, NUMBER_OF(G.rc_path), "%s%s.rc", dir, BBLG_NAME);

	read_settings(G.rc_path);

	G.field = (char*)malloc(sizeof(G.field[0]) * (G.x_num * G.y_num));
	if (G.field == NULL) {error("Out of memory");  goto E_MALLOC;}

	G.field_tmp = (char*)malloc( sizeof(G.field_tmp[0])
	                             * ((1+G.x_num+1) * (1+G.y_num+1)) );
	if (G.field_tmp == NULL) {error("Out of memory");  goto E_MALLOC2;}

	init_field(G.init);

	ZeroMemory(&wc, sizeof(wc));
	wc.lpfnWndProc = window_procedure;
	wc.hInstance = plugin_instance;
	wc.lpszClassName = BBLG_NAME;
	if (!RegisterClass(&wc))
		{error("RegisterClass failed");  goto E_REGISTER_CLASS;}

	G.window = CreateWindowEx( WS_EX_TOOLWINDOW,
	                           BBLG_NAME,
	                           NULL,
	                           WS_POPUP | WS_VISIBLE,
	                           G.x, G.y,
	                           G.cell_width * G.x_num + G.x_padding * 2,
	                           G.cell_height * G.y_num + G.y_padding * 2,
	                           NULL,
	                           NULL,
	                           plugin_instance,
	                           NULL );
	if (G.window == NULL)
		{error("CreateWindowEx failed");  goto E_CREATE_WINDOW_EX;}

	return 0;


E_CREATE_WINDOW_EX:
	UnregisterClass(BBLG_NAME, plugin_instance);
E_REGISTER_CLASS:
	free(G.field_tmp);
E_MALLOC2:
	free(G.field);
E_MALLOC:
	return !0;
}




extern "C" DLL_EXPORT void
endPlugin(HINSTANCE plugin_instance)
{
	RECT rect;

	GetWindowRect(G.window, &rect);

	DestroyWindow(G.window);  G.window = NULL;
	UnregisterClass(BBLG_NAME, plugin_instance);
	free(G.field_tmp);
	free(G.field);

	G.x = rect.left;
	G.y = rect.top;
	write_settings(G.rc_path);
}




extern "C" DLL_EXPORT LPCSTR
pluginInfo(int field)
{
	switch (field) {
	case PLUGIN_NAME:    return BBLG_NAME;
	case PLUGIN_VERSION: return BBLG_VERSION;
	case PLUGIN_AUTHOR:  return BBLG_AUTHOR;
	case PLUGIN_RELEASE: return BBLG_RELEASE;
	case PLUGIN_LINK:    return BBLG_LINK;
	case PLUGIN_EMAIL:   return BBLG_EMAIL;
	default:             return BBLG_NAME;
	}
}




/* __END__ */
