/*
 ============================================================================

  This program is free software, released under the GNU General Public License
  (GPL version 2 or later).

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  for more details.

  http://www.fsf.org/licenses/gpl.html

 ============================================================================
*/

#include "bbmemo.h"

char *plugin_info[] =
{
	"BBMemo 0.0.5",
	"BBMemo",
	"0.0.5",
	"ysuke",
	"2006-02-10",
	"http://zzbb.hp.infoseek.co.jp/",
	"zb2_460@yahoo.co.jp"
};

#define szVersion       plugin_info[0]
#define szAppName       plugin_info[1]
#define szInfoVersion	plugin_info[2]
#define szInfoAuthor	plugin_info[3]
#define szInfoRelDate	plugin_info[4]
#define szInfoLink	plugin_info[5]
#define szInfoEmail	plugin_info[6]


//===========================================================================

int beginSlitPlugin(HINSTANCE hMainInstance, HWND hBBSlit)
{
	inSlit = true;
	hSlit = hBBSlit;

	// Start the plugin like normal now..
	int res = beginPlugin(hMainInstance);

	// Are we to be in the Slit?
	if (inSlit && hSlit){
		// Yes, so Let's let BBSlit know.
		SendMessage(hSlit, SLIT_ADD, NULL, (LPARAM)hwndPlugin);
	}

	return res;
}

//===========================================================================

int beginPluginEx(HINSTANCE hMainInstance, HWND hBBSlit)
{
	return beginSlitPlugin(hMainInstance, hBBSlit);
}

//===========================================================================

int beginPlugin(HINSTANCE hPluginInstance)
{
	WNDCLASS wc;
	hwndBlackbox = GetBBWnd();
	hInstance = hPluginInstance;

	ZeroMemory(&osvinfo, sizeof(osvinfo));
	osvinfo.dwOSVersionInfoSize = sizeof (osvinfo);
	GetVersionEx(&osvinfo);
	usingWin2kXP =
		osvinfo.dwPlatformId == VER_PLATFORM_WIN32_NT
		&& osvinfo.dwMajorVersion > 4;

	// Register the window class...
	ZeroMemory(&wc,sizeof(wc));
	wc.lpfnWndProc = WndProc;			// our window procedure
	wc.hInstance = hPluginInstance;		// hInstance of .dll
	wc.lpszClassName = szAppName;		// our window class name
	if (!RegisterClass(&wc)) {
		MessageBox(hwndBlackbox, "Error registering window class", szAppName, MB_OK | MB_ICONERROR | MB_TOPMOST);
		return 1;
	}

	// Get plugin and style settings...
	ReadRCSettings();
	GetStyleSettings();

	ScreenWidth  = GetSystemMetrics(SM_CXVIRTUALSCREEN);
	ScreenHeight = GetSystemMetrics(SM_CYVIRTUALSCREEN);

	// Create the window...
	hwndPlugin = CreateWindowEx(
			usingWin2kXP ? WS_EX_TOOLWINDOW | WS_EX_LAYERED : WS_EX_TOOLWINDOW, // window style
			szAppName, // our window class name
			NULL, // NULL -> does not show up in task manager!
			WS_POPUP | WS_CLIPCHILDREN | WS_CLIPSIBLINGS, // window parameters
			xpos, // x position
			ypos, // y position
			width, // window width
			height, // window height
			NULL, // parent window
			NULL, // no menu
			hPluginInstance, // hInstance of .dll
			NULL);

	if (!hwndPlugin){						   
		MessageBox(0, "Error creating window", szAppName, MB_OK | MB_ICONERROR | MB_TOPMOST);
		// Unregister Blackbox messages...
		SendMessage(hwndBlackbox, BB_UNREGISTERMESSAGE, (WPARAM)hwndPlugin, (LPARAM)msgs);
		return 1;
	}

	// Register to receive Blackbox messages...
	SendMessage(hwndBlackbox, BB_REGISTERMESSAGE, (WPARAM)hwndPlugin, (LPARAM)msgs);
	// Make the window sticky
	MakeSticky(hwndPlugin);
	// Make the window AlwaysOnTop?
	if (alwaysOnTop) SetWindowPos(hwndPlugin, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOACTIVATE | SWP_NOSIZE | SWP_NOMOVE);

	SetWindowModes();
	is_autohidden = window_test_autohide(NULL);
	SetTimer(hwndPlugin, AUTOHIDE_TIMER, AUTOHIDE_DELAY, NULL);

	if (autoSave) SetTimer(hwndPlugin, AUTOSAVE_TIMER, saveInterval * 1000, NULL);

	// Show the window and force it to update...
	ShowWindow(hwndPlugin, SW_SHOW);
	InvalidateRect(hwndPlugin, NULL, true);
	return 0;
}

//===========================================================================

void endPlugin(HINSTANCE hPluginInstance)
{
	// Write the current plugin settings to the config file...
	WriteRCSettings();
	WriteTextFile();
	// Delete the main plugin menu if it exists (PLEASE NOTE that this takes care of submenus as well!)
	if (myMenu){ DelMenu(myMenu); myMenu = NULL;}

	// Make the window unsticky
	RemoveSticky(hwndPlugin);
	// Unregister Blackbox messages...
	SendMessage(hwndBlackbox, BB_UNREGISTERMESSAGE, (WPARAM)hwndPlugin, (LPARAM)msgs);

	if(inSlit && hSlit)
		SendMessage(hSlit, SLIT_REMOVE, NULL, (LPARAM)hwndPlugin);

  	if(hEditFont) DeleteObject(hEditFont);
  	if(hBrush) DeleteObject(hBrush);
  	KillTimer(hwndPlugin, AUTOSAVE_TIMER);
	// Destroy our window...
	DestroyWindow(hEdit);
	DestroyWindow(hwndPlugin);
	// Unregister window class...
	UnregisterClass(szAppName, hPluginInstance);
}

//===========================================================================

LPCSTR pluginInfo(int field)
{
	switch (field)
	{
		default:
		case 0: return szVersion;
		case 1: return szAppName;
		case 2: return szInfoVersion;
		case 3: return szInfoAuthor;
		case 4: return szInfoRelDate;
		case 5: return szInfoLink;
		case 6: return szInfoEmail;
	}
}

//===========================================================================

LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
		case WM_CREATE:
		{
			RECT rc;
			GetClientRect(hwnd, &rc);

			DWORD dwStyle;
			if(wordWrap) dwStyle = WS_CHILD | WS_VISIBLE | ES_MULTILINE |
						ES_WANTRETURN | ES_AUTOVSCROLL;
			else dwStyle = WS_CHILD | WS_VISIBLE | ES_MULTILINE |
						ES_WANTRETURN | ES_AUTOVSCROLL | ES_AUTOHSCROLL;

			hEdit = CreateWindow(
				"EDIT",
				NULL,
				dwStyle,
				edgeSize, edgeSize,
				rc.right - edgeSize * 2, rc.bottom - edgeSize * 2,
				hwnd,
				NULL,
				hInstance,
				NULL);

			GetStyleSettings();
			if(showScroll) ShowScrollBar(hEdit, wordWrap ? SB_VERT : SB_BOTH, true);
			else ShowScrollBar(hEdit,  wordWrap ? SB_VERT : SB_BOTH, false);

			oldEditProc = (WNDPROC)SetWindowLong(hEdit, GWL_WNDPROC,(long)EditProc);
			ReadTextFile();
		}
		break;

		case WM_PAINT:
		{
			OnPaint(hwnd);
		}
		break;

		// ==========

		case BB_RECONFIGURE:
		{
			if (myMenu){ DelMenu(myMenu); myMenu = NULL;}
			GetStyleSettings();
			InvalidateRect(hwndPlugin, NULL, true);
		}
		break;

		// for bbStylemaker
		case BB_REDRAWGUI:
		{
			GetStyleSettings();
			InvalidateRect(hwndPlugin, NULL, true);
		}
		break;

		case WM_DISPLAYCHANGE:
		{
			if(!inSlit)
			{
				// IntelliMove(tm)... <g>
				// (c) 2003 qwilk
				//should make this a function so it can be used on startup in case resolution changed since
				//the last time blackbox was used.
				int relx, rely;
				int oldscreenwidth = ScreenWidth;
				int oldscreenheight = ScreenHeight;
				ScreenWidth = GetSystemMetrics(SM_CXVIRTUALSCREEN);
				ScreenHeight = GetSystemMetrics(SM_CYVIRTUALSCREEN);
				if (xpos > oldscreenwidth / 2)
				{
					relx = oldscreenwidth - xpos;
					xpos = ScreenWidth - relx;
				}
				if (ypos > oldscreenheight / 2)
				{
					rely = oldscreenheight - ypos;
					ypos = ScreenHeight - rely;
				}
				MoveWindow(hwndPlugin, xpos, ypos, width, height, true);
			}
		}
		break;

		// ==========

		case BB_BROADCAST:
		{
			szTemp = (char*)lParam;

			if (!_stricmp(szTemp, "@BBShowPlugins") &&  pluginToggle && !inSlit)
			{
				// Show window and force update...
				ShowWindow( hwndPlugin, SW_SHOW);
				InvalidateRect( hwndPlugin, NULL, true);
			}
			else if (!_stricmp(szTemp, "@BBHidePlugins") && pluginToggle && !inSlit)
			{
				// Hide window...
				ShowWindow( hwndPlugin, SW_HIDE);
			}

			//===================

			if (strnicmp(szTemp, "@BBMemo", 7))
				return 0;
			szTemp += 7;

			if (!_stricmp(szTemp, "About"))
			{
				char tmp_str[MAX_LINE_LENGTH];
				SendMessage(hwndBlackbox, BB_HIDEMENU, 0, 0);
				sprintf(tmp_str, "%s", szVersion);
				MessageBox(hwndBlackbox, tmp_str, szAppName, MB_OK | MB_TOPMOST);
			}

			//===================

			else if (!_strnicmp(szTemp, "StyleType", 9))
			{
				styleType = atoi(szTemp + 10);
				GetStyleSettings();
				InvalidateRect(hwndPlugin, NULL, true);
			}

			//===================

			else if (!_strnicmp(szTemp, "WidthSize", 9))
			{
				newWidth = atoi(szTemp + 10);
				if(ResizeMyWindow(newWidth, height))
					InvalidateRect(hwndPlugin, NULL, true);
				SendMessage(hwnd, WM_SIZE, wParam, lParam);
			}
			else if (!_strnicmp(szTemp, "HeightSize", 10))
			{
				newHeight = atoi(szTemp + 11);
				if(ResizeMyWindow(width, newHeight))
					InvalidateRect(hwndPlugin, NULL, true);
				SendMessage(hwnd, WM_SIZE, wParam, lParam);
			}
			else if (!_strnicmp(szTemp, "EdgeSize", 8))
			{
				edgeSize = atoi(szTemp + 9);
				SendMessage(hwnd, WM_SIZE, wParam, lParam);
			}
			else if (!_strnicmp(szTemp, "FontSize", 8))
			{
				fontHeight = atoi(szTemp + 9);
				GetStyleSettings();
			}
			else if (!_strnicmp(szTemp, "FontFace", 8))
			{
				strcpy(fontFace, szTemp + 9);
				GetStyleSettings();
			}
			else if (!_strnicmp(szTemp, "SaveInterval", 12))
			{
				saveInterval = atoi(szTemp + 13);
				if(autoSave){
					KillTimer(hwnd, AUTOSAVE_TIMER);
					SetTimer(hwnd, AUTOSAVE_TIMER, saveInterval * 1000, NULL);
				}
			}

			//===================

			else if (!_stricmp(szTemp, "PluginToggle"))
			{
				pluginToggle = (pluginToggle ? false : true);
				SetWindowModes();
			}
			else if (!_stricmp(szTemp, "OnTop"))
			{
				alwaysOnTop = (alwaysOnTop ? false : true);
				if(!inSlit)SetWindowPos( hwndPlugin,
                                                         alwaysOnTop ? HWND_TOPMOST : HWND_NOTOPMOST,
                                                         0, 0, 0, 0,
                                                         SWP_NOACTIVATE | SWP_NOSIZE | SWP_NOMOVE);

				ShowMyMenu(false);
			}
			else if (!_stricmp(szTemp, "InSlit"))
			{
				if(inSlit && hSlit){
					// We are in the slit, so lets unload and get out..
					SendMessage(hSlit, SLIT_REMOVE, NULL, (LPARAM)hwndPlugin);

					// Here you can move to where ever you want ;)
					SetWindowPos(hwndPlugin, NULL, xpos, ypos, 0, 0, SWP_NOZORDER | SWP_NOSIZE);
					inSlit = false;
					SetWindowModes();
					is_autohidden = window_test_autohide(NULL);
					SetTimer(hwnd, AUTOHIDE_TIMER, AUTOHIDE_DELAY, NULL);
				}
				/* Make sure before you try and load into the slit that you have
				* the HWND of the slit ;)
				*/
				else if(hSlit){
					// (Back) into the slit..
					inSlit = true;
					SetWindowModes();
					SendMessage(hSlit, SLIT_ADD, NULL, (LPARAM)hwndPlugin);
				}
			}

			else if (!_stricmp(szTemp, "Transparent"))
			{
				transparency = (transparency ? false : true);
				SetWindowModes();
			}
			else if (!_strnicmp(szTemp, "AlphaValue", 10))
			{
				alpha = atoi(szTemp + 11);
				SetWindowModes();
			}
			else if (!_stricmp(szTemp, "SnapToEdge"))
			{
				snapWindow = (snapWindow ? false : true);
				ShowMyMenu(false);
			}
			else if (!_stricmp(szTemp, "ShowBorder"))
			{
				showBorder = (showBorder ? false : true);
				InvalidateRect(hwndPlugin, NULL, false);
				ShowMyMenu(false);
			}
			else if (!_stricmp(szTemp, "ShowScroll"))
			{
				showScroll = (showScroll ? false : true);

				if(showScroll) ShowScrollBar(hEdit, wordWrap ? SB_VERT : SB_BOTH, true);
				else ShowScrollBar(hEdit,  wordWrap ? SB_VERT : SB_BOTH, false);

				ShowMyMenu(false);
                        }
			else if (!_stricmp(szTemp, "LockPosition"))
			{
				lockPos = (lockPos ? false : true);
				ShowMyMenu(false);
                        }
			else if (!_stricmp(szTemp, "WordWrap"))
			{
				wordWrap = (wordWrap ? false : true);
				SetWordWrap();
                        }
			else if (!_stricmp(szTemp, "AutoHide"))
			{
				autoHide = (autoHide ? false : true);
				is_autohidden = window_test_autohide(NULL);

				bool set_pos = false == inSlit;
				POINT pt = { xpos, ypos };
				bool can_hide = window_test_autohide(&pt);

				SetWindowPos(
                                          hwndPlugin,
                                          can_hide || alwaysOnTop ? HWND_TOPMOST : HWND_NOTOPMOST,
                                          pt.x,
                                          pt.y,
                                          width,
                                          height,
                                          set_pos ? SWP_NOACTIVATE|SWP_NOZORDER|SWP_NOSENDCHANGING
                                                  : SWP_NOACTIVATE|SWP_NOZORDER|SWP_NOMOVE|SWP_NOSENDCHANGING);
				ShowMyMenu(false);
                        }
			else if (!_stricmp(szTemp, "AutoSave"))
			{
				autoSave = (autoSave ? false : true);

				if(autoSave) SetTimer(hwnd, AUTOSAVE_TIMER, saveInterval * 1000, NULL);
				else KillTimer(hwnd, AUTOSAVE_TIMER);

				ShowMyMenu(false);
                        }
			else if (!_stricmp(szTemp, "FontColor"))
			{
				CHOOSECOLOR cc;
				COLORREF custCol[16];
				ZeroMemory(&cc, sizeof(CHOOSECOLOR));
				cc.lStructSize = sizeof(CHOOSECOLOR);
				cc.hwndOwner = NULL;
				cc.lpCustColors = (LPDWORD) custCol;
				cc.rgbResult = fontColor;
				cc.Flags = CC_RGBINIT;

				if (ChooseColor(&cc)){
					    fontColor = cc.rgbResult;
					    InvalidateRect(hEdit, NULL, true);
				}
			}
			else if (!_stricmp(szTemp, "BackColor"))
			{
				CHOOSECOLOR cc;
				COLORREF custCol[16];
				ZeroMemory(&cc, sizeof(CHOOSECOLOR));
				cc.lStructSize = sizeof(CHOOSECOLOR);
				cc.hwndOwner = NULL;
				cc.lpCustColors = (LPDWORD) custCol;
				cc.rgbResult = backColor;
				cc.Flags = CC_RGBINIT;

				if (ChooseColor(&cc)){
					    backColor = cc.rgbResult;
					    GetStyleSettings();
					    InvalidateRect(hEdit, NULL, true);
				}
			}
			else if (!_stricmp(szTemp, "SetFont"))
			{
				LOGFONT lf;
				GetObject(hEditFont, sizeof(lf), &lf);

				CHOOSEFONT cf;
				memset(&cf, 0, sizeof(CHOOSEFONT));
				cf.lStructSize = sizeof(CHOOSEFONT);
				cf.hwndOwner = hwndPlugin;
				cf.lpLogFont = &lf;
				cf.nFontType = SCREEN_FONTTYPE;
				cf.Flags = CF_SCREENFONTS | CF_INITTOLOGFONTSTRUCT;

				if (ChooseFont(&cf)){
					    strcpy(fontFace, lf.lfFaceName);
					    fontHeight = -lf.lfHeight;
					    GetStyleSettings();
				}
			}


			//===================

			else if (!_stricmp(szTemp, "EditRC"))
			{
				BBExecute(GetDesktopWindow(), NULL, rcpath, NULL, NULL, SW_SHOWNORMAL, false);
			}
			else if (!_stricmp(szTemp, "ReloadSettings"))
			{
				ReadRCSettings();
				ReadTextFile();
				GetStyleSettings();

				if(inSlit && hSlit)
				{
					SetWindowModes();
					SendMessage(hSlit, SLIT_UPDATE, NULL, NULL);
				}
				else if(!inSlit || !hSlit)
					SendMessage(hSlit, SLIT_REMOVE, NULL, (LPARAM)hwndPlugin);

				else inSlit = false;
			}
			else if (!_stricmp(szTemp, "SaveSettings"))
			{
				WriteRCSettings();
				WriteTextFile();
			}
			else if (!_stricmp(szTemp, "SetFocus"))
			{
				SetWindowPos(hwndPlugin, HWND_TOP, 0, 0, 0, 0,
                                              SWP_NOMOVE | SWP_NOSIZE | SWP_SHOWWINDOW);
				SetForegroundWindow(hwndPlugin);
				SetFocus(hEdit);
			}
		}
		break;

		// ==========

		case WM_WINDOWPOSCHANGING:
		{
			// Is SnapWindowToEdge enabled?
			if (!inSlit && snapWindow)
			{
				// Snap window to screen edges (if the last bool is false it uses the current DesktopArea)
				if(IsWindowVisible(hwnd)) SnapWindowToEdge((WINDOWPOS*)lParam, 10, true);
			}
		}
		break;

		// ==========

		// Save window position if it changes...
		case WM_WINDOWPOSCHANGED:
		{
			if(!inSlit && !is_autohidden)
			{
				WINDOWPOS* windowpos = (WINDOWPOS*)lParam;
				xpos = windowpos->x;
				ypos = windowpos->y;

				if(ResizeMyWindow(windowpos->cx, windowpos->cy))
				{
					InvalidateRect(hwndPlugin, NULL, true);
					SendMessage(hwnd, WM_SIZE, wParam, lParam);
				}
			}
		}
		break;

		// ==========

		case WM_NCHITTEST:
		{
			if (GetKeyState(VK_MENU) & 0xF0 && !lockPos)
				return HTBOTTOMRIGHT;
			else
				return HTCAPTION;
		}
		break;

		case WM_EXITSIZEMOVE:
			is_autohidden = window_test_autohide(NULL);
			SetTimer(hwnd, AUTOHIDE_TIMER, AUTOHIDE_DELAY, NULL);
			break;

		case WM_NCMOUSEMOVE:
		{
			if (is_autohidden)
			{
				is_autohidden = false;
				if (false == inSlit)
				{
					bool set_pos = false == inSlit;
					POINT pt = { xpos, ypos };
					bool can_hide = window_test_autohide(&pt);

					SetWindowPos(
                                          hwndPlugin,
                                          can_hide || alwaysOnTop ? HWND_TOPMOST : HWND_NOTOPMOST,
                                          pt.x,
                                          pt.y,
                                          width,
                                          height,
                                          set_pos ? SWP_NOACTIVATE|SWP_NOZORDER|SWP_NOSENDCHANGING
                                                  : SWP_NOACTIVATE|SWP_NOZORDER|SWP_NOMOVE|SWP_NOSENDCHANGING);

					SetTimer(hwnd, AUTOHIDE_TIMER, AUTOHIDE_DELAY, NULL);
				}
			}
		}
		break;

		case WM_TIMER:
		{
			if (AUTOHIDE_TIMER == wParam)
			{
				bool hide_it = window_test_autohide(NULL);
				if (hide_it)
				{
					POINT pt; RECT rct;
					GetCursorPos(&pt);
					GetWindowRect(hwnd, &rct);
					if (PtInRect(&rct, pt) || (0x8000 & GetKeyState(VK_LBUTTON)))
						break;
				}
				is_autohidden = hide_it;
				KillTimer(hwnd, wParam);

				bool set_pos = false == inSlit;
				POINT pt = { xpos, ypos };
				bool can_hide = window_test_autohide(&pt);

				SetWindowPos(
                                          hwndPlugin,
                                          can_hide || alwaysOnTop ? HWND_TOPMOST : HWND_NOTOPMOST,
                                          pt.x,
                                          pt.y,
                                          width,
                                          height,
                                          set_pos ? SWP_NOACTIVATE|SWP_NOZORDER|SWP_NOSENDCHANGING
                                                  : SWP_NOACTIVATE|SWP_NOZORDER|SWP_NOMOVE|SWP_NOSENDCHANGING);
				break;
			}
			if (AUTOSAVE_TIMER == wParam)
			{
				WriteTextFile();
				break;
			}
		}
		break;

		case WM_SIZE:
		{
			RECT rc;
			GetClientRect(hwnd, &rc);
			MoveWindow(hEdit, rc.left + edgeSize, rc.top + edgeSize,
				rc.right - edgeSize * 2, rc.bottom - edgeSize * 2, true);
		}
		break;

		case WM_CTLCOLOREDIT:
		{
			SetBkMode((HDC)wParam, OPAQUE);
			SetTextColor((HDC)wParam, fontColor);
			SetBkColor((HDC)wParam, backColor);
			return (LRESULT)hBrush;
		}
		break;

		case WM_NCLBUTTONDOWN:
		{
			SetFocus(hEdit);
			if(!inSlit && !lockPos)
				return DefWindowProc(hwnd,message,wParam,lParam);
		}
		break;

		// ==========

		case WM_NCRBUTTONUP:
		{
			ShowMyMenu(true);
		}
		break;

		case WM_NCRBUTTONDOWN: {} break;


		case WM_CLOSE:
		{
			return 0;
		}
		break;

 		// ==========
		default:
			return DefWindowProc(hwnd,message,wParam,lParam);
	}
	return 0;
}

//===========================================================================

LRESULT CALLBACK EditProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch(message)
	{
		case WM_NCHITTEST:
		{
			if (GetKeyState(VK_MENU) & 0xF0 && !lockPos)
				return HTBOTTOMRIGHT;
			else if (GetKeyState(VK_CONTROL) & 0xF0)
				return HTCAPTION;
		}
		break;

		case WM_NCLBUTTONDOWN:
		{
			if (GetKeyState(VK_MENU) & 0xF0 || GetKeyState(VK_CONTROL) & 0xF0){
				if(!inSlit && !lockPos)
					return DefWindowProc(hwndPlugin,message,wParam,lParam);
				else return 0;
			}
		}
		break;

		case WM_LBUTTONDOWN:
			SetFocus(hwnd);
			break;

		case WM_NCRBUTTONUP:
			ShowMyMenu(true);
			break;

		case WM_SETFOCUS:
		{
			is_focus = true;
			if (is_autohidden)
			{
				is_autohidden = false;
				if (false == inSlit)
				{
					bool set_pos = false == inSlit;
					POINT pt = { xpos, ypos };
					bool can_hide = window_test_autohide(&pt);

					SetWindowPos(
                                          hwndPlugin,
                                          can_hide || alwaysOnTop ? HWND_TOPMOST : HWND_NOTOPMOST,
                                          pt.x,
                                          pt.y,
                                          width,
                                          height,
                                          set_pos ? SWP_NOACTIVATE|SWP_NOZORDER|SWP_NOSENDCHANGING
                                                  : SWP_NOACTIVATE|SWP_NOZORDER|SWP_NOMOVE|SWP_NOSENDCHANGING);
				}
			}
		}
		break;

		case WM_KILLFOCUS:
		{
			is_focus = false;
			is_autohidden = true;
			SetTimer(hwndPlugin, AUTOHIDE_TIMER, AUTOHIDE_DELAY, NULL);
		}
		break;

		case WM_CLOSE:
		{
			return 0;
		}
		break;

	}
	return CallWindowProc(oldEditProc,hwnd,message,wParam,lParam);
}

//===========================================================================

void ShowMyMenu(bool popup)
{
	if (myMenu){ DelMenu(myMenu); myMenu = NULL;}

	configSubmenu = MakeNamedMenu("Configuration", "BBM_Config", popup);
	MakeMenuItemInt(configSubmenu, "Width Size", "@BBMemoWidthSize", width, 10, ScreenWidth);
	MakeMenuItemInt(configSubmenu, "Height Size", "@BBMemoHeightSize", height, 10, ScreenHeight);
	MakeMenuItemInt(configSubmenu, "Edge Size", "@BBMemoEdgeSize", edgeSize, 1, 10);
	MakeMenuNOP(configSubmenu, NULL);
	if(hSlit) MakeMenuItem(configSubmenu, "In Slit", "@BBMemoInSlit", inSlit);
	MakeMenuItem(configSubmenu, "Plugin Toggle", "@BBMemoPluginToggle", pluginToggle);
	MakeMenuItem(configSubmenu, "Always on Top", "@BBMemoOnTop", alwaysOnTop);
	MakeMenuItem(configSubmenu, "Transparency", "@BBMemoTransparent", transparency);
	MakeMenuItemInt(configSubmenu, "Alpha Value", "@BBMemoAlphaValue", alpha, 0, 255);
	MakeMenuItem(configSubmenu, "Snap To Edge", "@BBMemoSnapToEdge", snapWindow);
	MakeMenuItem(configSubmenu, "Show Border", "@BBMemoShowBorder", showBorder);
	MakeMenuItem(configSubmenu, "Lock Position", "@BBMemoLockPosition", lockPos);
	MakeMenuItem(configSubmenu, "Auto Hide", "@BBMemoAutoHide", autoHide);
	MakeMenuNOP(configSubmenu, NULL);
	MakeMenuItem(configSubmenu, "Show Scroll", "@BBMemoShowScroll", showScroll);
	MakeMenuItem(configSubmenu, "Auto Save", "@BBMemoAutoSave", autoSave);
	MakeMenuItemInt(configSubmenu, "Save Interval (sec)", "@BBMemoSaveInterval", saveInterval, 1, 3600);

	formatSubmenu = MakeNamedMenu("Format", "BBM_Format", popup);
	MakeMenuItem(formatSubmenu, "Word Wrap",  "@BBMemoWordWrap", wordWrap);
	fontSubmenu = MakeNamedMenu("Font", "BBM_Font", popup);
	MakeMenuItemInt(fontSubmenu, "Font Size", "@BBMemoFontSize", fontHeight, 1, 64);
	MakeMenuItemString(fontSubmenu, "Font Face", "@BBMemoFontFace", fontFace);
	MakeMenuItem(fontSubmenu, "Font Color",  "@BBMemoFontColor", false);
	MakeMenuItem(fontSubmenu, "Browse...",  "@BBMemoSetFont", false);
	MakeSubmenu(formatSubmenu, fontSubmenu, "Font");
	MakeMenuItem(formatSubmenu, "Back Color",  "@BBMemoBackColor", false);

	styleSubmenu = MakeNamedMenu("Style Type", "BBM_StyleType", popup);
	MakeMenuItem(styleSubmenu, "Toolbar",  "@BBMemoStyleType 1", (styleType == 1));
	MakeMenuItem(styleSubmenu, "Button",   "@BBMemoStyleType 2", (styleType == 2));
	MakeMenuItem(styleSubmenu, "ButtonP",  "@BBMemoStyleType 3", (styleType == 3));
	MakeMenuItem(styleSubmenu, "Label",    "@BBMemoStyleType 4", (styleType == 4));
	MakeMenuItem(styleSubmenu, "WinLabel", "@BBMemoStyleType 5", (styleType == 5));
	MakeMenuItem(styleSubmenu, "Clock",    "@BBMemoStyleType 6", (styleType == 6));

	settingsSubmenu = MakeNamedMenu("Settings", "BBM_Settings", popup);
	MakeMenuItem(settingsSubmenu, "Edit Settings",   "@BBMemoEditRC", false);
	MakeMenuItem(settingsSubmenu, "Reload Settings", "@BBMemoReloadSettings", false);
	MakeMenuItem(settingsSubmenu, "Save Settings",   "@BBMemoSaveSettings", false);

	myMenu = MakeNamedMenu("BBMemo", "BBM_Main", popup);
	MakeSubmenu(myMenu, configSubmenu, "Configuration");
	MakeSubmenu(myMenu, formatSubmenu, "Format");
	MakeSubmenu(myMenu, styleSubmenu, "Style Type");
	MakeSubmenu(myMenu, settingsSubmenu, "Settings");
  	MakeMenuItem(myMenu, "About", "@BBMemoAbout", false);
	ShowMenu(myMenu);
}

//===========================================================================

void GetStyleSettings()
{
	if(hEditFont != NULL)
		DeleteObject(hEditFont);

	hEditFont = CreateFont(fontHeight,
                               0, 0, 0, FW_NORMAL,
                               false, false, false,
                               DEFAULT_CHARSET,
                               OUT_DEFAULT_PRECIS,
                               CLIP_DEFAULT_PRECIS,
                               DEFAULT_QUALITY,
                               DEFAULT_PITCH, fontFace);

	SendMessage(hEdit, WM_SETFONT, (WPARAM)hEditFont, true);

	if(hBrush != NULL)
		DeleteObject(hBrush);

	hBrush = CreateSolidBrush(backColor);

	bevelWidth  = *(int*)GetSettingPtr(SN_BEVELWIDTH);
	borderWidth = *(int*)GetSettingPtr(SN_BORDERWIDTH);
	borderColor = *(COLORREF*)GetSettingPtr(SN_BORDERCOLOR);

	myStyleItem = *(StyleItem*)GetSettingPtr(styleType);

	ShowMyMenu(false);
}

//===========================================================================

void ReadRCSettings()
{
	char temp[MAX_LINE_LENGTH], path[MAX_LINE_LENGTH], defaultpath[MAX_LINE_LENGTH];
	int nLen;

	// First we look for the config file in the same folder as the plugin...
	GetModuleFileName(hInstance, rcpath, sizeof(rcpath));
	nLen = strlen(rcpath) - 1;
	while (nLen >0 && rcpath[nLen] != '\\') nLen--;
	rcpath[nLen + 1] = 0;
	strcpy(temp, rcpath);
	strcpy(path, rcpath);
	strcat(temp, "bbmemo.rc");
	strcat(path, "bbmemorc");
	// ...checking the two possible filenames bbmemo.rc and bbmemorc ...
	if (FileExists(temp)) strcpy(rcpath, temp);
	else if (FileExists(path)) strcpy(rcpath, path);
	// ...if not found, we try the Blackbox directory...
	else
	{
		// ...but first we save the default path (bbmemo.rc in the same
		// folder as the plugin) just in case we need it later (see below)...
		strcpy(defaultpath, temp);
		GetBlackboxPath(rcpath, sizeof(rcpath));
		strcpy(temp, rcpath);
		strcpy(path, rcpath);
		strcat(temp, "bbmemo.rc");
		strcat(path, "bbmemorc");
		if (FileExists(temp)) strcpy(rcpath, temp);
		else if (FileExists(path)) strcpy(rcpath, path);
		else // If no config file was found, we use the default path and settings, and return
		{
			strcpy(rcpath, defaultpath);
			xpos = ypos = 10;
			width = 120;
			height = 80;
			edgeSize = 4;
			fontHeight = 12;
			alpha = 160;
			styleType = 1;
			saveInterval = 120;
			alwaysOnTop = false;
			transparency = false;
			snapWindow = true;
			pluginToggle = false;
			inSlit = false;
			showBorder = true;
			showScroll = false;
			lockPos = false;
			wordWrap = false;
			autoHide = false;
			autoSave = false;
			strcpy(fontFace, "Tahoma");
			WriteRCSettings(); //write a file for editing later
			return;
		}
	}

	// If a config file was found we read the plugin settings from the file...
	xpos = ReadInt(rcpath, "bbmemo.x:", 10);
	ypos = ReadInt(rcpath, "bbmemo.y:", 10);
	if (xpos >= GetSystemMetrics(SM_CXVIRTUALSCREEN)) xpos = 10;
	if (ypos >= GetSystemMetrics(SM_CYVIRTUALSCREEN)) ypos = 10;
	width = ReadInt(rcpath, "bbmemo.width:", 120);
  	height = ReadInt(rcpath, "bbmemo.height:", 80);
  	edgeSize = ReadInt(rcpath, "bbmemo.edgeSize:", 4);
	fontHeight = ReadInt(rcpath, "bbmemo.fontHeight:", 12);
	alpha = ReadInt(rcpath, "bbmemo.alpha:", 160);
	styleType = ReadInt(rcpath, "bbmemo.styleType:", 1);
	saveInterval = ReadInt(rcpath, "bbmemo.saveInterval(sec):", 120);
	alwaysOnTop = ReadBool(rcpath, "bbmemo.alwaysontop:", false);
	transparency = ReadBool(rcpath, "bbmemo.transparency:", false);
	snapWindow = ReadBool(rcpath, "bbmemo.snapwindow:", true);
	pluginToggle = ReadBool(rcpath, "bbmemo.pluginToggle:", false);
	inSlit = ReadBool(rcpath, "bbmemo.inSlit:", false);
	showBorder = ReadBool(rcpath, "bbmemo.showBorder:", true);
	showScroll = ReadBool(rcpath, "bbmemo.showScroll:", false);
	lockPos = ReadBool(rcpath, "bbmemo.lockPos:", false);
	wordWrap = ReadBool(rcpath, "bbmemo.wordWrap:", false);
	autoHide = ReadBool(rcpath, "bbmemo.autoHide:", false);
	autoSave = ReadBool(rcpath, "bbmemo.autoSave:", false);
	fontColor = ReadColor(rcpath, "bbmemo.fontColor:", "#000000");
	backColor = ReadColor(rcpath, "bbmemo.backColor:", "#FFFFFF");
	strcpy(fontFace, ReadString(rcpath, "bbmemo.font:", "Tahoma"));
}

//===========================================================================

void WriteRCSettings()
{
	WriteInt(rcpath, "bbmemo.x:", xpos);
	WriteInt(rcpath, "bbmemo.y:", ypos);
	WriteInt(rcpath, "bbmemo.width:", width);
	WriteInt(rcpath, "bbmemo.height:", height);
	WriteInt(rcpath, "bbmemo.edgeSize:", edgeSize);
	WriteInt(rcpath, "bbmemo.fontHeight:", fontHeight);
	WriteInt(rcpath, "bbmemo.alpha:", alpha);
	WriteInt(rcpath, "bbmemo.styleType:", styleType);
	WriteInt(rcpath, "bbmemo.saveInterval(sec):", saveInterval);
	WriteBool(rcpath, "bbmemo.alwaysontop:", alwaysOnTop);
	WriteBool(rcpath, "bbmemo.transparency:", transparency);
	WriteBool(rcpath, "bbmemo.snapWindow:", snapWindow);
	WriteBool(rcpath, "bbmemo.pluginToggle:", pluginToggle);
	WriteBool(rcpath, "bbmemo.inSlit:", inSlit);
	WriteBool(rcpath, "bbmemo.showBorder:", showBorder);
	WriteBool(rcpath, "bbmemo.showScroll:", showScroll);
	WriteBool(rcpath, "bbmemo.lockPos:", lockPos);
	WriteBool(rcpath, "bbmemo.wordWrap:", wordWrap);
	WriteBool(rcpath, "bbmemo.autoHide:", autoHide);
	WriteBool(rcpath, "bbmemo.autoSave:", autoSave);
	WriteColor(rcpath, "bbmemo.fontColor:", fontColor);
	WriteColor(rcpath, "bbmemo.backColor:", backColor);
	WriteString(rcpath, "bbmemo.font:", fontFace);
}

//===========================================================================

void ReadTextFile()
{
	char path[MAX_LINE_LENGTH];
	HANDLE hFile;
	DWORD dwBytes;
	GetModuleFileName(hInstance, path, sizeof(path));
	int nLen = strlen(path) - 1;
	while (nLen >0 && path[nLen] != '\\') nLen--;
	path[nLen + 1] = 0;
	sprintf(textPath, "%s%s", path, "bbmemo.txt");

	hFile = CreateFile(textPath, GENERIC_READ, 0, 0, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
	SetFilePointer(hFile, 0, 0, FILE_BEGIN);
	ReadFile(hFile, szStr, GetFileSize(hFile , NULL), &dwBytes, NULL);
	szStr[dwBytes] = 0;
	Edit_SetText(hEdit, szStr);
	CloseHandle(hFile);
}

//===========================================================================

void WriteTextFile()
{
	HANDLE hFile;
	DWORD dwBytes;
	memset(szStr, '\0', sizeof(szStr));
	int nLen = GetWindowTextLength(hEdit);
	GetWindowText(hEdit, szStr, nLen + 1);

	hFile = CreateFile(textPath, GENERIC_WRITE, 0, 0, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
	WriteFile(hFile, szStr, strlen(szStr) + 1, &dwBytes, NULL);
	CloseHandle(hFile);
}

//===========================================================================

void OnPaint(HWND hwnd)
{
	// Create buffer hdc's, bitmaps etc.
	PAINTSTRUCT ps;  RECT r;

	//get screen buffer
	HDC hdc_scrn = BeginPaint(hwnd, &ps);

	//get window rectangle.
	GetClientRect(hwnd, &r);

	//first get a new 'device context'
	HDC hdc = CreateCompatibleDC(NULL);

	//then create a buffer in memory with the window size
	HBITMAP bufbmp = CreateCompatibleBitmap(hdc_scrn, r.right, r.bottom);

	//select the bitmap into the DC
	HGDIOBJ otherbmp = SelectObject(hdc, bufbmp);


	MakeStyleGradient(hdc, &r, &myStyleItem, showBorder);

	// Finally, copy from the paint buffer to the window...
	BitBlt(hdc_scrn, 0, 0, width, height, hdc, 0, 0, SRCCOPY);

	// Remember to delete all objects!
	DeleteObject(SelectObject(hdc, otherbmp));
	DeleteDC(hdc);

	EndPaint(hwnd, &ps);
}

//=========================================================================

//so there you just use BBSLWA like normal SLWA
//(c)grischka
BOOL WINAPI BBSetLayeredWindowAttributes(HWND hwnd, COLORREF crKey, BYTE bAlpha, DWORD dwFlags)
{
	static BOOL (WINAPI *pSLWA)(HWND, COLORREF, BYTE, DWORD);
	static int f=0;
	for (;;) {
		if (2==f)   return pSLWA(hwnd, crKey, bAlpha, dwFlags);
		// if it's not there, just do nothing and report success
		if (f)      return TRUE;
		*(FARPROC*)&pSLWA = GetProcAddress(GetModuleHandle("USER32"), "SetLayeredWindowAttributes");
		f = pSLWA ? 2 : 1;
	}
}

//=========================================================================

void SetWindowModes(void)
{
	if(usingWin2kXP){
		if(!inSlit){
			// Add the WS_EX_LAYERED atribute to the window.
			SetWindowLong(hwndPlugin, GWL_EXSTYLE , GetWindowLong(hwndPlugin, GWL_EXSTYLE) | WS_EX_LAYERED);
			// Make it transparent...
			BBSetLayeredWindowAttributes(hwndPlugin, NULL, (transparency) ? (unsigned char)alpha : 255, LWA_ALPHA);
		}else{
			// Remove the WS_EX_LAYERED atribute to the window, cos we in di slit!
			SetWindowLong(hwndPlugin, GWL_EXSTYLE, GetWindowLong(hwndPlugin, GWL_EXSTYLE) & ~WS_EX_LAYERED);
		}
	}
	InvalidateRect(hwndPlugin, NULL, false);
	ShowMyMenu(false);
}

//===========================================================================

bool ResizeMyWindow(int newWidth, int newHeight)
{
	if(newWidth != width || newHeight != height){
		width = newWidth;
		height = newHeight;
		if (width <= 10)  width  = 10;
		if (height <= 10) height = 10;
	}
	else
		return false;

	if(inSlit){
		SetWindowPos(hwndPlugin, 0, 0, 0, width, height, SWP_NOACTIVATE | SWP_NOZORDER | SWP_NOMOVE);
		SendMessage(hSlit, SLIT_UPDATE, NULL, NULL);
		return true;
	}else{
		SetWindowPos(hwndPlugin, 0, xpos, ypos, width, height, SWP_NOACTIVATE | SWP_NOZORDER);
		return true;
	}
}

//===========================================================================

void SetWordWrap()
{
	memset(szStr, '\0', sizeof(szStr));
	int nLen = GetWindowTextLength(hEdit);
	GetWindowText(hEdit, szStr, nLen + 1);

	DestroyWindow(hEdit);

	RECT rc;
	GetClientRect(hwndPlugin, &rc);

	DWORD dwStyle;
	if(wordWrap) dwStyle = WS_CHILD | WS_VISIBLE | ES_MULTILINE |
			ES_WANTRETURN | ES_AUTOVSCROLL;
	else dwStyle = WS_CHILD | WS_VISIBLE | ES_MULTILINE |
			ES_WANTRETURN | ES_AUTOVSCROLL | ES_AUTOHSCROLL;

	hEdit = CreateWindow(
			"EDIT",
			NULL,
			dwStyle,
			edgeSize, edgeSize,
			rc.right - edgeSize * 2, rc.bottom - edgeSize * 2,
			hwndPlugin,
			NULL,
			hInstance,
			NULL);

	if(showScroll) ShowScrollBar(hEdit, wordWrap ? SB_VERT : SB_BOTH, true);
	else ShowScrollBar(hEdit,  wordWrap ? SB_VERT : SB_BOTH, false);

	oldEditProc = (WNDPROC)SetWindowLong(hEdit, GWL_WNDPROC,(long)EditProc);

	SendMessage(hEdit, WM_SETFONT, (WPARAM)hEditFont, 0);
	SetWindowText(hEdit,szStr);

	SetFocus(hEdit);
	ShowMyMenu(false);
}

//===========================================================================
//get_monitor - multimon api, win 9x compatible

static HMONITOR (WINAPI *pMonitorFromWindow)(HWND hwnd, DWORD dwFlags);
static BOOL     (WINAPI *pGetMonitorInfoA)(HMONITOR hMonitor, LPMONITORINFO lpmi);

HMONITOR get_monitor(HWND hw)
{
	if (NULL == pMonitorFromWindow)
	{
		HMODULE hDll = GetModuleHandle("USER32");
		*(FARPROC*)&pMonitorFromWindow = GetProcAddress(hDll, "MonitorFromWindow" );
		*(FARPROC*)&pGetMonitorInfoA   = GetProcAddress(hDll, "GetMonitorInfoA"   );
		if (NULL == pMonitorFromWindow)
			*(int*)&pMonitorFromWindow = 1;
	}

	if (*(DWORD*)&pMonitorFromWindow <= 1)
		return NULL;

	return pMonitorFromWindow(hw, MONITOR_DEFAULTTOPRIMARY);
}

//===========================================================================
//get_mon_rect

void get_mon_rect(HMONITOR hMon, RECT *r)
{
	if (hMon)
	{
		MONITORINFO mi;
		mi.cbSize = sizeof(mi);
		if (pGetMonitorInfoA(hMon, &mi))
		{
			*r = mi.rcMonitor;
			//*r = mi.rcWork;
			return;
		}
	}
	//SystemParametersInfo(SPI_GETWORKAREA, 0, r, 0);
	r->top = r->left = 0;
	r->right = GetSystemMetrics(SM_CXSCREEN);
	r->bottom = GetSystemMetrics(SM_CYSCREEN);
}

//===========================================================================

bool window_test_autohide(POINT *pt)
{
	if (false == autoHide || inSlit || is_focus)
		return false;

	int x = xpos;
	int y = ypos;

	RECT scrn;
	get_mon_rect(get_monitor(hwndPlugin), &scrn);

	if (x == scrn.left) x = 1-width;
	else
	if (x == scrn.right - width) x = scrn.right - 1;
	else
	if (y == scrn.top) y = 1 - height;
	else
	if (y == scrn.bottom - height) y = scrn.bottom - 1;
	else
		return false;

	if (pt && is_autohidden)
		pt->x = x, pt->y = y;

	return true;
}

//===========================================================================