/*
 ============================================================================

  This program is free software, released under the GNU General Public License
  (GPL version 2 or later).

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  for more details.

  http://www.fsf.org/licenses/gpl.html

 ============================================================================
*/

#include "BBRssReader.h"

char *plugin_info[] =
{
	"BBRssReader 0.0.1a",
	"BBRssReader",
	"0.0.1a",
	"ysuke",
	"2006-02-18",
	"http://zzbb.hp.infoseek.co.jp/",
	"zb2_460@yahoo.co.jp"
};

#define szVersion       plugin_info[0]
#define szAppName       plugin_info[1]
#define szInfoVersion	plugin_info[2]
#define szInfoAuthor	plugin_info[3]
#define szInfoRelDate	plugin_info[4]
#define szInfoLink	plugin_info[5]
#define szInfoEmail	plugin_info[6]


//===========================================================================

int beginSlitPlugin(HINSTANCE hMainInstance, HWND hBBSlit)
{
	inSlit = true;
	hSlit = hBBSlit;

	// Start the plugin like normal now..
	int res = beginPlugin(hMainInstance);

	// Are we to be in the Slit?
	if (inSlit && hSlit){
		// Yes, so Let's let BBSlit know.
		SendMessage(hSlit, SLIT_ADD, NULL, (LPARAM)hwndPlugin);
	}

	return res;
}

//===========================================================================

int beginPluginEx(HINSTANCE hMainInstance, HWND hBBSlit)
{
	return beginSlitPlugin(hMainInstance, hBBSlit);
}

//===========================================================================

int beginPlugin(HINSTANCE hPluginInstance)
{
	WNDCLASS wc;
	hwndBlackbox = GetBBWnd();
	hInstance = hPluginInstance;

	ZeroMemory(&osvinfo, sizeof(osvinfo));
	osvinfo.dwOSVersionInfoSize = sizeof (osvinfo);
	GetVersionEx(&osvinfo);
	usingWin2kXP =
		osvinfo.dwPlatformId == VER_PLATFORM_WIN32_NT
		&& osvinfo.dwMajorVersion > 4;

	// Register the window class...
	ZeroMemory(&wc,sizeof(wc));
	wc.lpfnWndProc = WndProc;			// our window procedure
	wc.hInstance = hPluginInstance;		// hInstance of .dll
	wc.lpszClassName = szAppName;		// our window class name
	if (!RegisterClass(&wc)) {
		MessageBox(hwndBlackbox, "Error registering window class", szAppName, MB_OK | MB_ICONERROR | MB_TOPMOST);
		return 1;
	}

	// Get plugin and style settings...
	ReadRCSettings();
	GetStyleSettings();

	ScreenWidth  = GetSystemMetrics(SM_CXVIRTUALSCREEN);
	ScreenHeight = GetSystemMetrics(SM_CYVIRTUALSCREEN);

	// Create the window...
	hwndPlugin = CreateWindowEx(
			usingWin2kXP ? WS_EX_TOOLWINDOW | WS_EX_LAYERED : WS_EX_TOOLWINDOW, // window style
			szAppName, // our window class name
			NULL, // NULL -> does not show up in task manager!
			WS_POPUP | WS_CLIPCHILDREN | WS_CLIPSIBLINGS, // window parameters
			xpos, // x position
			ypos, // y position
			width, // window width
			height, // window height
			NULL, // parent window
			NULL, // no menu
			hPluginInstance, // hInstance of .dll
			NULL);

	if (!hwndPlugin){						   
		MessageBox(0, "Error creating window", szAppName, MB_OK | MB_ICONERROR | MB_TOPMOST);
		// Unregister Blackbox messages...
		SendMessage(hwndBlackbox, BB_UNREGISTERMESSAGE, (WPARAM)hwndPlugin, (LPARAM)msgs);
		return 1;
	}

	// Register to receive Blackbox messages...
	SendMessage(hwndBlackbox, BB_REGISTERMESSAGE, (WPARAM)hwndPlugin, (LPARAM)msgs);
	// Make the window sticky
	MakeSticky(hwndPlugin);
	// Make the window AlwaysOnTop?
	if (alwaysOnTop) SetWindowPos(hwndPlugin, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOACTIVATE | SWP_NOSIZE | SWP_NOMOVE);

	SetWindowModes();

	GetRssText();
	SetTimer(hwndPlugin, GETTEXT_TIMER, updateInterval * 60000, NULL); // Start timer.

	is_autohidden = window_test_autohide(NULL);
	SetTimer(hwndPlugin, AUTOHIDE_TIMER, AUTOHIDE_DELAY, NULL);

	// Show the window and force it to update...
	ShowWindow(hwndPlugin, SW_SHOW);
	InvalidateRect(hwndPlugin, NULL, false);

  	//====================

	INITCOMMONCONTROLSEX ic;
	ic.dwSize = sizeof(INITCOMMONCONTROLSEX);
	ic.dwICC = ICC_BAR_CLASSES;

        if (InitCommonControlsEx(&ic))
        { // Load "tab" controls, including tooltips.
	        hToolTips = CreateWindowEx(
		        WS_EX_TOPMOST,
		        TOOLTIPS_CLASS, // "tooltips_class32"
		        NULL, //"BBSBTT",
		        WS_POPUP | TTS_ALWAYSTIP | TTS_NOPREFIX,
		        CW_USEDEFAULT,
		        CW_USEDEFAULT,
		        CW_USEDEFAULT,
		        CW_USEDEFAULT,
		        NULL,
		        NULL,
		        hPluginInstance,
		        NULL);

	        SendMessage(hToolTips, TTM_SETMAXTIPWIDTH, 0, tipWidth);

	        //SendMessage(hToolTips, TTM_SETDELAYTIME, TTDT_AUTOMATIC, 200);

	        SendMessage(hToolTips, TTM_SETDELAYTIME, TTDT_AUTOPOP, tipHideDelay * 1000);
	        SendMessage(hToolTips, TTM_SETDELAYTIME, TTDT_INITIAL, 120);
	        SendMessage(hToolTips, TTM_SETDELAYTIME, TTDT_RESHOW,   60);

                if (NULL != hToolTips)
                      SetAllowTip(allowtip);
                else
                      SetAllowTip(false);
	} 
	else
        SetAllowTip(false);

	return 0;
}

//===========================================================================

void endPlugin(HINSTANCE hPluginInstance)
{
	// Write the current plugin settings to the config file...
	WriteRCSettings();
	// Delete the main plugin menu if it exists (PLEASE NOTE that this takes care of submenus as well!)
	if (myMenu){ DelMenu(myMenu); myMenu = NULL;}

	if(download_thread_semaphore)
	{
		// Signal the download thread that we want to quit.
		abort_download = TRUE;
		// Wait for 10 seconds max.
		WaitForSingleObject(download_thread_semaphore, 10000);

		CloseHandle(download_thread_semaphore);
		download_thread_semaphore = NULL;
	}
	delete_text_line_list();
	delete_favorite_list();
	// Release our timer resources
	KillTimer(hwndPlugin, GETTEXT_TIMER);
	// Make the window unsticky
	RemoveSticky(hwndPlugin);
	// Unregister Blackbox messages...
	SendMessage(hwndBlackbox, BB_UNREGISTERMESSAGE, (WPARAM)hwndPlugin, (LPARAM)msgs);

	if(inSlit && hSlit)
		SendMessage(hSlit, SLIT_REMOVE, NULL, (LPARAM)hwndPlugin);

	ClearToolTips();
  	DestroyWindow(hToolTips);
	// Destroy our window...
	DestroyWindow(hwndPlugin);
	// Unregister window class...
	UnregisterClass(szAppName, hPluginInstance);
}

//===========================================================================

LPCSTR pluginInfo(int field)
{
	switch (field)
	{
		default:
		case 0: return szVersion;
		case 1: return szAppName;
		case 2: return szInfoVersion;
		case 3: return szInfoAuthor;
		case 4: return szInfoRelDate;
		case 5: return szInfoLink;
		case 6: return szInfoEmail;
	}
}

//===========================================================================

LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
		case WM_PAINT:
		{
			OnPaint(hwnd);
		}
		break;

		// ==========

		case BB_RECONFIGURE:
		{
			if (myMenu){ DelMenu(myMenu); myMenu = NULL;}
			GetStyleSettings();
			InvalidateRect(hwnd, NULL, false);
		}
		break;

		// for bbStylemaker
		case BB_REDRAWGUI:
		{
			GetStyleSettings();
			InvalidateRect(hwnd, NULL, false);
			if(highlight == ALL) SetTimer(hwnd, REDRAW_TIMER, 0, NULL);
		}
		break;

		case WM_DISPLAYCHANGE:
		{
			if(!inSlit)
			{
				// IntelliMove(tm)... <g>
				// (c) 2003 qwilk
				//should make this a function so it can be used on startup in case resolution changed since
				//the last time blackbox was used.
				int relx, rely;
				int oldscreenwidth = ScreenWidth;
				int oldscreenheight = ScreenHeight;
				ScreenWidth = GetSystemMetrics(SM_CXVIRTUALSCREEN);
				ScreenHeight = GetSystemMetrics(SM_CYVIRTUALSCREEN);
				if (xpos > oldscreenwidth / 2)
				{
					relx = oldscreenwidth - xpos;
					xpos = ScreenWidth - relx;
				}
				if (ypos > oldscreenheight / 2)
				{
					rely = oldscreenheight - ypos;
					ypos = ScreenHeight - rely;
				}
				MoveWindow(hwnd, xpos, ypos, width, height, true);
			}
		}
		break;

		// ==========

		case BB_BROADCAST:
		{
			szTemp = (char*)lParam;

			if (!_stricmp(szTemp, "@BBShowPlugins") &&  pluginToggle && !inSlit)
			{
				// Show window and force update...
				ShowWindow( hwnd, SW_SHOW);
				InvalidateRect( hwnd, NULL, false);
			}
			else if (!_stricmp(szTemp, "@BBHidePlugins") && pluginToggle && !inSlit)
			{
				// Hide window...
				ShowWindow( hwnd, SW_HIDE);
			}

			//===================

			if (strnicmp(szTemp, "@BBRssReader", 12))
				return 0;
			szTemp += 12;

			//===================

			if (!_stricmp(szTemp, "About"))
			{
				char tmp_str[MAX_LINE_LENGTH];
				SendMessage(hwndBlackbox, BB_HIDEMENU, 0, 0);
				sprintf(tmp_str, "%s", szVersion);
				MessageBox(hwndBlackbox, tmp_str, szAppName, MB_OK | MB_TOPMOST);
			}

			//===================

			else if (!_strnicmp(szTemp, "feedUrl", 7))
			{
				strcpy(feedUrl, szTemp + 8);

				if(GetRssText())
					InvalidateRect(hwnd, NULL, false);
				ShowMyMenu(false);
			}
			else if (!_stricmp(szTemp, "UpdateNow"))
			{
				if(GetRssText())
					InvalidateRect(hwnd, NULL, false);
				ShowMyMenu(false);
			}
			else if (!_strnicmp(szTemp, "Favorites", 9))
			{
				int f = atoi(szTemp + 10);
				favorite *fav = favorite_head;
				for (int i = 0; i < f; i++){
					fav = fav->next;
				}
				strcpy(feedUrl, fav->url);

				if(GetRssText())
					InvalidateRect(hwnd, NULL, false);
				ShowMyMenu(false);
			}
			else if (!_stricmp(szTemp, "AddFavorites"))
			{
				char buf[256];
				sprintf(buf, "bbrssreader.fav%d.url:", numFavorites);
				WriteString(rcpath, buf, feedUrl);

				text_line *line = line_list_head;
				char *text = NULL;
				if(encoding == UTF8){
					text = Utf8ToAnsi(line->title);
				}else{
					text = new char[strlen(line->title) + 1];
					strcpy(text, line->title);
				}
 				sprintf(buf, "bbrssreader.fav%d.title:", numFavorites);
				WriteString(rcpath, buf, text);
				delete text;

				WriteRCSettings();
				ReadRCSettings();
				ShowMyMenu(false);
			}

			else if (!_strnicmp(szTemp, "UpdateInterval", 14))
			{
				updateInterval = atoi(szTemp + 15);
				KillTimer(hwnd, GETTEXT_TIMER);
				SetTimer(hwnd, GETTEXT_TIMER, updateInterval * 60000, NULL);
			}
			else if (!_strnicmp(szTemp, "StyleType", 9))
			{
				styleType = atoi(szTemp + 10);
				GetStyleSettings();
				InvalidateRect(hwnd, NULL, false);
			}
			else if (!_strnicmp(szTemp, "Encode", 6))
			{
				encoding = atoi(szTemp + 7);
				InvalidateRect(hwnd, NULL, false);
          			if(highlight == ALL) SetTimer(hwnd, REDRAW_TIMER, 0, NULL);
				ShowMyMenu(false);
			}
			else if (!_strnicmp(szTemp, "Highlight", 9))
			{
				highlight = atoi(szTemp + 10);
				InvalidateRect(hwnd, NULL, false);
				ShowMyMenu(false);
			}
			//===================

			else if (!_strnicmp(szTemp, "JustifyType", 11))
			{
				justify = atoi(szTemp + 12);
				InvalidateRect(hwnd, NULL, false);
				ShowMyMenu(false);
			}

			//===================

			else if (!_strnicmp(szTemp, "WidthSize", 9))
			{
				newWidth = atoi(szTemp + 10);
				if(ResizeMyWindow(newWidth, height))
					InvalidateRect(hwnd, NULL, false);
			}
			else if (!_strnicmp(szTemp, "HeightSize", 10))
			{
				newHeight = atoi(szTemp + 11);
				if(ResizeMyWindow(width, newHeight))
					InvalidateRect(hwnd, NULL, false);
			}
			else if (!_strnicmp(szTemp, "PaddingSize", 11))
			{
				padding = atoi(szTemp + 12);
				InvalidateRect(hwnd, NULL, false);
          			if(highlight == ALL) SetTimer(hwnd, REDRAW_TIMER, 0, NULL);
			}
			else if (!_strnicmp(szTemp, "TitleOffsetSize", 15))
			{
				title_offset = atoi(szTemp + 16);
				InvalidateRect(hwnd, NULL, false);
          			if(highlight == ALL) SetTimer(hwnd, REDRAW_TIMER, 0, NULL);
			}
			else if (!_strnicmp(szTemp, "DescOffsetSize", 14))
			{
				desc_offset = atoi(szTemp + 15);
				InvalidateRect(hwnd, NULL, false);
          			if(highlight == ALL) SetTimer(hwnd, REDRAW_TIMER, 0, NULL);
			}
			else if (!_strnicmp(szTemp, "TipWidth", 8))
			{
				tipWidth = atoi(szTemp + 9);
				SendMessage(hToolTips, TTM_SETMAXTIPWIDTH, 0, tipWidth);
			}
			else if (!_strnicmp(szTemp, "TipHideDelay", 12))
			{
				tipHideDelay = atoi(szTemp + 13);
				SendMessage(hToolTips, TTM_SETDELAYTIME, TTDT_AUTOPOP, tipHideDelay * 1000);
			}

			else if (!_strnicmp(szTemp, "FontSize", 8))
			{
				fontHeight = atoi(szTemp + 9);
				InvalidateRect(hwnd, NULL, false);
          			if(highlight == ALL) SetTimer(hwnd, REDRAW_TIMER, 0, NULL);
			}

			else if (!_strnicmp(szTemp, "FontFace", 8))
			{
				strcpy(fontFace, szTemp + 9);
				InvalidateRect(hwnd, NULL, false);
          			if(highlight == ALL) SetTimer(hwnd, REDRAW_TIMER, 0, NULL);
				ShowMyMenu(false);
			}
			else if (!_stricmp(szTemp, "UseRefFont"))
			{
				useRefFont = (useRefFont ? false : true);
				InvalidateRect(hwnd, NULL, false);
          			if(highlight == ALL) SetTimer(hwnd, REDRAW_TIMER, 0, NULL);
				ShowMyMenu(false);
			}

			//===================

			else if (!_stricmp(szTemp, "pluginToggle"))
			{
				pluginToggle = (pluginToggle ? false : true);
				SetWindowModes();
			}
			else if (!_stricmp(szTemp, "OnTop"))
			{
				alwaysOnTop = (alwaysOnTop ? false : true);
				if(!inSlit)SetWindowPos( hwnd,
                                                         alwaysOnTop ? HWND_TOPMOST : HWND_NOTOPMOST,
                                                         0, 0, 0, 0,
                                                         SWP_NOACTIVATE | SWP_NOSIZE | SWP_NOMOVE);

				ShowMyMenu(false);
			}

			else if (!_stricmp(szTemp, "InSlit"))
			{
				if(inSlit && hSlit){
					// We are in the slit, so lets unload and get out..
					SendMessage(hSlit, SLIT_REMOVE, NULL, (LPARAM)hwnd);

					// Here you can move to where ever you want ;)
					SetWindowPos(hwnd, NULL, xpos, ypos, 0, 0, SWP_NOZORDER | SWP_NOSIZE);
					inSlit = false;
					SetWindowModes();
					is_autohidden = window_test_autohide(NULL);
					SetTimer(hwnd, AUTOHIDE_TIMER, AUTOHIDE_DELAY, NULL);
				}
				/* Make sure before you try and load into the slit that you have
				* the HWND of the slit ;)
				*/
				else if(hSlit){
					// (Back) into the slit..
					inSlit = true;
					SetWindowModes();
					SendMessage(hSlit, SLIT_ADD, NULL, (LPARAM)hwnd);
				}
			}

			else if (!_stricmp(szTemp, "Transparent"))
			{
				transparency = (transparency ? false : true);
				SetWindowModes();
			}
			else if (!_strnicmp(szTemp, "AlphaValue", 10))
			{
				alpha = atoi(szTemp + 11);
				SetWindowModes();
			}
			else if (!_stricmp(szTemp, "SnapToEdge"))
			{
				snapWindow = (snapWindow ? false : true);
				ShowMyMenu(false);
			}
			else if (!_stricmp(szTemp, "ShowBorder"))
			{
				showBorder = (showBorder ? false : true);
				InvalidateRect(hwnd, NULL, false);
				ShowMyMenu(false);
			}
			else if (!_stricmp(szTemp, "AllowTip"))
			{
				allowtip = (allowtip ? false : true);
				SetAllowTip(allowtip);
				ShowMyMenu(false);
			}
			else if (!_stricmp(szTemp, "LockPosition"))
			{
				lockPos = (lockPos ? false : true);
				ShowMyMenu(false);
                        }
			else if (!_stricmp(szTemp, "AutoHide"))
			{
				autoHide = (autoHide ? false : true);
				is_autohidden = window_test_autohide(NULL);

				bool set_pos = false == inSlit;
				POINT pt = { xpos, ypos };
				bool can_hide = window_test_autohide(&pt);

				SetWindowPos(
                                          hwnd,
                                          can_hide || alwaysOnTop ? HWND_TOPMOST : HWND_NOTOPMOST,
                                          pt.x,
                                          pt.y,
                                          width,
                                          height,
                                          set_pos ? SWP_NOACTIVATE|SWP_NOZORDER|SWP_NOSENDCHANGING
                                                  : SWP_NOACTIVATE|SWP_NOZORDER|SWP_NOMOVE|SWP_NOSENDCHANGING);
				ShowMyMenu(false);
                        }
			else if (!_stricmp(szTemp, "ShowDescription"))
			{
				showDesc = (showDesc ? false : true);
				InvalidateRect(hwnd, NULL, false);
          			if(highlight == ALL) SetTimer(hwnd, REDRAW_TIMER, 0, NULL);
				ShowMyMenu(false);
			}
			else if (!_stricmp(szTemp, "ShowDate"))
			{
				showDate = (showDate ? false : true);
				InvalidateRect(hwnd, NULL, false);
          			if(highlight == ALL) SetTimer(hwnd, REDRAW_TIMER, 0, NULL);
				ShowMyMenu(false);
			}
			else if (!_stricmp(szTemp, "AbbreviatedDate"))
			{
				abbreviatedDate = (abbreviatedDate ? false : true);
				InvalidateRect(hwnd, NULL, false);
          			if(highlight == ALL) SetTimer(hwnd, REDRAW_TIMER, 0, NULL);
				ShowMyMenu(false);
			}
			else if (!_stricmp(szTemp, "DatePosHead"))
			{
				datePosHead = (datePosHead ? false : true);
				InvalidateRect(hwnd, NULL, false);
				ShowMyMenu(false);
			}
			else if (!_stricmp(szTemp, "ShowItemCounts"))
			{
				showItemCounts = (showItemCounts ? false : true);
				InvalidateRect(hwnd, NULL, false);
          			if(highlight == ALL) SetTimer(hwnd, REDRAW_TIMER, 0, NULL);
				ShowMyMenu(false);
			}
			else if (!_stricmp(szTemp, "ShowUpdateTime"))
			{
				showUpdateTime = (showUpdateTime ? false : true);
				InvalidateRect(hwnd, NULL, false);
          			if(highlight == ALL) SetTimer(hwnd, REDRAW_TIMER, 0, NULL);
				ShowMyMenu(false);
			}
			else if (!_stricmp(szTemp, "WordWrap"))
			{
				wordWrap = (wordWrap ? false : true);
				InvalidateRect(hwnd, NULL, false);
          			if(highlight == ALL) SetTimer(hwnd, REDRAW_TIMER, 0, NULL);
				ShowMyMenu(false);
			}
			else if (!_stricmp(szTemp, "TipShowTitle"))
			{
				tipShowTitle = (tipShowTitle ? false : true);
				InvalidateRect(hwnd, NULL, false);
				ShowMyMenu(false);
			}
			else if (!_stricmp(szTemp, "HilightBorder"))
			{
				hilightBorder = (hilightBorder ? false : true);
				InvalidateRect(hwnd, NULL, false);
				ShowMyMenu(false);
			}

			else if (!_stricmp(szTemp, "SetBrowser"))
			{
				OPENFILENAME ofn;
				memset(&ofn, 0, sizeof(OPENFILENAME));
				ofn.lStructSize= sizeof(OPENFILENAME);
				ofn.lpstrFilter= "Programs(*.exe)\0*.exe\0\0";
				ofn.lpstrFile= browserPath;
				ofn.nMaxFile= sizeof(browserPath);
				ofn.Flags= OFN_FILEMUSTEXIST | OFN_HIDEREADONLY;
				GetOpenFileName(&ofn);
			}
			else if (!_stricmp(szTemp, "DefaultBrowser"))
			{
				strcpy(browserPath, "");
			}
			//===================

			else if (!_stricmp(szTemp, "FontColor"))
			{
				CHOOSECOLOR cc;
				COLORREF custCol[16];
				ZeroMemory(&cc, sizeof(CHOOSECOLOR));
				cc.lStructSize = sizeof(CHOOSECOLOR);
				cc.hwndOwner = NULL;
				cc.lpCustColors = (LPDWORD) custCol;
				cc.rgbResult = fontColor;
				cc.Flags = CC_RGBINIT;

				if (ChooseColor(&cc)){
					    fontColor = cc.rgbResult;
					    InvalidateRect(hwnd, NULL, false);
				}
			}

			else if (!_stricmp(szTemp, "SetFont"))
			{
				LOGFONT lf;
				hFont = CreateFont(fontHeight,
						0, 0, 0, FW_NORMAL,
						false, false, false,
						DEFAULT_CHARSET,
						OUT_DEFAULT_PRECIS,
						CLIP_DEFAULT_PRECIS,
						DEFAULT_QUALITY,
						DEFAULT_PITCH, fontFace);
				GetObject(hFont, sizeof(lf), &lf);
				DeleteObject(hFont);

				CHOOSEFONT cf;
				memset(&cf, 0, sizeof(CHOOSEFONT));
				cf.lStructSize = sizeof(CHOOSEFONT);
				cf.hwndOwner = hwnd;
				cf.lpLogFont = &lf;
				cf.nFontType = SCREEN_FONTTYPE;
				cf.Flags = CF_SCREENFONTS | CF_INITTOLOGFONTSTRUCT;

				if (ChooseFont(&cf)){
					strcpy(fontFace, lf.lfFaceName);
					fontHeight = -lf.lfHeight;
					InvalidateRect(hwnd, NULL, false);
					if(highlight == ALL) SetTimer(hwnd, REDRAW_TIMER, 0, NULL);
					ShowMyMenu(false);
				}
			}

			//===================

			else if (!_stricmp(szTemp, "EditRC"))
			{
				BBExecute(GetDesktopWindow(), NULL, rcpath, NULL, NULL, SW_SHOWNORMAL, false);
			}
			else if (!_stricmp(szTemp, "ReloadSettings"))
			{
				ReadRCSettings();
				GetStyleSettings();

				if(inSlit && hSlit)
				{
					SetWindowModes();
					SendMessage(hSlit, SLIT_UPDATE, NULL, NULL);
				}
				else if(!inSlit || !hSlit)
					SendMessage(hSlit, SLIT_REMOVE, NULL, (LPARAM)hwnd);
                                else
					inSlit = false;

				InvalidateRect(hwnd, NULL, false);
			}
			else if (!_stricmp(szTemp, "SaveSettings"))
			{
				WriteRCSettings();
			}
		}
		break;

		// ==========

		case WM_WINDOWPOSCHANGING:
		{
			// Is SnapWindowToEdge enabled?
			if (!inSlit && snapWindow)
			{
				// Snap window to screen edges (if the last bool is false it uses the current DesktopArea)
				if(IsWindowVisible(hwnd)) SnapWindowToEdge((WINDOWPOS*)lParam, 10, true);
			}
		}
		break;

		// ==========

		// Save window position if it changes...
		case WM_WINDOWPOSCHANGED:
		{
			if(!inSlit && !is_autohidden)
			{
				WINDOWPOS* windowpos = (WINDOWPOS*)lParam;
				xpos = windowpos->x;
				ypos = windowpos->y;

				if(ResizeMyWindow(windowpos->cx, windowpos->cy))
					InvalidateRect(hwnd, NULL, false);
			}
		}
		break;

		// ==========

		case WM_NCHITTEST:
		{
			if (GetKeyState(VK_MENU) & 0xF0 && !lockPos)
				return HTBOTTOMRIGHT;
			else
				return HTCAPTION;
		}
		break;

		case WM_EXITSIZEMOVE:
		{
			is_autohidden = window_test_autohide(NULL);
			SetTimer(hwnd, AUTOHIDE_TIMER, AUTOHIDE_DELAY, NULL);
          		if(highlight == ALL) InvalidateRect(hwnd, NULL, false);
		}
		break;

		case WM_NCLBUTTONDOWN:
		{
			if(!inSlit && !lockPos)
				return DefWindowProc(hwnd,message,wParam,lParam);
		}
		break;

		// ==========

		// Right mouse button clicked?
		case WM_NCRBUTTONUP:
		{
			ShowMyMenu(true);
		}
		break;

		case WM_TIMER:
		{
			if (GETTEXT_TIMER == wParam)
			{
				if(GetRssText())
					InvalidateRect(hwnd, NULL, false);
				break;
			}
			if (TRACKMOUSE_TIMER == wParam)
			{
				if (hwnd != window_under_mouse())
				{
					KillTimer(hwnd, wParam);
					PostMessage(hwnd, WM_MOUSELEAVE, 0, 0);
				}
				break;
			}
			if (AUTOHIDE_TIMER == wParam)
			{
				bool hide_it = window_test_autohide(NULL);
				if (hide_it)
				{
					POINT pt; RECT rct;
					GetCursorPos(&pt);
					GetWindowRect(hwnd, &rct);
					if (PtInRect(&rct, pt) || (0x8000 & GetKeyState(VK_LBUTTON)))
						break;
				}
				is_autohidden = hide_it;
				KillTimer(hwnd, wParam);

				bool set_pos = false == inSlit;
				POINT pt = { xpos, ypos };
				bool can_hide = window_test_autohide(&pt);

				SetWindowPos(
                                          hwnd,
                                          can_hide || alwaysOnTop ? HWND_TOPMOST : HWND_NOTOPMOST,
                                          pt.x,
                                          pt.y,
                                          width,
                                          height,
                                          set_pos ? SWP_NOACTIVATE|SWP_NOZORDER|SWP_NOSENDCHANGING
                                                  : SWP_NOACTIVATE|SWP_NOZORDER|SWP_NOMOVE|SWP_NOSENDCHANGING);
				break;
			}
			if (REDRAW_TIMER == wParam)
			{
				InvalidateRect(hwnd, NULL, false);
				KillTimer(hwnd, wParam);
				break;
			}
                }
		break;

		case WM_TEXTUPDATED:
		{
			// Plugin has finished updating it's text. A text_line pointer to the
			// list it has created is passed via the LPARAM of the wndproc.
			text_line *update_head = (text_line *)lParam;
			if(update_head){
				clear_list();
				copy_list(update_head);
			}
			InvalidateRect(hwnd, NULL, false);
          		if(highlight == ALL) SetTimer(hwnd, REDRAW_TIMER, 0, NULL);
		}
		break;

		case WM_NCLBUTTONDBLCLK:
		{
			POINT click;
			click.x = LOWORD(lParam);
			click.y = HIWORD(lParam);
			ScreenToClient(hwnd, &click);

			text_line *line = NULL;

			if(check_label_click(click, &line))
				ClickLabel(line);
		}
		break;

		case WM_NCMOUSEMOVE:
		{
			if(highlight == MOUSEOVER){
				if (false == mouse_over){
					mouse_over = true;
					track_mouse_event(hwnd);
				}
				InvalidateRect(hwnd, NULL, false);
			}
			if (is_autohidden)
			{
				is_autohidden = false;
				if (false == inSlit)
				{
					bool set_pos = false == inSlit;
					POINT pt = { xpos, ypos };
					bool can_hide = window_test_autohide(&pt);

					SetWindowPos(
                                          hwnd,
                                          can_hide || alwaysOnTop ? HWND_TOPMOST : HWND_NOTOPMOST,
                                          pt.x,
                                          pt.y,
                                          width,
                                          height,
                                          set_pos ? SWP_NOACTIVATE|SWP_NOZORDER|SWP_NOSENDCHANGING
                                                  : SWP_NOACTIVATE|SWP_NOZORDER|SWP_NOMOVE|SWP_NOSENDCHANGING);

					SetTimer(hwnd, AUTOHIDE_TIMER, AUTOHIDE_DELAY, NULL);
				}
			}
		}
		break;

		case WM_MOUSELEAVE:
		{
			mouse_over = false;
			InvalidateRect(hwnd, NULL, false);
		}
		break;

		case WM_CLOSE:
		{
			return 0;
		}
		break;
 		// ==========
		default:
			return DefWindowProc(hwnd,message,wParam,lParam);
	}
	return 0;
}

//===========================================================================

void ShowMyMenu(bool popup)
{
	int gap = bevelWidth + borderWidth;
	if (myMenu){ DelMenu(myMenu); myMenu = NULL;}

	rssSubmenu = MakeNamedMenu("RSS", "BBRR_RSS", popup);

	favSubmenu = MakeNamedMenu("Favorites", "BBRR_Favorites", popup);
	favorite *fav = favorite_head;
	for (int i = 0; i < numFavorites; i++){
    		char buf[256];
    		sprintf(buf, "@BBRssReaderFavorites %d", i);
		MakeMenuItem(favSubmenu, fav->title, buf, false);
		fav = fav->next;
	}

	MakeSubmenu(rssSubmenu, favSubmenu, "Favorites");
	MakeMenuItemString(rssSubmenu, "Feed URL", "@BBRssReaderFeedUrl", feedUrl);
	MakeMenuItem(rssSubmenu, "Add Favorites",   "@BBRssReaderAddFavorites", false);
	MakeMenuItem(rssSubmenu, "Update Now",   "@BBRssReaderUpdateNow", false);
	MakeMenuItemInt(rssSubmenu, "Update Interval (min)", "@BBRssReaderUpdateInterval ", updateInterval, 1, 300);
	MakeMenuNOP(rssSubmenu, NULL);

	encodeSubmenu = MakeNamedMenu("encoding", "BBRR_encoding", popup);
	MakeMenuItem(encodeSubmenu, "None",  "@BBRssReaderEncode 0", (encoding == 0));
	MakeMenuItem(encodeSubmenu, "UTF-8", "@BBRssReaderEncode 1", (encoding == 1));
	MakeSubmenu(rssSubmenu, encodeSubmenu, "encoding");

	highlightSubmenu = MakeNamedMenu("highlight", "BBRR_highlight", popup);
	MakeMenuItem(highlightSubmenu, "None",      "@BBRssReaderHighlight 0", (highlight == 0));
	MakeMenuItem(highlightSubmenu, "MouseOver", "@BBRssReaderHighlight 1", (highlight == 1));
	MakeMenuItem(highlightSubmenu, "ALL",       "@BBRssReaderHighlight 2", (highlight == 2));
	MakeMenuNOP(highlightSubmenu, NULL);
	MakeMenuItem(highlightSubmenu, "Show Border", "@BBRssReaderHilightBorder", hilightBorder);
	MakeSubmenu(rssSubmenu, highlightSubmenu, "highlight");

	browserSubmenu = MakeNamedMenu("browser", "BBRR_browser", popup);
	MakeMenuItem(browserSubmenu, "Browse...", "@BBRssReaderSetBrowser", false);
	MakeMenuItem(browserSubmenu, "Default", "@BBRssReaderDefaultBrowser", false);
	MakeSubmenu(rssSubmenu, browserSubmenu, "browser");

	tooltipSubmenu = MakeNamedMenu("tooltip", "BBRR_tooltip", popup);
	MakeMenuItem(tooltipSubmenu, "Allow Tooltip", "@BBRssReaderAllowTip", allowtip);
	MakeMenuItem(tooltipSubmenu, "Show Title", "@BBRssReaderTipShowTitle", tipShowTitle);
	MakeMenuNOP(tooltipSubmenu, NULL);
	MakeMenuItemInt(tooltipSubmenu, "Tooltip Width", "@BBRssReaderTipWidth", tipWidth, 1, 1000);
	MakeMenuItemInt(tooltipSubmenu, "Hide Delay (sec)", "@BBRssReaderTipHideDelay", tipHideDelay, 1, 300);
	MakeSubmenu(rssSubmenu, tooltipSubmenu, "tooltip");

	MakeMenuNOP(rssSubmenu, NULL);
	MakeMenuItemInt(rssSubmenu, "Padding Size", "@BBRssReaderPaddingSize", padding, 0, 50);

	offsetSubmenu = MakeNamedMenu("Offset Size", "BBRR_offset", popup);
	MakeMenuItemInt(offsetSubmenu, "title", "@BBRssReaderTitleOffsetSize", title_offset, 0, 50);
	MakeMenuItemInt(offsetSubmenu, "description", "@BBRssReaderDescOffsetSize", desc_offset, 0, 50);
	MakeSubmenu(rssSubmenu, offsetSubmenu, "Offset Size");

	MakeMenuNOP(rssSubmenu, NULL);
	MakeMenuItem(rssSubmenu, "Show Description", "@BBRssReaderShowDescription", showDesc);
	MakeMenuItem(rssSubmenu, "Show Date", "@BBRssReaderShowDate", showDate);
	MakeMenuItem(rssSubmenu, "Abbreviated Date", "@BBRssReaderAbbreviatedDate", abbreviatedDate);
	MakeMenuItem(rssSubmenu, "Date Position Head", "@BBRssReaderDatePosHead", datePosHead);
	MakeMenuItem(rssSubmenu, "Show Item Counts", "@BBRssReaderShowItemCounts", showItemCounts);
	MakeMenuItem(rssSubmenu, "Show Update Time", "@BBRssReaderShowUpdateTime", showUpdateTime);
	MakeMenuItem(rssSubmenu, "Word Wrap", "@BBRssReaderWordWrap", wordWrap);

	configSubmenu = MakeNamedMenu("Configuration", "BBRR_Config", popup);
	MakeMenuItemInt(configSubmenu, "Width Size", "@BBRssReaderWidthSize", width, 1, GetSystemMetrics(SM_CXVIRTUALSCREEN));
	MakeMenuItemInt(configSubmenu, "Height Size", "@BBRssReaderHeightSize", height, 1, GetSystemMetrics(SM_CYVIRTUALSCREEN));

	fontSubmenu = MakeNamedMenu("Font", "BBRR_Font", popup);
 	MakeMenuItem(fontSubmenu, "Use Ref Font", "@BBRssReaderUseRefFont", useRefFont);
	if(!useRefFont){
	MakeMenuItemInt(fontSubmenu, "Font Size", "@BBRssReaderFontSize", fontHeight, 1, 64);
	MakeMenuItemString(fontSubmenu, "Font Face", "@BBRssReaderFontFace", fontFace);
	MakeMenuItem(fontSubmenu, "Font Color",  "@BBRssReaderFontColor", false);
	MakeMenuItem(fontSubmenu, "Browse...",  "@BBRssReaderSetFont", false);
	}
	MakeSubmenu(configSubmenu, fontSubmenu, "Font");

	MakeMenuNOP(configSubmenu, NULL);
	if(hSlit) MakeMenuItem(configSubmenu, "In Slit", "@BBRssReaderInSlit", inSlit);
	MakeMenuItem(configSubmenu, "Plugin Toggle", "@BBRssReaderPluginToggle", pluginToggle);
	MakeMenuItem(configSubmenu, "Always on Top", "@BBRssReaderOnTop", alwaysOnTop);
	MakeMenuItem(configSubmenu, "Transparency", "@BBRssReaderTransparent", transparency);
	MakeMenuItemInt(configSubmenu, "Alpha Value", "@BBRssReaderAlphaValue", alpha, 0, 255);
	MakeMenuItem(configSubmenu, "Snap To Edge", "@BBRssReaderSnapToEdge", snapWindow);
	MakeMenuItem(configSubmenu, "Show Border", "@BBRssReaderShowBorder", showBorder);
	MakeMenuItem(configSubmenu, "Lock Position", "@BBRssReaderLockPosition", lockPos);
	MakeMenuItem(configSubmenu, "Auto Hide", "@BBRssReaderAutoHide", autoHide);

	styleSubmenu = MakeNamedMenu("Style Type", "BBRR_StyleType", popup);
	MakeMenuItem(styleSubmenu, "Toolbar",  "@BBRssReaderStyleType 1", (styleType == 1));
	MakeMenuItem(styleSubmenu, "Button",   "@BBRssReaderStyleType 2", (styleType == 2));
	MakeMenuItem(styleSubmenu, "ButtonP",  "@BBRssReaderStyleType 3", (styleType == 3));
	MakeMenuItem(styleSubmenu, "Label",    "@BBRssReaderStyleType 4", (styleType == 4));
	MakeMenuItem(styleSubmenu, "WinLabel", "@BBRssReaderStyleType 5", (styleType == 5));
	MakeMenuItem(styleSubmenu, "Clock",    "@BBRssReaderStyleType 6", (styleType == 6));

	justifySubmenu = MakeNamedMenu("Justification", "BBRR_JustifyType", popup);
	MakeMenuItem(justifySubmenu, "Left",     "@BBRssReaderJustifyType 0", (justify == 0));
	MakeMenuItem(justifySubmenu, "Center",   "@BBRssReaderJustifyType 1", (justify == 1));
	MakeMenuItem(justifySubmenu, "Right",    "@BBRssReaderJustifyType 2", (justify == 2));

	settingsSubmenu = MakeNamedMenu("Settings", "BBRR_Settings", popup);
	MakeMenuItem(settingsSubmenu, "Edit Settings",   "@BBRssReaderEditRC", false);
	MakeMenuItem(settingsSubmenu, "Reload Settings", "@BBRssReaderReloadSettings", false);
	MakeMenuItem(settingsSubmenu, "Save Settings",   "@BBRssReaderSaveSettings", false);

	myMenu = MakeNamedMenu("BBRssReader", "BBRR_Main", popup);
	MakeSubmenu(myMenu, rssSubmenu, "RSS");
	MakeSubmenu(myMenu, configSubmenu, "Configuration");
	MakeSubmenu(myMenu, styleSubmenu, "Style Type");
	MakeSubmenu(myMenu, justifySubmenu, "Justification");
	MakeSubmenu(myMenu, settingsSubmenu, "Settings");
  	MakeMenuItem(myMenu, "About", "@BBRssReaderAbout", false);
	ShowMenu(myMenu);
}

//===========================================================================

void GetStyleSettings()
{
	bevelWidth  = *(int*)GetSettingPtr(SN_BEVELWIDTH);
	borderWidth = *(int*)GetSettingPtr(SN_BORDERWIDTH);
	borderColor = *(COLORREF*)GetSettingPtr(SN_BORDERCOLOR);

	myStyleItem = *(StyleItem*)GetSettingPtr(styleType);

	ShowMyMenu(false);
}

//===========================================================================

int countFavorites()
{
	int count = 0;
	char strTemp[256] = "";
	char strLine[256] = "";
	FILE *frc = fopen(rcpath, "rt");
	if (NULL != frc)
	{
		while (0 == feof(frc))
		{
			sprintf(strTemp, "bbrssreader.fav%d.url:", count);
			fgets(strLine, 256, frc);
			if (0 == strncmp(strLine, strTemp, strlen(strTemp)))
			{
				fseek(frc, 0, SEEK_SET);
				count++;
			}
		}
	}
	fclose(frc);
	return count;
}

//===========================================================================

void ReadRCSettings()
{
	char temp[MAX_LINE_LENGTH], path[MAX_LINE_LENGTH], defaultpath[MAX_LINE_LENGTH];
	int nLen;

	// First we look for the config file in the same folder as the plugin...
	GetModuleFileName(hInstance, rcpath, sizeof(rcpath));
	nLen = strlen(rcpath) - 1;
	while (nLen >0 && rcpath[nLen] != '\\') nLen--;
	rcpath[nLen + 1] = 0;
	strcpy(temp, rcpath);
	strcpy(path, rcpath);
	strcat(temp, "bbrssreader.rc");
	strcat(path, "bbrssreaderrc");
	// ...checking the two possible filenames bbrssreader.rc and bbrssreaderrc ...
	if (FileExists(temp)) strcpy(rcpath, temp);
	else if (FileExists(path)) strcpy(rcpath, path);
	// ...if not found, we try the Blackbox directory...
	else
	{
		// ...but first we save the default path (bbrssreader.rc in the same
		// folder as the plugin) just in case we need it later (see below)...
		strcpy(defaultpath, temp);
		GetBlackboxPath(rcpath, sizeof(rcpath));
		strcpy(temp, rcpath);
		strcpy(path, rcpath);
		strcat(temp, "bbrssreader.rc");
		strcat(path, "bbrssreaderrc");
		if (FileExists(temp)) strcpy(rcpath, temp);
		else if (FileExists(path)) strcpy(rcpath, path);
		else // If no config file was found, we use the default path and settings, and return
		{
			strcpy(rcpath, defaultpath);
			xpos = ypos = 10;
			width = 200;
			height = 250;
			fontHeight = 12;
			alpha = 160;
			styleType = 1;
			justify = 0;
			padding = 5;
			title_offset = 5;
			desc_offset = 15;
			updateInterval = 15;
			encoding = 1;
			highlight = 1;
			tipWidth = 350;
			tipHideDelay = 10;
			alwaysOnTop = false;
			transparency = false;
			snapWindow = true;
			pluginToggle = false;
			inSlit = false;
			showBorder = false;
			allowtip = true;
			lockPos = false;
			autoHide = false;
			useRefFont = true;
			showDesc = false;
			showDate = false;
			abbreviatedDate = true;
			datePosHead = false;
			showItemCounts = false;
			showUpdateTime = false;
			wordWrap = false;
			tipShowTitle = true;
			hilightBorder = false;
			strcpy(fontFace, "Tahoma");
			WriteRCSettings(); //write a file for editing later
			return;
		}
	}

	// If a config file was found we read the plugin settings from the file...
	xpos = ReadInt(rcpath, "bbrssreader.x:", 10);
	ypos = ReadInt(rcpath, "bbrssreader.y:", 10);
	if (xpos >= GetSystemMetrics(SM_CXVIRTUALSCREEN)) xpos = 10;
	if (ypos >= GetSystemMetrics(SM_CYVIRTUALSCREEN)) ypos = 10;
	width = ReadInt(rcpath, "bbrssreader.width:", 200);
  	height = ReadInt(rcpath, "bbrssreader.height:", 250);
	fontHeight = ReadInt(rcpath, "bbrssreader.fontHeight:", 12);
	alpha = ReadInt(rcpath, "bbrssreader.alpha:", 160);
	styleType = ReadInt(rcpath, "bbrssreader.styleType:", 1);
	justify = ReadInt(rcpath, "bbrssreader.justification:", 0);
	padding = ReadInt(rcpath, "bbrssreader.padding:", 5);
	title_offset = ReadInt(rcpath, "bbrssreader.title_offset:", 5);
	desc_offset = ReadInt(rcpath, "bbrssreader.desc_offset:", 15);
	updateInterval = ReadInt(rcpath, "bbrssreader.updateInterval(min):", 15);
	encoding = ReadInt(rcpath, "bbrssreader.encoding:", 1);
	highlight = ReadInt(rcpath, "bbrssreader.highlight:", 1);
	tipWidth = ReadInt(rcpath, "bbrssreader.tipWidth:", 350);
	tipHideDelay = ReadInt(rcpath, "bbrssreader.tipHideDelay:", 10);
	alwaysOnTop = ReadBool(rcpath, "bbrssreader.alwaysontop:", false);
	transparency = ReadBool(rcpath, "bbrssreader.transparency:", false);
	snapWindow = ReadBool(rcpath, "bbrssreader.snapwindow:", true);
	pluginToggle = ReadBool(rcpath, "bbrssreader.pluginToggle:", false);
	inSlit = ReadBool(rcpath, "bbrssreader.inSlit:", false);
	showBorder = ReadBool(rcpath, "bbrssreader.showBorder:", false);
	allowtip = ReadBool(rcpath, "bbrssreader.allowtooltip:", true);
	lockPos = ReadBool(rcpath, "bbrssreader.lockPos:", false);
	autoHide = ReadBool(rcpath, "bbrssreader.autoHide:", false);
	useRefFont = ReadBool(rcpath, "bbrssreader.useRefFont:", true);
	showDesc = ReadBool(rcpath, "bbrssreader.showDesc:", false);
	showDate = ReadBool(rcpath, "bbrssreader.showDate:", false);
	abbreviatedDate = ReadBool(rcpath, "bbrssreader.abbreviatedDate:", true);
	datePosHead = ReadBool(rcpath, "bbrssreader.datePosHead:", false);
	showItemCounts = ReadBool(rcpath, "bbrssreader.showItemCounts:", false);
	showUpdateTime = ReadBool(rcpath, "bbrssreader.showUpdateTime:", false);
	wordWrap = ReadBool(rcpath, "bbrssreader.wordWrap:", false);
	tipShowTitle = ReadBool(rcpath, "bbrssreader.tipShowTitle:", true);
	hilightBorder = ReadBool(rcpath, "bbrssreader.hilightBorder:", false);
	fontColor = ReadColor(rcpath, "bbrssreader.fontColor:", "#000000");
	strcpy(fontFace, ReadString(rcpath, "bbrssreader.font:", "Tahoma"));
	strcpy(feedUrl, ReadString(rcpath, "bbrssreader.feedUrl:", ""));
	strcpy(browserPath, ReadString(rcpath, "bbrssreader.browserPath:", ""));

	numFavorites = countFavorites();
	char buf[256];
	favorite_head = NULL;
	for (int i = 0; i < numFavorites; i++){
	      if(!favorite_head){
		      favorite_head = new favorite;
		      favorite_tail = favorite_head;
	      }else{
		      favorite *new_fav = new favorite;
		      favorite_tail->next = new_fav;
		      favorite_tail = new_fav;
	      }
	      memset(favorite_tail, 0, sizeof(favorite));

	      sprintf(buf, "bbrssreader.fav%d.url:", i);
	      strcpy(favorite_tail->url, ReadString(rcpath, buf, ""));
	      sprintf(buf, "bbrssreader.fav%d.title:", i);
	      strcpy(favorite_tail->title, ReadString(rcpath, buf, ""));
	}
}

//===========================================================================

void WriteRCSettings()
{
	WriteInt(rcpath, "bbrssreader.x:", xpos);
	WriteInt(rcpath, "bbrssreader.y:", ypos);
	WriteInt(rcpath, "bbrssreader.width:", width);
	WriteInt(rcpath, "bbrssreader.height:", height);
	WriteInt(rcpath, "bbrssreader.fontHeight:", fontHeight);
	WriteInt(rcpath, "bbrssreader.alpha:", alpha);
	WriteInt(rcpath, "bbrssreader.styleType:", styleType);
	WriteInt(rcpath, "bbrssreader.justification:", justify);
	WriteInt(rcpath, "bbrssreader.padding:", padding);
	WriteInt(rcpath, "bbrssreader.title_offset:", title_offset);
	WriteInt(rcpath, "bbrssreader.desc_offset:", desc_offset);
	WriteInt(rcpath, "bbrssreader.updateInterval(min):", updateInterval);
	WriteInt(rcpath, "bbrssreader.encoding:", encoding);
	WriteInt(rcpath, "bbrssreader.highlight:", highlight);
	WriteInt(rcpath, "bbrssreader.tipWidth:", tipWidth);
	WriteInt(rcpath, "bbrssreader.tipHideDelay:", tipHideDelay);
	WriteBool(rcpath, "bbrssreader.alwaysontop:", alwaysOnTop);
	WriteBool(rcpath, "bbrssreader.transparency:", transparency);
	WriteBool(rcpath, "bbrssreader.snapWindow:", snapWindow);
	WriteBool(rcpath, "bbrssreader.pluginToggle:", pluginToggle);
	WriteBool(rcpath, "bbrssreader.inSlit:", inSlit);
	WriteBool(rcpath, "bbrssreader.showBorder:", showBorder);
	WriteBool(rcpath, "bbrssreader.allowtooltip:", allowtip);
	WriteBool(rcpath, "bbrssreader.lockPos:", lockPos);
	WriteBool(rcpath, "bbrssreader.autoHide:", autoHide);
	WriteBool(rcpath, "bbrssreader.useRefFont:", useRefFont);
	WriteBool(rcpath, "bbrssreader.showDesc:", showDesc);
	WriteBool(rcpath, "bbrssreader.showDate:", showDate);
	WriteBool(rcpath, "bbrssreader.abbreviatedDate:", abbreviatedDate);
	WriteBool(rcpath, "bbrssreader.datePosHead:", datePosHead);
	WriteBool(rcpath, "bbrssreader.showItemCounts:", showItemCounts);
	WriteBool(rcpath, "bbrssreader.showUpdateTime:", showUpdateTime);
	WriteBool(rcpath, "bbrssreader.wordWrap:", wordWrap);
	WriteBool(rcpath, "bbrssreader.tipShowTitle:", tipShowTitle);
	WriteBool(rcpath, "bbrssreader.hilightBorder:", hilightBorder);
	WriteColor(rcpath, "bbrssreader.fontColor:", fontColor);
	WriteString(rcpath, "bbrssreader.font:", fontFace);
	WriteString(rcpath, "bbrssreader.feedUrl:", feedUrl);
	WriteString(rcpath, "bbrssreader.browserPath:", browserPath);
}

//===========================================================================

char *Utf8ToAnsi(char *utf8)
{
	WCHAR *unicode = NULL;
	int unicode_length = 0;
	char *ascii = NULL;
	int ascii_length = 0;

	int result = MultiByteToWideChar(CP_UTF8, 0, utf8, strlen(utf8), NULL, 0);

	unicode_length = result;
	unicode = new WCHAR[unicode_length + 1];
	memset(unicode, 0, (unicode_length + 1) * sizeof(WCHAR));
	MultiByteToWideChar(CP_UTF8, 0, utf8, strlen(utf8), unicode, unicode_length);

	result = WideCharToMultiByte(CP_ACP, 0, unicode, unicode_length, NULL, 0, NULL, NULL);

	ascii_length = result;
	ascii = new char[ascii_length + 1];
	memset(ascii, 0, (ascii_length + 1) * sizeof(char));
	WideCharToMultiByte(CP_ACP, 0, unicode, unicode_length, ascii, ascii_length, NULL, NULL);

	delete unicode;
	return ascii;
}

char *abbreviatedDateString(char *date)
{
       const char *p = date;
       int i = 0;
       static char months[12][4] = {
       "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
       };
       char *buf = NULL;
       buf = new char[strlen(date)];

       if ( !strchr( date, '-' ) ) {
       // RFC822
       int yy = 0, mm = 0, dd = 00;
       while ( *p && ((*p < '0') || (*p > '9')) )
              p++;
       if ( (*p >= '0') && (*p <= '9') ) {
              dd = *p - '0';
              p++;
              if ( (*p >= '0') && (*p <= '9') ) {
                     dd = dd * 10 + (*p - '0');
                     p++;
              }
       }

       while ( *p && ((*p < 'A') || (*p > 'Z')) )
              p++;
       if ( (*p >= 'A') && (*p <= 'Z') ) {
              for ( i = 0; i < 12; i++ ) {
                     if ( !strncmp( p, months[i], 3 ) ) {
                            mm = i + 1;
                            break;
                     }
              }
       }
       while ( *p && ((*p < '0') || (*p > '9')) )
       p++;
       p += 4;

       sprintf(buf, "%02d/%02d", mm, dd );
       i = strlen(buf);

       }else{

       // ISO8601
       while( *p && (*p != '-') ) p++;
       p += 1;

       while ( *p && (*p != '-') )
           buf[i++] = *p++;
       strcpy( &(buf[i]), "/" );
       p++;
       i += 1;

       while ( *p && (*p != 'T') )
           buf[i++] = *p++;
       strcpy( &(buf[i]), " " );
       p++;
       i += 1;

       }

       // time
       while ( *p && (*p != ':') )
       buf[i++] = *p++;
       strcpy( &(buf[i]), ":" );
       p++;
       i += 1;

       while ( *p && (*p != ':') )
       buf[i++] = *p++;
       strcpy( &(buf[i]), "\0");
       p++;
       i += 1;

       return buf;
}

//=========================================================================

void OnPaint(HWND hwnd)
{
	ClearToolTips();

	// Create buffer hdc's, bitmaps etc.
	PAINTSTRUCT ps; RECT r, textrc, descrc;
	int gap = bevelWidth + borderWidth;

	//get screen buffer
	HDC hdc_scrn = BeginPaint(hwnd, &ps);

	//get window rectangle.
	GetClientRect(hwnd, &r);

	//first get a new 'device context'
	HDC hdc = CreateCompatibleDC(NULL);

	//then create a buffer in memory with the window size
	HBITMAP bufbmp = CreateCompatibleBitmap(hdc_scrn, r.right, r.bottom);

	//select the bitmap into the DC
	HGDIOBJ otherbmp = SelectObject(hdc, bufbmp);

	MakeStyleGradient(hdc, &r, &myStyleItem, showBorder);

  	r.left += gap;
	r.top += gap;
	r.bottom -= gap;
	r.right -= gap;

	if(useRefFont){
		tbStyleItem  = *(StyleItem*)GetSettingPtr(SN_TOOLBAR);
		refStyleItem = *(StyleItem*)GetSettingPtr(styleType);
		otherfont = SelectObject(hdc,
					CreateFont(tbStyleItem.FontHeight,
					0, 0, 0, FW_NORMAL,
					false, false, false,
					DEFAULT_CHARSET,
					OUT_DEFAULT_PRECIS,
					CLIP_DEFAULT_PRECIS,
					DEFAULT_QUALITY,
					DEFAULT_PITCH, tbStyleItem.Font));
		SetTextColor(hdc, refStyleItem.TextColor);
	}else{
		otherfont = SelectObject(hdc,
					CreateFont(fontHeight,
					0, 0, 0, FW_NORMAL,
					false, false, false,
					DEFAULT_CHARSET,
					OUT_DEFAULT_PRECIS,
					CLIP_DEFAULT_PRECIS,
					DEFAULT_QUALITY,
					DEFAULT_PITCH, fontFace));

		SetTextColor(hdc, fontColor);
	}
	SetBkMode(hdc, TRANSPARENT);


	if(highlight == MOUSEOVER){
	POINT pt;
	GetCursorPos(&pt);
	ScreenToClient(hwnd, &pt);
	text_line *line = NULL;
	if(check_label_click(pt, &line) && &line->text_rect && mouse_over)
		MakeStyleGradient(hdc, &line->text_rect, &myStyleItem, hilightBorder);
	}

	text_line *draw = line_list_head;
	text_draw_y = r.top;

	while(draw)
	{

		if(highlight == ALL && &draw->text_rect) MakeStyleGradient(hdc, &draw->text_rect, &myStyleItem, hilightBorder);

		char *text = NULL;
		char *temp = NULL;
		char *desc = NULL;
		char *title = NULL;

		if(encoding == UTF8){
			text = Utf8ToAnsi(draw->title);
                }else{
			text = new char[strlen(draw->title) + 1];
			strcpy(text, draw->title);
                }

		// top title
		if(draw == line_list_head){
			if(showItemCounts && showUpdateTime){
				title = new char[strlen(text) + strlen(updateTime) + 15];
				sprintf(title,"%s  (%d) (%s)", text, Item_count - 1, updateTime);
                        }else if(showItemCounts){
				title = new char[strlen(text) + 10];
				sprintf(title,"%s  (%d)", text, Item_count - 1);
                        }else if(showUpdateTime){
				title = new char[strlen(text) + strlen(updateTime) + 5];
				sprintf(title,"%s  (%s)", text, updateTime);
                        }else{
				title = new char[strlen(text) + 1];
				strcpy(title, text);
                        }

			draw->text_rect.top = text_draw_y;
			draw->text_rect.left = r.left;
			draw->text_rect.right = r.right;

			textrc = draw->text_rect;
			textrc.left = draw->text_rect.left + gap;
			textrc.right = draw->text_rect.right - gap;
			if(wordWrap) DrawText(hdc, title, strlen(title), &textrc, justify | DT_WORDBREAK | DT_CALCRECT);
			else DrawText(hdc, title, strlen(title), &textrc, justify | DT_SINGLELINE | DT_WORD_ELLIPSIS | DT_CALCRECT);
			draw->text_rect.bottom = textrc.bottom + padding;

			int i = (draw->text_rect.bottom - draw->text_rect.top - (textrc.bottom - textrc.top));
			textrc.top = draw->text_rect.top + i / 2;
			textrc.left = draw->text_rect.left + gap;
			textrc.right = draw->text_rect.right - gap;
			textrc.bottom = draw->text_rect.bottom + i / 2;
			if(wordWrap) DrawText(hdc, title, strlen(title), &textrc, justify | DT_WORDBREAK);
			else DrawText(hdc, title, strlen(title), &textrc, justify | DT_SINGLELINE | DT_WORD_ELLIPSIS);

			text_draw_y = draw->text_rect.bottom;
			if(tipShowTitle) SetToolTip(&draw->text_rect, title);
                }else{


		// title + date
		if(draw->date && showDate){
			if(abbreviatedDate){
				char *date = NULL;
				date = abbreviatedDateString(draw->date);
				temp = new char[strlen(text) + strlen(date) + 5];
				if(datePosHead) sprintf(temp,"(%s)  %s", date, text);
				else sprintf(temp,"%s  (%s)", text, date);
          			delete date;
			}else{
				temp = new char[strlen(text) + strlen(draw->date) + 5];
				if(datePosHead) sprintf(temp,"(%s)  %s", draw->date, text);
				else sprintf(temp,"%s  (%s)", text, draw->date);
			}
			draw->text_rect.top = text_draw_y;
			draw->text_rect.left = r.left;
			draw->text_rect.right = r.right;

			textrc = draw->text_rect;
			textrc.left = draw->text_rect.left + title_offset + gap;
			textrc.right = draw->text_rect.right - gap;
			if(wordWrap) DrawText(hdc, temp, strlen(temp), &textrc, justify | DT_WORDBREAK | DT_CALCRECT);
			else DrawText(hdc, temp, strlen(temp), &textrc, justify | DT_SINGLELINE | DT_WORD_ELLIPSIS | DT_CALCRECT);
			draw->text_rect.bottom = textrc.bottom + padding;

			int i = (draw->text_rect.bottom - draw->text_rect.top - (textrc.bottom - textrc.top));
			textrc.top = draw->text_rect.top + i / 2;
			textrc.left = draw->text_rect.left + title_offset + gap;
			textrc.right = draw->text_rect.right - gap;
			textrc.bottom = draw->text_rect.bottom + i / 2;
			if(wordWrap) DrawText(hdc, temp, strlen(temp), &textrc, justify | DT_WORDBREAK);
			else DrawText(hdc, temp, strlen(temp), &textrc, justify | DT_SINGLELINE | DT_WORD_ELLIPSIS);

                }else{ // title only
			draw->text_rect.top = text_draw_y;
			draw->text_rect.left = r.left;
			draw->text_rect.right = r.right;

			textrc = draw->text_rect;
			textrc.left = draw->text_rect.left + title_offset + gap;
			textrc.right = draw->text_rect.right - gap;
			if(wordWrap) DrawText(hdc, text, strlen(text), &textrc, justify | DT_WORDBREAK | DT_CALCRECT);
			else DrawText(hdc, text, strlen(text), &textrc, justify | DT_SINGLELINE | DT_WORD_ELLIPSIS | DT_CALCRECT);
			draw->text_rect.bottom = textrc.bottom + padding;

			int i = (draw->text_rect.bottom - draw->text_rect.top - (textrc.bottom - textrc.top));
			textrc.top = draw->text_rect.top + i / 2;
			textrc.left = draw->text_rect.left + title_offset + gap;
			textrc.right = draw->text_rect.right - gap;
			textrc.bottom = draw->text_rect.bottom + i / 2;
			if(wordWrap) DrawText(hdc, text, strlen(text), &textrc, justify | DT_WORDBREAK);
			else DrawText(hdc, text, strlen(text), &textrc, justify | DT_SINGLELINE | DT_WORD_ELLIPSIS);
                }
		text_draw_y = draw->text_rect.bottom;

		// description
		if(draw->description){
			if(encoding == UTF8){
				desc = Utf8ToAnsi(draw->description);
			}else{
				desc = new char[strlen(draw->description) + 1];
				strcpy(desc, draw->description);
			}

			if(showDesc){
				descrc = draw->text_rect;
				descrc.top = text_draw_y;
				descrc.left += desc_offset + gap;
				descrc.right -= gap;
				if(wordWrap) DrawText(hdc, desc, strlen(desc), &descrc, justify | DT_WORDBREAK | DT_CALCRECT);
				else DrawText(hdc, desc, strlen(desc), &descrc, justify | DT_SINGLELINE | DT_WORD_ELLIPSIS | DT_CALCRECT);
				draw->text_rect.bottom = descrc.bottom + padding;

				descrc = draw->text_rect;
				descrc.top = text_draw_y;
				descrc.left += desc_offset + gap;
				descrc.right -= gap;
				if(wordWrap) DrawText(hdc, desc, strlen(desc), &descrc, justify | DT_WORDBREAK);
				else DrawText(hdc, desc, strlen(desc), &descrc, justify | DT_SINGLELINE | DT_WORD_ELLIPSIS);
				text_draw_y = draw->text_rect.bottom;
			}

			// tooltip
			char *tip = NULL;
			if(tipShowTitle){
				tip = new char[strlen(temp ? temp : text) + strlen(desc) + 3];
				sprintf(tip,"%s\n\n%s", temp ? temp : text, desc);
			}else{
				tip = new char[strlen(desc) + 1];
				sprintf(tip,"%s", desc);
			}

			if(strlen(tip) > 1024){
				char tipText[1024];
				strncpy(tipText, tip, 1023);
				SetToolTip(&draw->text_rect, tipText);
			}
			else SetToolTip(&draw->text_rect, tip);
 			delete tip;
		}
		else if(tipShowTitle) SetToolTip(&draw->text_rect, temp ? temp : text);

		}

		if(text){ delete text; text = NULL; }
		if(temp){ delete temp; temp = NULL; }
		if(desc){ delete desc; desc = NULL; }
		if(title){ delete title; title = NULL; }

		draw = draw->next_line;
	}

	// Finally, copy from the paint buffer to the window...
	BitBlt(hdc_scrn, 0, 0, width, height, hdc, 0, 0, SRCCOPY);

	// Remember to delete all objects!
	DeleteObject(SelectObject(hdc, otherfont));
	DeleteObject(SelectObject(hdc, otherbmp));
	DeleteDC(hdc);

	EndPaint(hwnd, &ps);
}

//=========================================================================

//so there you just use BBSLWA like normal SLWA
//(c)grischka
BOOL WINAPI BBSetLayeredWindowAttributes(HWND hwnd, COLORREF crKey, BYTE bAlpha, DWORD dwFlags)
{
	static BOOL (WINAPI *pSLWA)(HWND, COLORREF, BYTE, DWORD);
	static int f=0;
	for (;;) {
		if (2==f)   return pSLWA(hwnd, crKey, bAlpha, dwFlags);
		// if it's not there, just do nothing and report success
		if (f)      return TRUE;
		*(FARPROC*)&pSLWA = GetProcAddress(GetModuleHandle("USER32"), "SetLayeredWindowAttributes");
		f = pSLWA ? 2 : 1;
	}
}

//=========================================================================

struct tt
{
	struct tt *next;
	char used_flg;
	char text[1024];
	TOOLINFO ti;
} *tt0;

void SetToolTip(RECT *tipRect, CHAR *tipText)
{
	if (NULL==hToolTips) return;

	struct tt **tp, *t; unsigned n=0;
	for (tp=&tt0; NULL!=(t=*tp); tp=&t->next){
		if (0==memcmp(&t->ti.rect, tipRect, sizeof(RECT))){
			t->used_flg = 1;
			if (0!=strcmp(t->ti.lpszText, tipText)){
				strcpy(t->text, tipText);
				SendMessage(hToolTips, TTM_UPDATETIPTEXT, 0, (LPARAM)&t->ti);
			}
			return;
		}
		if (t->ti.uId > n)
			n = t->ti.uId;
	}

	t = (struct tt*)c_alloc(sizeof (*t));
	t->used_flg  = 1;
	t->next = NULL;
	strcpy(t->text, tipText);
	*tp = t;

	memset(&t->ti, 0, sizeof(TOOLINFO));

	t->ti.cbSize   = sizeof(TOOLINFO);
	t->ti.uFlags   = TTF_SUBCLASS;
	t->ti.hwnd     = hwndPlugin;
	t->ti.uId      = n+1;
	//t->ti.hinst    = NULL;
	t->ti.lpszText = t->text;
	t->ti.rect     = *tipRect;

	SendMessage(hToolTips, TTM_ADDTOOL, 0, (LPARAM)&t->ti);
}

void ClearToolTips(void)
{
	struct tt **tp, *t;
	tp=&tt0; while (NULL!=(t=*tp)){
		if (0==t->used_flg){
			SendMessage(hToolTips, TTM_DELTOOL, 0, (LPARAM)&t->ti);
			*tp=t->next;
			m_free(t);
		}else{
			t->used_flg = 0;
			tp=&t->next;
		}
	}
}

void SetAllowTip(bool allowtip)
{
	if (NULL != hToolTips){
		SendMessage(hToolTips, TTM_ACTIVATE, (WPARAM)allowtip, 0);
	}else{
		SendMessage(hToolTips, TTM_ACTIVATE, (WPARAM)false, 0);
	}
}

//===========================================================================

void SetWindowModes(void)
{
	if(usingWin2kXP){
		if(!inSlit){
			// Add the WS_EX_LAYERED atribute to the window.
			SetWindowLong(hwndPlugin, GWL_EXSTYLE , GetWindowLong(hwndPlugin, GWL_EXSTYLE) | WS_EX_LAYERED);
			// Make it transparent...
			BBSetLayeredWindowAttributes(hwndPlugin, NULL, (transparency) ? (unsigned char)alpha : 255, LWA_ALPHA);
		}else{
			// Remove the WS_EX_LAYERED atribute to the window, cos we in di slit!
			SetWindowLong(hwndPlugin, GWL_EXSTYLE, GetWindowLong(hwndPlugin, GWL_EXSTYLE) & ~WS_EX_LAYERED);
		}
	}
	InvalidateRect(hwndPlugin, NULL, false);
	ShowMyMenu(false);
}

//===========================================================================

bool ResizeMyWindow(int newWidth, int newHeight)
{
	if(newWidth != width || newHeight != height){
		width = newWidth;
		height = newHeight;
		if (width <= 10)  width  = 10;
		if (height <= 10) height = 10;
	}
	else
		return false;

	if(inSlit){
		SetWindowPos(hwndPlugin, 0, 0, 0, width, height, SWP_NOACTIVATE | SWP_NOZORDER | SWP_NOMOVE);
		SendMessage(hSlit, SLIT_UPDATE, NULL, NULL);
		return true;
	}else{
		SetWindowPos(hwndPlugin, 0, xpos, ypos, width, height, SWP_NOACTIVATE | SWP_NOZORDER);
		return true;
	}
}

//===========================================================================

text_line *GetTextFromFeed()
{
	if(download_thread_semaphore)
		return NULL; // Don't update if we're already in the update thread.

	// We maybe called on a timer, so clear out the old lines.
	delete_text_line_list();
	ClearToolTips();

	// Try and get the file off the internet.
	unsigned int download_thread_id = 0;

	download_thread_semaphore = CreateSemaphore(NULL, 1, 1, NULL);

	// Start up a thread to get the RSS file.
	_beginthreadex(NULL, 0, &download_thread, NULL, 0, &download_thread_id);

	add_single_entry(connecting);

	return text_line_head;
}

bool GetRssText()
{
	text_line *copy = GetTextFromFeed();

	if(copy){
		clear_list();
		copy_list(copy);
	}

	if(line_list_head && line_list_tail)
		return TRUE;
	return FALSE;
}

void add_single_entry(char *entry)
{
	int entry_length = strlen(entry);

	text_line_head = new text_line;
	memset(text_line_head, 0, sizeof(text_line));
	text_line_tail = text_line_head;
	text_line_tail->title = create_string(entry, entry_length);
	text_line_tail->text_length = entry_length;
}

void thread_wrapper()
{
	// Get the semaphore.
	WaitForSingleObject(download_thread_semaphore, 0);

	char *rss_buffer = NULL;
	if(feedUrl)
		rss_buffer = get_http_file(feedUrl);

	if(rss_buffer){
		if(!abort_download){
			delete_text_line_list();
			ClearToolTips();
			parse_lines_from_file(rss_buffer);

			time_t systemTime;
			time(&systemTime);
			struct tm *ltp = localtime(&systemTime);
			strftime(updateTime, sizeof(updateTime), "%m/%d %H:%M", ltp);
		}
		free(rss_buffer);
	}else{
		add_single_entry(connect_error);
	}

	// Tell the main window we're done here.
	if(!abort_download)
		PostMessage(hwndPlugin, WM_TEXTUPDATED, 0, (LPARAM)text_line_head);

	ReleaseSemaphore(download_thread_semaphore, 1, NULL);
	CloseHandle(download_thread_semaphore);
	download_thread_semaphore = NULL;
}

void parse_lines_from_file(char *file_buffer)
{
	BOOL got_item = FALSE;
	long current_item = 0;
	text_line *last_title = NULL;
	char *url = NULL;
	char *desc = NULL;
	char *date = NULL;

	int text_index = 0;
	char *token = strtok(file_buffer, "><"), *dest = (char *)file_buffer;
	while(token)
	{
		BOOL append = FALSE;
		BOOL is_title = FALSE;
		BOOL is_description = FALSE;
		BOOL is_link = FALSE;

		token = strtok(NULL, "><");
		if(!token)
			// No more tokens, so we're done here.
			break;

		if(*token==0xa)
			continue;

		strlwr(token);

		// We use strncmp as the original XML tag may be '<item blah blah blah', meaning
		// a basic string compare will fail.
		if(!strncmp(token, "item", 4)){
			// Try and associate a url with the RSS 'title'.
			if(url && last_title && last_title==text_line_head){
				last_title->link = url;
				url = NULL;
			}
			if(url){
				delete url;
				url = NULL;
			}

			if(desc && last_title && last_title==text_line_head){
				last_title->description = desc;
				desc = NULL;
			}
			if(desc){
				delete desc;
				desc = NULL;
			}

			if(date && last_title && last_title==text_line_head){
				last_title->date = date;
				date = NULL;
			}
			if(date){
				delete date;
				date = NULL;
			}
			// We only add title and description tags when we're inside an <item> tag.
			got_item = TRUE;
			last_title = NULL;
		}else if(!strncmp(token, "/item", 5)){
			// Hit the end of the </item> tag, so stop adding stuff.
			got_item = FALSE;
			if(last_title){
				// We added a title, so see if we have a URL to append to it.
				if(url){
					last_title->link = url;
				}
				url = NULL;

				if(desc){
					last_title->description = desc;
				}
				desc = NULL;

				if(date){
					last_title->date = date;
				}
				date = NULL;
			}else{
				// No title, so delete URL if we found one.
				if(url){
					delete url;
					url = NULL;
				}

				if(desc){
					delete desc;
					desc = NULL;
				}

				if(date){
					delete date;
					date = NULL;
				}
			}
			current_item++;
			if(item_count && current_item > item_count){
				// We've specified that we only want a certain amount of
				// items, so stop parsing the RSS feed and break out.
			break;
			}
		}
		else if(!strncmp(token, "link", 4) || !strncmp(token, "guid",4)){
			append = TRUE;
			is_link = TRUE;

		}else if(!strncmp(token, "description", 11)){
			if(got_item){
				append = TRUE;
				is_description = TRUE;
			}
		}else if(!strncmp(token, "title", 5)){
			if(!text_index || got_item){
				if(!text_index)
					current_item++;
				append = TRUE;
				is_title = TRUE;
			}
		}else if(!strncmp(token, "pubdate", 7) || !strncmp(token, "dc:date",7)){
			if(token && !date && got_item){
				token = strtok(NULL, "><");
				int length = strlen(token);
				date = new char[length + 1];
				memset(date, 0, length + 1);

				tidy_token(token, date, length);
			}
		}
		if(append){
			// Get the next token - the content (hopefully).
			token = strtok(NULL, "><");

			int text_length = strlen(token), scan = 0, length = 0;

			if(!strnicmp(token, "![CDATA[", 8)){
				token = token + 8;
				text_length = text_length - 8;
				text_length = text_length - 2;
			}

			char *pointer = token;
			while(scan <= text_length){
				if(scan != text_length) //pointer!=0xa && *pointer !=0xd &&
				{
					length++;
					pointer++;
					scan++;
				}else{
					if(is_title){
					// Add a line structure to the list.
					if(!text_line_head){
						text_line_head = new text_line;
						text_line_tail = text_line_head;
					}else{
						text_line *new_line = new text_line;
						text_line_tail->next_line = new_line;
						text_line_tail = new_line;
					}
					memset(text_line_tail, 0, sizeof(text_line));
					text_line_tail->index = text_index;
					text_line_tail->link = NULL;
					text_line_tail->description = NULL;
					text_line_tail->date = NULL;

					// Store this away if it's a title - we may need to
					// store a future URL here...
					if(!last_title && is_title)
						last_title = text_line_tail;

					// We take a copy of the text.
					if(length){
						text_line_tail->title = new char[length + 1];
						memset(text_line_tail->title, 0, length + 1);
						tidy_token(pointer - length, text_line_tail->title, length);
                                        }else{
						text_line_tail->title = NULL;
                                        }
					text_line_tail->text_length = length;
					text_index++;
                                        }

					if(is_description){
					if(length){
						desc = new char[length + 1];
						memset(desc, 0, length + 1);
						tidy_token(pointer - length, desc, length);
                                        }else{
						desc = NULL;
                                        }
                                        }

					if(is_link){
					if(length){
						url = new char[length + 1];
						memset(url, 0, length + 1);
						tidy_token(pointer - length, url, length);
                                        }else{
						url = NULL;
                                        }
                                        }

					if(*pointer =0xd){
						pointer++;
						scan++;
					}
					if(*pointer =0xa){
						pointer++;
						scan++;
					}
					length = 0;
				}
			}

		}
	}
	if(text_line_tail)
		text_line_tail->next_line = NULL;

	Item_count = text_index;
}

void tidy_token(char *source, char *dest, int length)
{
	// Need to parse out control codes from the URL.
	char *source_pointer = source;
	char *dest_pointer = dest;
	char *end = source + length;

	while(source_pointer < end)
	{
		switch(*source_pointer)
		{
			case 0xd:
			case 0xa:
				break;

			case 0x26:
			{
				if(!strnicmp(source_pointer, "&amp;", 5)){
					source_pointer = source_pointer + 5;
					if(!strnicmp(source_pointer, "quot;", 5)){
						*dest_pointer = 0x22; // "
						dest_pointer++;
						// source_poiner will be incremented later on
						source_pointer = source_pointer + 4;
					}else if(!strnicmp(source_pointer, "lt;", 2)){
						*dest_pointer = 0x3c; // <
						dest_pointer++;
						// source_poiner will be incremented later on
						source_pointer = source_pointer + 2;
					}else if(!strnicmp(source_pointer, "gt;", 2)){
						*dest_pointer = 0x3e; // >
						dest_pointer++;
						// source_poiner will be incremented later on
						source_pointer = source_pointer + 2;
					}else if(*source_pointer==0x23){
						// We need to convert the value after the hash into
						// an ASCII code.
						int add = 0;
						char buff[8]={0};
						// Skip '#'
						source_pointer++;
						while(*source_pointer!=0x3b && add < 3){
							buff[add] = *source_pointer;
							add++;
							source_pointer++;
						}
						int ascii = atoi(buff);
						if(ascii >= 20 && ascii <= 255){
							*dest_pointer = ascii;
							dest_pointer++;
						}
					}else{
						*dest_pointer = 0x26; // &
						dest_pointer++;
						source_pointer--;
					}
					break;
				}
				if(!strnicmp(source_pointer, "&quot;", 5)){
					*dest_pointer = 0x22; // "
					dest_pointer++;
					// source_poiner will be incremented later on
					source_pointer = source_pointer + 4;
				}
				// Make this into a function, as I'm repeating code here?
				if(!strnicmp(source_pointer, "&#", 2)){
					int add = 0;
					char buff[8]={0};
					// Skip '&#'
					source_pointer++;
					source_pointer++;
					while(*source_pointer!=0x3b && add < 3){
						buff[add] = *source_pointer;
						add++;
						source_pointer++;
					}
					int ascii = atoi(buff);
					if(ascii >= 20 && ascii <= 255){
						*dest_pointer = ascii;
						dest_pointer++;
					}
					break;
				}
				// Fall through and just copy the ampersand as we don't handle it.
			}
			default:
				*dest_pointer = *source_pointer;
				dest_pointer++;
		}
		source_pointer++;
	}
}

void clear_list()
{
	text_line *line = line_list_head, *next = NULL;
	while(line)
	{
		next = line->next_line;
		delete line;
		line = next;
	}
	line_list_head = NULL;
	line_list_tail = NULL;
}

void copy_list(text_line *copy)
{
	while(copy)
	{
		text_line *new_copy = new text_line;
		memcpy(new_copy, copy, sizeof(text_line));

		if(!line_list_head){
			line_list_head = new_copy;
			line_list_tail = line_list_head;
		}else{
			line_list_tail->next_line = new_copy;
			line_list_tail = new_copy;
		}
		if(copy->title)
		{
			int length = strlen(copy->title);
			line_list_tail->title = new char[length + 1];
			memset(line_list_tail->title, 0, length +1);
			memcpy(line_list_tail->title, copy->title, length);
		}
		if(copy->link)
		{
			int length = strlen(copy->link);
			line_list_tail->link = new char[length + 1];
			memset(line_list_tail->link, 0, length +1);
			memcpy(line_list_tail->link, copy->link, length);
		}
		if(copy->description)
		{
			int length = strlen(copy->description);
			line_list_tail->description = new char[length + 1];
			memset(line_list_tail->description, 0, length +1);
			memcpy(line_list_tail->description, copy->description, length);
		}
		line_list_tail->next_line = NULL;
		copy = copy->next_line;
	}
}

//===========================================================================

char *get_http_file(char *url)
{
	char *server_name = NULL;
	char *file_name = NULL;

	// Split the url - should be in the form 'http://www.servername/file_toget'.
	// Get past the 'http'
	char *split = strchr(url, 0x3a);// ':'
	if(split)
		{
		split++;
		if(*split==0x2f) // '/'
			{
			split++;
			if(*split==0x2f) // '/'
				{
				split++;
				// We now have the server name.
				
				// Find the next forward slash to get the filename.
				char *slash = strchr(split, 0x2f);// '/'
				if(slash)
					{
					int server_length = slash - split;
					server_name = create_string(split, server_length);
//					server_name = new char[server_length + 1];
//					memset(server_name, 0, server_length + 1);
//					memcpy(server_name, split, server_length);
					int file_name_length = (url + strlen(url)) - slash;
					file_name = create_string(slash, file_name_length);
//					memset(file_name, 0, file_name_length + 1);
//					memcpy(file_name, slash, file_name_length);
					}
				}
			}
		}

	if(!file_name || !server_name)
		{
		delete file_name;
		delete server_name;
		return NULL;
		}

	// Use inet_addr() to determine if we're dealing with a name or an IP address.
	IN_ADDR  iaHost;
	LPHOSTENT lpHostEntry;

	iaHost.s_addr = inet_addr(server_name);
	if(iaHost.s_addr == INADDR_NONE)
		{
		// Wasn't an IP address string, assume it is a name.
		lpHostEntry = gethostbyname(server_name);
		}
	else
		{
		// It was a valid IP address string.
		lpHostEntry = gethostbyaddr((const char *)&iaHost, sizeof(struct in_addr), AF_INET);
		}

	if(!lpHostEntry)
		// Couldn't get an IP address, so bail out.
		return NULL;

	// Create a TCP/IP stream socket.
	SOCKET url_socket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
	if(url_socket == INVALID_SOCKET)
		// Couldn't create a socket.
		return NULL;

	// Find the port number for the HTTP service on TCP.
	LPSERVENT lpServEnt;
	SOCKADDR_IN saServer;

	lpServEnt = getservbyname("http", "tcp");
// We always go for port 80...
//	if(!lpServEnt)
		// Set a default port.
//		saServer.sin_port = htons(80);
//	else
//		saServer.sin_port = lpServEnt->s_port;

	// Fill in the rest of the server address structure.
	saServer.sin_family = AF_INET;
	saServer.sin_addr = *((LPIN_ADDR)*lpHostEntry->h_addr_list);
	saServer.sin_port = htons(80);

	// Connect the socket.
	int result = connect(url_socket, (LPSOCKADDR)&saServer, sizeof(SOCKADDR_IN));
	if(result == SOCKET_ERROR)
		{
		// Couldn't connect the socket!
		closesocket(url_socket);
		return NULL;
		}

	// Send a HTTP request to the server.
	send_http_request("GET ", url_socket);
	send_http_request(file_name, url_socket);
	send_http_request(" HTTP/1.0\r\n", url_socket);
	send_http_request("Accept: */*\r\n", url_socket);
	send_http_request("Host: ", url_socket);
	send_http_request(server_name, url_socket);
	send_http_request("\r\n", url_socket);
	send_http_request("\r\n", url_socket);

	if(result == SOCKET_ERROR)
		{
		closesocket(url_socket);
		return NULL;
		}

	int block_size = 4096, size = block_size, offset = 0;
	char *buffer = (char *)malloc(size), *buffer_pointer = buffer;

	// Receive the file contents and print to stdout
	while(!abort_download) // !!! need a better way to break out here
		{
		// Wait to receive, nRet = NumberOfBytesReceived
		result = recv(url_socket, buffer_pointer, block_size, 0);
		if(result == SOCKET_ERROR)
			break;

		// Did the server close the connection?
		if(!result)
			break;

		size = size + block_size;
		buffer = (char *)realloc(buffer, size);
		// NULL terminate the buffer.
		offset = offset + result;
		*(buffer + offset) = 0;
		buffer_pointer = buffer + offset;
		}
	
	if(server_name)
		delete server_name;
	if(file_name)
		delete file_name;

	closesocket(url_socket);
	return buffer;
}

int send_http_request(char *request, SOCKET socket)
{
	int result = send(socket, request, strlen(request), 0);
	return result;
}

BOOL check_label_click(POINT click, text_line **line)
{
	if(!line_list_head)
	{
		// We only have the one piece of text, so we must have double clicked on it.
		*line = NULL;
		return TRUE;
	}

	// We have multiple lines of text, so see which one we clicked.
	text_line *check = line_list_head;
       	while(check)
	{
		if(PtInRect(&check->text_rect, click))
		{
			// Click is inside the bounding rectangle for our text, so we've found
			// our line.
			*line = check;
			return TRUE;
		}
		check = check->next_line;
	}
	return FALSE;
}

BOOL ClickLabel(text_line *clicked)
{
	// This will open the RSS link in a browser window.
	if(clicked && clicked->link)
	{
		if(0 == *browserPath) BBExecute(GetDesktopWindow(), NULL, clicked->link, NULL, NULL, SW_SHOWNORMAL, false);
		else BBExecute(GetDesktopWindow(), NULL, browserPath, clicked->link, NULL, SW_SHOWNORMAL, false);
		return TRUE;
	}
	return FALSE;
}

//===========================================================================

BOOL init_winsock()
{
	// Try and open winsock.
	WORD wVersionRequested = MAKEWORD(1,1);
	WSADATA wsaData;
	int result = WSAStartup(wVersionRequested, &wsaData);
	if(result || wsaData.wVersion!=wVersionRequested)
	{
		// Initialisation failed.
		WSACleanup();
		got_winsock = FALSE;
	}
	else
		got_winsock = TRUE;

	return got_winsock;
}

void close_winsock()
{
	if(got_winsock)
		WSACleanup();
}

BOOL WINAPI DllMain(HINSTANCE hInstDLL, DWORD fdwReason, LPVOID lpvReserved)
{
	switch(fdwReason)
	{
		case DLL_PROCESS_ATTACH:
		{
			if(!init_winsock())
			{
				MessageBox(NULL, "Couldn't open Winsock!", "Error!", MB_ICONEXCLAMATION|MB_OK);
				return FALSE;
			}
			return TRUE;
		}
		break;

		case DLL_PROCESS_DETACH:
		{
			close_winsock();
		}
		break;
	}
	return TRUE;
}

//===========================================================================

// Function which creates a thread to handle the HTTP download.
unsigned int __stdcall download_thread(LPVOID thread_param)
{
	thread_wrapper();
	_endthreadex(0);
	return 0;
}

//===========================================================================

void track_mouse_event(HWND hwnd)
{
#if 0
	TRACKMOUSEEVENT tme;
	tme.cbSize  = sizeof(tme);
	tme.dwFlags = TME_LEAVE;
	tme.hwndTrack = hwnd;
	tme.dwHoverTime = HOVER_DEFAULT;
	_TrackMouseEvent(&tme); // commctrl api
#else
	SetTimer(hwnd, TRACKMOUSE_TIMER, 80, NULL);
#endif
}

HWND window_under_mouse(void)
{
	POINT pt; GetCursorPos(&pt);
	HWND hwnd = WindowFromPoint(pt);
	return hwnd;
}

//===========================================================================
//get_monitor - multimon api, win 9x compatible

static HMONITOR (WINAPI *pMonitorFromWindow)(HWND hwnd, DWORD dwFlags);
static BOOL     (WINAPI *pGetMonitorInfoA)(HMONITOR hMonitor, LPMONITORINFO lpmi);

HMONITOR get_monitor(HWND hw)
{
	if (NULL == pMonitorFromWindow)
	{
		HMODULE hDll = GetModuleHandle("USER32");
		*(FARPROC*)&pMonitorFromWindow = GetProcAddress(hDll, "MonitorFromWindow" );
		*(FARPROC*)&pGetMonitorInfoA   = GetProcAddress(hDll, "GetMonitorInfoA"   );
		if (NULL == pMonitorFromWindow)
			*(int*)&pMonitorFromWindow = 1;
	}

	if (*(DWORD*)&pMonitorFromWindow <= 1)
		return NULL;

	return pMonitorFromWindow(hw, MONITOR_DEFAULTTOPRIMARY);
}

//===========================================================================
//get_mon_rect

void get_mon_rect(HMONITOR hMon, RECT *r)
{
	if (hMon)
	{
		MONITORINFO mi;
		mi.cbSize = sizeof(mi);
		if (pGetMonitorInfoA(hMon, &mi))
		{
			*r = mi.rcMonitor;
			//*r = mi.rcWork;
			return;
		}
	}
	//SystemParametersInfo(SPI_GETWORKAREA, 0, r, 0);
	r->top = r->left = 0;
	r->right = GetSystemMetrics(SM_CXSCREEN);
	r->bottom = GetSystemMetrics(SM_CYSCREEN);
}

//===========================================================================

bool window_test_autohide(POINT *pt)
{
	if (false == autoHide || inSlit)
		return false;

	int x = xpos;
	int y = ypos;

	RECT scrn;
	get_mon_rect(get_monitor(hwndPlugin), &scrn);

	if (x == scrn.left) x = 1-width;
	else
	if (x == scrn.right - width) x = scrn.right - 1;
	else
	if (y == scrn.top) y = 1 - height;
	else
	if (y == scrn.bottom - height) y = scrn.bottom - 1;
	else
		return false;

	if (pt && is_autohidden)
		pt->x = x, pt->y = y;

	return true;
}

//===========================================================================
