/*
 ============================================================================

  This program is free software, released under the GNU General Public License
  (GPL version 2 or later).

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  for more details.

  http://www.fsf.org/licenses/gpl.html

 ============================================================================
*/

#ifdef _MSC_VER
  #ifdef BBOPT_STACKDUMP
    #define TRY if (1)
    #define EXCEPT if(0)
  #else
    #define TRY _try
    #define EXCEPT _except(1)
  #endif
  #define stricmp _stricmp
  #define strnicmp _strnicmp
  #define memicmp _memicmp
  #define strlwr _strlwr
  #define strupr _strupr
#else
  #undef BBOPT_STACKDUMP
#endif

#ifndef __BBRSSREADER_H
#define __BBRSSREADER_H

#ifndef _WIN32_WINNT
#define _WIN32_WINNT 0x0501
#endif

#include <windows.h>
#include <shlwapi.h>
#include <commctrl.h>
#include <process.h> // For thread creation.
#include <tchar.h>
#include <time.h>
#include "BBApi.h"

#ifndef SLIT_ADD
#define SLIT_ADD 11001
#endif

#ifndef SLIT_REMOVE
#define SLIT_REMOVE 11002
#endif

#ifndef SLIT_UPDATE
#define SLIT_UPDATE 11003
#endif

#ifndef WS_EX_LAYERED
#define WS_EX_LAYERED	0x00080000
#define LWA_COLORKEY	0x00000001
#define LWA_ALPHA	0x00000002
#endif

#define GETTEXT_TIMER 1
#define TRACKMOUSE_TIMER 2
#define AUTOHIDE_TIMER 3
#define REDRAW_TIMER 4

#define AUTOHIDE_DELAY 300

#define WM_TEXTUPDATED	WM_USER + 100

//===========================================================================

HINSTANCE hInstance;
HWND hwndPlugin, hwndBlackbox;

bool inSlit = false;
HWND hSlit = NULL;

int msgs[] = {BB_RECONFIGURE, BB_REDRAWGUI, BB_BROADCAST, 0};

char rcpath[MAX_PATH];
char stylepath[MAX_PATH];

int xpos, ypos;
int width, height;
int alpha;
int fontHeight;
int styleType;
bool alwaysOnTop;
bool snapWindow;
bool transparency;
bool pluginToggle;
bool showBorder;
bool allowtip;
bool lockPos;

DWORD justify;
bool usingWin2kXP;
OSVERSIONINFO  osvinfo;

HDC hdc;
HGDIOBJ otherfont;
HBITMAP bufbmp;
HFONT hFont;

char fontFace[256];
int fontSize;
COLORREF fontColor;

void ShowMyMenu(bool popup);
void SetWindowModes(void);
void OnPaint(HWND hwnd);
StyleItem myStyleItem, refStyleItem, tbStyleItem;
int bevelWidth;
int borderWidth;
COLORREF borderColor;

Menu *myMenu, *configSubmenu, *settingsSubmenu,  *styleSubmenu, *justifySubmenu, *fontSubmenu, *rssSubmenu,
*favSubmenu, *encodeSubmenu, *highlightSubmenu, *browserSubmenu, *tooltipSubmenu, *offsetSubmenu, *hMenu;

char displayString[MAX_LINE_LENGTH];
char *szTemp;

int ScreenWidth;
int ScreenHeight;

int newWidth;
int newHeight;

//===========================================================================

LRESULT CALLBACK WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);
BOOL WINAPI BBSetLayeredWindowAttributes(HWND hwnd, COLORREF crKey, BYTE bAlpha, DWORD dwFlags);

void GetStyleSettings();
void ReadRCSettings();
void WriteRCSettings();

void ShowMyMenu();

//===========================================================================

#define m_alloc(n) malloc(n)
#define c_alloc(n) calloc(1,n)
#define m_free(v) free(v)

HWND hToolTips;
void SetToolTip(RECT *tipRect, char *tipText);
void ClearToolTips(void);
void SetAllowTip(bool);

bool ResizeMyWindow(int newWidth, int newHeight);
bool useRefFont;

bool window_test_autohide(POINT *pt);
bool autoHide;
bool is_autohidden;

//===========================================================================

struct text_line
{
	char *title;
	char *link;
	char *description;
	char *date;
	int text_length;
	int index;
	RECT text_rect;
	struct text_line *next_line;
};

text_line *text_line_head;
text_line *text_line_tail;

void delete_text_line_list()
{
	text_line *line = text_line_head;
	while(line)
	{
		text_line *next = line->next_line;
		delete line->title;
		delete line->link;
		delete line->description;
		delete line->date;
		delete line;
		line = next;
	}
	text_line_head = NULL;
	text_line_tail = NULL;
};

char *create_string(char *source, int length)
{
	if(!length && source)
		length = strlen(source);

	if(!length)
		return NULL;

	char *new_string = new char[length + 1];
	if(new_string)
	{
	       	memset(new_string, 0, length + 1);
		memcpy(new_string, source, length);
	}
	return new_string;
};

text_line *line_list_head; // List of text lines to draw.
text_line *line_list_tail;

bool GetRssText();
void clear_list();
void copy_list(text_line *copy);
bool CheckLabelClick(POINT click, text_line **line);

BOOL abort_download;
int item_count;
HANDLE download_thread_semaphore;

void add_single_entry(char *entry);
void tidy_token(char *source, char *dest, int length);
void parse_lines_from_file(char *file_buffer);

static BOOL got_winsock = TRUE;
char connecting[] = "Connecting...";
char connect_error[] = "Couldn't connect!";

unsigned int __stdcall download_thread(LPVOID thread_param);
void thread_wrapper();
char rss_buffer[1024*30];
int send_http_request(char *request, SOCKET socket);
char * get_http_file(char *url);

BOOL check_label_click(POINT click, text_line **line);
BOOL ClickLabel(text_line *clicked);

char feedUrl[256];
char browserPath[256];
char updateTime[20];

bool showDesc;
bool showDate;
bool abbreviatedDate;
bool datePosHead;
bool wordWrap;

bool showItemCounts;
bool showUpdateTime;

int text_draw_y;
int padding;
int title_offset;
int desc_offset;
int updateInterval;

int Item_count;

char *Utf8ToAnsi(char *utf8);

bool mouse_over;
void track_mouse_event(HWND hwnd);
HWND window_under_mouse(void);

struct favorite
{
	char title[256];
	char url[256];
	struct favorite *next;
};

favorite *favorite_head;
favorite *favorite_tail;

void delete_favorite_list()
{
	favorite *fav = favorite_head;
	while(fav)
	{
		favorite *next = fav->next;
		delete fav;
		fav = next;
	}
	favorite_head = NULL;
	favorite_tail = NULL;
};

int numFavorites;

DWORD JustifyStyle[] =
{
	DT_LEFT,
	DT_CENTER,
	DT_RIGHT
};

enum
{
	LEFT,
	CENTER,
	RIGHT
};

int encoding;

enum
{
	UTF8 = 1
};

int highlight;

enum
{
	MOUSEOVER = 1,
	ALL
};

bool hilightBorder;

bool tipShowTitle;
int tipWidth;
int tipHideDelay;


//===========================================================================

extern "C"
{
	__declspec(dllexport) LPCSTR pluginInfo(int field);
	__declspec(dllexport) void endPlugin(HINSTANCE hMainInstance);
	__declspec(dllexport) int beginPlugin(HINSTANCE hMainInstance);
	__declspec(dllexport) int beginSlitPlugin(HINSTANCE hMainInstance, HWND hBBSlit);
	__declspec(dllexport) int beginPluginEx(HINSTANCE hMainInstance, HWND hBBSlit);
}

//===========================================================================

#endif
