/*
 ============================================================================
 BBSpectrum - A simple fft spectrum analyser for BB4win
 ============================================================================
 Created by MADant aka Kaloth - http://www.geocities.com/madant8
 ============================================================================

  Blackbox for Windows is free software and so is this plugin
  released under the GNU General Public License (GPL version 2 or later).
  
  You may mess with it as you wish! (But keep it open source please :))

 ============================================================================
*/

#ifndef _WIN32_WINNT
#define _WIN32_WINNT 0x0500
#endif

#ifdef __GNUC__
#define _WIN32_IE 0x0500
#define GetBlackboxPath _GetBlackboxPath
#define DLL_EXPORT //__declspec(dllexport)
#define LWA_ALPHA 2
#endif

#ifdef __BORLANDC__
#define DLL_EXPORT
#endif

#include "../../includes/BBApi.h"
#include <stdlib.h>
#include <mmsystem.h>
#include <windows.h>
#include <stdio.h>
#include <math.h>

#pragma comment (lib, "msimg32.lib")

#define STREQ(str1, str2) (strcmp((str1), (str2)) == 0)

#define POS_ONLY(x) ((x)>0?(x):0)

extern "C"
{
	DLL_EXPORT int beginSlitPlugin(HINSTANCE hMainInstance, HWND hBBSlit);
	DLL_EXPORT int beginPluginEx(HINSTANCE hMainInstance, HWND hBBSlit);
	DLL_EXPORT int beginPlugin(HINSTANCE hMainInstance);
	DLL_EXPORT void endPlugin(HINSTANCE hMainInstance);
	DLL_EXPORT LPCSTR pluginInfo(int field);
}

LPSTR szAppName         = "BBSpectrum";          // The name of our window class, etc.
LPSTR szVersion         = "BBSpectrum v0.53b";   // Used in MessageBox titlebars
LPSTR szInfoVersion     = "v0.53b";
LPSTR szInfoAuthor      = "Kaloth";
LPSTR szInfoRelDate     = "2006-02-12";
LPSTR szInfoLink        = "http://www.geocities.com/madant8";
LPSTR szInfoEmail       = "mulletwarriorextreem@hotmail.com";
char updateURL[MAX_PATH];

#define PLUGIN_UPDATE_URL 8

//===========================================================================
HINSTANCE hInstance;
HWND hwndPlugin, hwndBlackbox;
OSVERSIONINFO  osvinfo;

char rcpath[MAX_PATH];                                  

int xpos, ypos;
int width, height;

bool alwaysOnTop;
bool snapWindow;
bool hidden;
bool pluginToggle;        
bool transparencyEnabled;
bool transback;
bool showBorder;
bool startInRecord;
int transparencyAlpha;

int wave_left, wave_right, freqsize;

HDC buf;
HBITMAP bufbmp;
HGDIOBJ otherbmp;

bool usingWin2kXP;

StyleItem myStyleItem;

//blend stuff...
BLENDFUNCTION blend;
bool motionblur;

int bevelWidth;
int borderWidth;
COLORREF borderColor;
COLORREF ScopeColor;
COLORREF PeakColor;
COLORREF TRANSCOLOUR = RGB(255,0,230);

int tick_count;

#define MAX_LINE_LENGTH 100
char ScopeColorTxt[MAX_LINE_LENGTH], PeakColorTxt[MAX_LINE_LENGTH];
char tabPos[3], scopeType[2];

void GetStyleSettings();
void ReadRCSettings();
void WriteRCSettings();

HFONT hFont;

//Sound stuff...
//===========================================================================

// Handle to the WAVE In Device
HWAVEIN WaveInHandle;

// Variable used to indicate whether we are in record
BOOL InRecord = false;

// WAVEHDR for recording.
WAVEHDR WaveHeader;

// variable to signify wether all the WAVEHDRs have been returned by the snd card driver.
int DoneAll;

// Slit stuff...
bool inSlit;
HWND hSlit;

// misc stuff...
int buffsize, avg;
int wait, KHz, bit;
unsigned char dat[512];
float peak[512];
char peaktype[32];
int maxheight;
float peakdrop;

bool peaks;

#include <complex.h>
#define pi 3.14159265358979323846

void fft(double *data, long size);
//get messages from audio driver.
void CALLBACK waveInProc(HWAVEIN hwi, UINT uMsg, DWORD dwInstance, DWORD dwParam1, DWORD dwParam2)
{
		// Figure out which message was sent
		switch (uMsg)
		{
			// A buffer has been filled by the driver
			case WIM_DATA:
			{
				if (((WAVEHDR *)dwParam1)->dwBytesRecorded)
				{
					int size = ((WAVEHDR *)dwParam1)->dwBytesRecorded;
					char *data = ((WAVEHDR *)dwParam1)->lpData;
					int n;
					double *tp;
					if(size > 512){size = 512;}
					double dataf[512];
					
					// Averaging Filter...
					/*n = 0;
					dataf[n] = (double)(data[n] + data[n+1]) / 2;
					for(n = 1; n < size - 1; n++){
						dataf[n] = (double)(data[n-1] + data[n] + data[n+1]) / 3;
					}
					dataf[n] = (double)(data[n-1] + data[n]) / 2;*/
					
					for(n = 0; n < size; n++){
						dataf[n] = (double)abs(data[n]);
					}
					
					// Hamming Window & data conversion...
					/*int temp = 2*pi*n;
					tp = dataf;
					for(n = 0; n < size; n++){
						*tp = *tp * (0.54 - 0.46 * cos(temp/size));
						tp++;
					}*/
					
					// FFT (Time/Amplitude -> Frequency/Amplitude)
					fft(dataf, size>>1);
					
					float scale;
					if(strcmp(scopeType, "h") == 0){
						scale = ((float)height/(float)maxheight);
					}else{
						scale = ((float)width/(float)maxheight);
					}
					
					// Delay Filter...
					unsigned int x;
					for(n = wave_left; n < wave_right; n++){
						x = abs(dataf[n]) * scale;
						if(x > dat[n]){
							dat[n] = x;
						}else{
							dat[n] = ((dat[n] * 3) + x) >> 2;
						}
					}
					
					// Peak Markers...
					if(peaks){
						// Modify peaks...
						for(n = wave_left; n < wave_right; n++){
							if(peak[n] > dat[n]){
								peak[n] -= peakdrop;
								continue;
							}
							peak[n] = dat[n];
						}
					}
					
					if((GetTickCount() - tick_count) > wait){
						tick_count = GetTickCount();
						InvalidateRect(hwndPlugin, NULL, false); // Force the window to redraw.
					}
				}

				// Are we still recording?
				if (InRecord){
					// Recording in progress, so requeue buffer...
					waveInAddBuffer(WaveInHandle, (WAVEHDR *)dwParam1, sizeof(WAVEHDR));
				}

				// We aren't recording, so another WAVEHDR has been returned to us after recording has stopped.
				// When we get all of them back, DoneAll will be equal to how many WAVEHDRs we queued
				else{
					++DoneAll;
				}
			}

			// Our main thread is opening the WAVE device
			case WIM_OPEN:
			{
				// Re-initialize 'DoneAll'
				//DoneAll = 0;
				//SendMessage(GetBBWnd(), BB_SETTOOLBARLABEL, 0, (LPARAM)"Wave Device Open");
			}

			// Our main thread is closing the WAVE device */
			case WIM_CLOSE:
			{
				// Terminate this thread (by return'ing) */
				//SendMessage(GetBBWnd(), BB_SETTOOLBARLABEL, 0, (LPARAM)"Wave Device Closed");
				break;
			}
		}
}

//===========================================================================
// like the name says...

void set_window_modes(void)
{
	if (hidden && pluginToggle)
	{
		ShowWindow(hwndPlugin, SW_HIDE);
	}
	else
	{
		if(usingWin2kXP){
			if(!inSlit){
				// Add the WS_EX_LAYERED atribute to the window.
				SetWindowLong(hwndPlugin, GWL_EXSTYLE , GetWindowLong(hwndPlugin, GWL_EXSTYLE) | WS_EX_LAYERED);
				// Make it transparent...
				SetLayeredWindowAttributes(	hwndPlugin,
								TRANSCOLOUR,
								(transparencyEnabled) ? (unsigned char)transparencyAlpha : 255,
								LWA_COLORKEY | LWA_ALPHA);
			}else{
				// Remove the WS_EX_LAYERED atribute to the window, cos we in di slit!
				SetWindowLong(hwndPlugin, GWL_EXSTYLE, GetWindowLong(hwndPlugin, GWL_EXSTYLE) & ~WS_EX_LAYERED);
			}
		}
		
		SetWindowPos(
			hwndPlugin,
			alwaysOnTop ? HWND_TOPMOST : HWND_NOTOPMOST,
			xpos, ypos, width, height, SWP_NOACTIVATE|SWP_SHOWWINDOW
			);
		
		InvalidateRect(hwndPlugin, NULL, false);
	}
}

int startRecording(){
	if(!InRecord){
		MMRESULT err;
		WAVEFORMATEX waveFormat;
		MIXERLINE mixerLine;
		unsigned long n, numSrc, thing;
	
		ZeroMemory(&WaveHeader, sizeof(WAVEHDR));
	
		// Initialize the WAVEFORMATEX*/
		waveFormat.wFormatTag = WAVE_FORMAT_PCM;
		waveFormat.nChannels = 1;
		waveFormat.nSamplesPerSec = KHz *1000;
		waveFormat.wBitsPerSample = bit;
		waveFormat.nBlockAlign = waveFormat.nChannels * (waveFormat.wBitsPerSample/8);
		waveFormat.nAvgBytesPerSec = waveFormat.nSamplesPerSec * waveFormat.nBlockAlign;
		waveFormat.cbSize = 0;
		
		err = waveInOpen(&WaveInHandle, WAVE_MAPPER, &waveFormat, (DWORD)waveInProc, 0, CALLBACK_FUNCTION);
		if(err){
			MessageBox(0, "Can't open WAVE In Device!", szVersion,
				MB_OK | MB_ICONERROR | MB_TOPMOST);
			return(1);
		}else{
			WaveHeader.dwBufferLength = buffsize;
			if ((WaveHeader.lpData = (char *)VirtualAlloc(NULL, WaveHeader.dwBufferLength, MEM_COMMIT, PAGE_READWRITE)) == NULL){
				MessageBox(0, "Can't allocate memory for WAVE buffer!", szVersion,
					MB_OK | MB_ICONERROR | MB_TOPMOST);
			}else{
				// Prepare the 2 WAVEHDR's
				if ((err = waveInPrepareHeader(WaveInHandle, &WaveHeader, sizeof(WAVEHDR)))){
					MessageBox(0, "Error preparing WAVEHDR!", szVersion,
						MB_OK | MB_ICONERROR | MB_TOPMOST);
				}else{
					// Queue first WAVEHDR (recording hasn't started yet)
					if ((err = waveInAddBuffer(WaveInHandle, &WaveHeader, sizeof(WAVEHDR)))){
						MessageBox(0, "Error queueing WAVEHDR!", szVersion,
							MB_OK | MB_ICONERROR | MB_TOPMOST);
					}else{
						//Start recording.
						DoneAll = 0;
						InRecord = TRUE;
						if ((err = waveInStart(WaveInHandle))){
							MessageBox(0, "Error starting record!", szVersion,
								MB_OK | MB_ICONERROR | MB_TOPMOST);
						}else{
							//SendMessage(GetBBWnd(), BB_SETTOOLBARLABEL, 0, (LPARAM)"Recording has started.");
						}
					}
				}
			}
		}
		return 0;
	}
	return 1;
}

int stopRecording(){
	if(InRecord){
		MMRESULT err;
		
		InRecord = false;
		
		waveInReset(WaveInHandle);
		
		// wait for mixer to return header.
		while (DoneAll < 1){
			Sleep(100);
		}
		
		// Unprepare WAVEHDR
		if ((err = waveInPrepareHeader(WaveInHandle, &WaveHeader, sizeof(WAVEHDR)))){
			MessageBox(0, "Error unpreparing WAVEHDR!", szVersion,
					MB_OK | MB_ICONERROR | MB_TOPMOST);
		}
		
		// Close the WAVE In device
		err = waveInClose(WaveInHandle);
		if(err){
			MessageBox(0, "Can't close WAVE In Device!", szVersion,
					MB_OK | MB_ICONERROR | MB_TOPMOST);
		}
		
		// Free any memory allocated for our buffers
		if (WaveHeader.lpData) VirtualFree(WaveHeader.lpData, 0, MEM_RELEASE);
		return 0;
	}
	return 1;
}

int beginSlitPlugin(HINSTANCE hMainInstance, HWND hBBSlit)
{
	inSlit = true;
	hSlit = hBBSlit;
	
	// Start the plugin like normal now..
	int res = beginPlugin(hMainInstance);
	
	// Are we to be in the Slit?
	if (inSlit && hSlit){
		// Yes, so Let's let BBSlit know.
		SendMessage(hSlit, SLIT_ADD, NULL, (LPARAM)hwndPlugin);
	}
	
	return res;
}

int beginPluginEx(HINSTANCE hMainInstance, HWND hBBSlit)
{
	return beginSlitPlugin(hMainInstance, hBBSlit);
}

//===========================================================================
int beginPlugin(HINSTANCE hPluginInstance)
{
	blend.BlendOp = AC_SRC_OVER;
	blend.BlendFlags = 0;
	blend.SourceConstantAlpha = 128;
	blend.AlphaFormat = NULL;
	
	tick_count = GetTickCount();
	
	ZeroMemory(peak, 512);
	
	DoneAll = 1;
	int i;
	for(i = 0; i <= buffsize; i++){
		dat[i] = 0;
	}
	
	LRESULT CALLBACK WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);
	WNDCLASS wc;
	
	ZeroMemory(&osvinfo, sizeof(osvinfo));
	osvinfo.dwOSVersionInfoSize = sizeof (osvinfo);
	GetVersionEx(&osvinfo);
	usingWin2kXP =
		osvinfo.dwPlatformId == VER_PLATFORM_WIN32_NT
		&& osvinfo.dwMajorVersion > 4;

	hwndBlackbox = GetBBWnd();
	hInstance = hPluginInstance;

	// Register the window class...
	ZeroMemory(&wc,sizeof(wc));
	wc.lpfnWndProc = WndProc;           // our window procedure
	wc.hInstance = hPluginInstance;     // hInstance of .dll
	wc.lpszClassName = szAppName;       // our window class name
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.style = CS_DBLCLKS;

	if (!RegisterClass(&wc)) 
	{
		MessageBox(hwndBlackbox,
			"Error registering window class", szVersion,
				MB_OK | MB_ICONERROR | MB_TOPMOST);
		return 1;
	}

	// Get plugin and style settings...
	ReadRCSettings();
	GetStyleSettings();

	// Create the window...
	if (!CreateWindowEx(
		usingWin2kXP ? WS_EX_TOOLWINDOW | WS_EX_LAYERED : WS_EX_TOOLWINDOW, // window style
		szAppName,                      // our window class name
		NULL,                           // NULL -> does not show up in task manager!                
		WS_POPUP
		| WS_CLIPCHILDREN
		| WS_CLIPSIBLINGS
		| WS_VISIBLE,                   // window parameters
		xpos,                           // x position
		ypos,                           // y position
		width,                          // window width
		height,                         // window height
		NULL,                           // parent window
		NULL,                           // no menu
		hPluginInstance,                // hInstance of .dll
		NULL))
	{                          
		MessageBox(0,
			"Error creating window", szVersion,
				MB_OK | MB_ICONERROR | MB_TOPMOST);
		return 1;
	}
	
	InRecord = false;
	
	if(startInRecord){startRecording();}
	
	return(0);
}
                   
//===========================================================================

void endPlugin(HINSTANCE hPluginInstance)
{
	stopRecording();
	
	// Jump outa da slit!
	if(inSlit && hSlit){
		// We are in the slit, so lets unload and get out..
		SendMessage(hSlit, SLIT_REMOVE, NULL, (LPARAM)hwndPlugin);
		
		// Here you can move to where ever you want ;)
		SetWindowPos(hwndPlugin, NULL, xpos, ypos, 0, 0, SWP_NOZORDER | SWP_NOSIZE);
		inSlit = false;
	}
	
	// Destroy the window...
	DestroyWindow(hwndPlugin);

	// Unregister window class...
	UnregisterClass(szAppName, hPluginInstance);

	if (hFont){
		DeleteObject(hFont), hFont = NULL;
	}
}

//===========================================================================

LPCSTR pluginInfo(int field)
{
	// pluginInfo is used by Blackbox for Windows to fetch information about
	// a particular plugin. At the moment this information is simply displayed
	// in an "About loaded plugins" MessageBox, but later on this will be
	// expanded into a more advanced plugin handling system. Stay tuned! :)
	switch (field)
	{
		case PLUGIN_NAME:
			return szAppName;       // Plugin name
		case PLUGIN_VERSION:
			return szInfoVersion;   // Plugin version
		case PLUGIN_AUTHOR:
			return szInfoAuthor;    // Author
		case PLUGIN_RELEASE:
			return szInfoRelDate;   // Release date, preferably in yyyy-mm-dd format
		case PLUGIN_LINK:
			return szInfoLink;      // Link to author's website
		case PLUGIN_EMAIL:
			return szInfoEmail;     // Author's email
		case PLUGIN_UPDATE_URL:
			return updateURL;	// Dev's online update meta file
		case PLUGIN_BROAMS:
		{
			return
			"@BBSpectrum SetStyle 1"
			"@BBSpectrum SetStyle 2"
			"@BBSpectrum SetStyle 3"
			"@BBSpectrum SetStyle 4";
		}
		default:
			return szVersion;       // Fallback: Plugin name + version, e.g. "MyPlugin 1.0"
	}
}

//===========================================================================

int powt(int n){
	if(n <= 0)return 1;
	return 2 * powt(n-1);
}

void ShowMyMenu(bool popup)
{
	char buff1[50], buff2[5];
	int n, t;
	Menu *pMenu, *pSub, *tabPosMenu, *scopeTypeMenu, *KHzMenu, *peakdropmenu, *maxheightmenu, *stylemenu;
	Menu *BitMenu, *BufferMenu, *FPSMenu, *BlurMenu, *wave_leftMenu, *wave_rightMenu, *peakmenu;

	// We create the main plugin menu.
	// Each menu or submenu has an unique ID-string, which identifies it
	// for later updates.

	pMenu = MakeNamedMenu("BBSpectrumPlugin", "BBSpectrum_IDMain", popup);

	// --------------------------------------------------------------
	// ... and the configuration options menu, note the "internal" bro@ms!...
	pSub = MakeNamedMenu("Configuration", "BBSpectrum_IDConfig", popup);

	MakeMenuItem(pSub, "Always On Top", "@BBSpectrumInternal AlwaysOnTop", alwaysOnTop);
	MakeMenuItem(pSub, "Docked to Slit", "@BBSpectrumInternal SlitToggle", inSlit);
	MakeMenuItem(pSub, "Snap Window To Edge", "@BBSpectrumInternal SnapWindowToEdge", snapWindow);
	MakeMenuItem(pSub, "Toggle With Plugins", "@BBSpectrumInternal ToggleWithPlugins", pluginToggle);
	MakeMenuItem(pSub, "Show Border", "@BBSpectrumInternal ToggleBorder", showBorder);
	
	MakeMenuNOP(pSub, NULL);
	
	MakeMenuItem(pSub, "Start Monitor on Startup", "@BBSpectrumInternal MonitorStartUp", startInRecord);
	
	MakeMenuNOP(pSub, NULL);
	
	MakeMenuItem(pSub, "Transparency Enabled", "@BBSpectrumInternal TransparencyEnabled", transparencyEnabled);
	MakeMenuItem(pSub, "Transparent Back Enabled", "@BBSpectrumInternal TransbackEnabled", transback);
	// insert an item to adjust an integer value. Implemented in bblean for now
	MakeMenuItemInt(pSub, "Alpha Value", "@BBSpectrumInternal TransparencyAlpha", transparencyAlpha, 0, 255);
	
	tabPosMenu = MakeNamedMenu("Tab Position", "BBSpectrum_TabPosition", popup);
	MakeMenuItem(tabPosMenu, "Hide Tab", "@BBSpectrumInternal SetTabHT", !strcmp(tabPos, "ht"));
	MakeMenuItem(tabPosMenu, "Top Right", "@BBSpectrumInternal SetTabTR", !strcmp(tabPos, "tr"));
	MakeMenuItem(tabPosMenu, "Bottom Right", "@BBSpectrumInternal SetTabBR", !strcmp(tabPos, "br"));
	MakeMenuItem(tabPosMenu, "Top Left", "@BBSpectrumInternal SetTabTL", !strcmp(tabPos, "tl"));
	MakeMenuItem(tabPosMenu, "Bottom Left", "@BBSpectrumInternal SetTabBL", !strcmp(tabPos, "bl"));
	MakeSubmenu(pSub, tabPosMenu, "Tab Position");
	
	MakeMenuNOP(pSub, NULL);
	
	scopeTypeMenu = MakeNamedMenu("Orientation", "BBSpectrum_Orientation", popup);
	MakeMenuItem(scopeTypeMenu, "Vertical", "@BBSpectrumInternal SetScopeV", !strcmp(scopeType, "v"));
	MakeMenuItem(scopeTypeMenu, "Horizontal", "@BBSpectrumInternal SetScopeH", !strcmp(scopeType, "h"));
	MakeSubmenu(pSub, scopeTypeMenu, "Orientation");
	
	MakeMenuNOP(pSub, NULL);
	
	KHzMenu = MakeNamedMenu("KHz", "BBSpectrum_KHz", popup);
	MakeMenuItem(KHzMenu, "8", "@BBSpectrumInternal SetKHz 8" , KHz == 8);
	MakeMenuItem(KHzMenu, "16", "@BBSpectrumInternal SetKHz 16" , KHz == 16);
	MakeMenuItem(KHzMenu, "44", "@BBSpectrumInternal SetKHz 44" , KHz == 44);
	MakeSubmenu(pSub, KHzMenu, "KHz");
	
	BitMenu = MakeNamedMenu("Bit", "BBSpectrum_Bit", popup);
	MakeMenuItem(BitMenu, "8", "@BBSpectrumInternal SetBit 8" , bit == 8);
	MakeMenuItem(BitMenu, "16", "@BBSpectrumInternal SetBit 16" , bit == 16);
	MakeSubmenu(pSub, BitMenu, "Bit");
	
	FPSMenu = MakeNamedMenu("FPS", "BBSpectrum_FPS", popup);
	for(n = 10; n < 101; n += 10){
		sprintf(buff1, "@BBSpectrumInternal SetFPS %d", n);
		sprintf(buff2, "%d", n);
		MakeMenuItem(FPSMenu, buff2, buff1, wait == 1000/n);
	}
	MakeSubmenu(pSub, FPSMenu, "FPS");
	
	BufferMenu = MakeNamedMenu("Buffer Size", "BBSpectrum_BufferSize", popup);
	for(n = 5; n <= 8; n++){
		t = powt(n);
		sprintf(buff1, "@BBSpectrumInternal SetBufferSize %d", t);
		sprintf(buff2, "%d", t);
		MakeMenuItem(BufferMenu, buff2, buff1, buffsize == t);
	}
	MakeSubmenu(pSub, BufferMenu, "Buffer Size");
	
	wave_leftMenu = MakeNamedMenu("Freq Min", "BBSpectrum_FreqMin", popup);
	for(n = 0; n < wave_right; n++){
		sprintf(buff1, "@BBSpectrumInternal SetFreqMin %d", n);
		sprintf(buff2, "%d", n);
		MakeMenuItem(wave_leftMenu, buff2, buff1, wave_left == n);
	}
	MakeSubmenu(pSub, wave_leftMenu, "Freq Min");
	
	wave_rightMenu = MakeNamedMenu("Freq Max", "BBSpectrum_FreqMax", popup);
	for(n = wave_left + 1; n < buffsize; n++){
		sprintf(buff1, "@BBSpectrumInternal SetFreqMax %d", n);
		sprintf(buff2, "%d", n);
		MakeMenuItem(wave_rightMenu, buff2, buff1, wave_right == n);
	}
	MakeSubmenu(pSub, wave_rightMenu, "Freq Max");
	
	
	MakeMenuNOP(pSub, NULL);
	
	MakeMenuItem(pSub, "Motion Blur", "@BBSpectrumInternal ToggleBlur", motionblur);
	BlurMenu = MakeNamedMenu("Motion Blur Alpha", "BBSpectrum_BlurAlpha", popup);
	for(n = 10; n < 250; n += 10){
		sprintf(buff1, "@BBSpectrumInternal SetBlurAlpha %d", n);
		sprintf(buff2, "%d", n);
		MakeMenuItem(BlurMenu, buff2, buff1, blend.SourceConstantAlpha == n);
	}
	MakeSubmenu(pSub, BlurMenu, "Motion Blur Alpha");
	
	MakeMenuNOP(pSub, NULL);
	
	MakeMenuItem(pSub, "Show peak markers", "@BBSpectrumInternal PeakMarkers", peaks);
	
	peakmenu = MakeNamedMenu("Peak Marker Type", "BBSpectrum_SetPeakType", popup);
	MakeMenuItem(peakmenu, "Dot", "@BBSpectrumInternal SetPeakDot", !strcmp(peaktype, "dot"));
	MakeMenuItem(peakmenu, "Line", "@BBSpectrumInternal SetPeakLine", !strcmp(peaktype, "line"));
	MakeSubmenu(pSub, peakmenu, "Peak Marker Type");
	
	peakdropmenu = MakeNamedMenu("Peak Drop Speed", "BBSpectrum_SetPeakDropSpeed", popup);
	for(n = 1; n <= 20; n++){
		t = (int)(peakdrop*10);
		sprintf(buff1, "@BBSpectrumInternal SetPeakDrop %d", n);
		sprintf(buff2, "%d/10", n);
		MakeMenuItem(peakdropmenu, buff2, buff1, t == n);
	}
	MakeSubmenu(pSub, peakdropmenu, "Peak Drop Speed");
	
	MakeMenuNOP(pSub, NULL);
	
	maxheightmenu = MakeNamedMenu("Max Bar Height", "BBSpectrum_SetMaxBarHeight", popup);
	for(n = 1; n <= 128; n++){
		sprintf(buff1, "@BBSpectrumInternal SetMaxHeight %d", n);
		sprintf(buff2, "%d", n);
		MakeMenuItem(maxheightmenu, buff2, buff1, maxheight == n);
	}
	MakeSubmenu(pSub, maxheightmenu, "Max Bar Height");
	
	MakeMenuNOP(pSub, NULL);
	
	//Style Menu...
	stylemenu = MakeNamedMenu("Visual Style", "BBSpectrum_SetStyle", popup);
	MakeMenuItem(stylemenu, "Style 1", "@BBSpectrum SetStyle 1", false);
	MakeMenuItem(stylemenu, "Style 2", "@BBSpectrum SetStyle 2", false);
	MakeMenuItem(stylemenu, "Style 3", "@BBSpectrum SetStyle 3", false);
	MakeMenuItem(stylemenu, "Style 4", "@BBSpectrum SetStyle 4", false);
	MakeSubmenu(pSub, stylemenu, "Visual Style");
	
	// ...and add it to the main plugin menu
	MakeSubmenu(pMenu, pSub, "Configuration");

	// --------------------------------------------------------------
	
	// Recording toggle.
	MakeMenuItem(pMenu, "Monitor Wave?", "@BBSpectrumInternal ToggleRecord", InRecord);
	// Blank space...
	MakeMenuNOP(pMenu, NULL);
	// Edit the RC-File, using the default Blackbox Editor
	MakeMenuItem(pMenu, "Edit BBSpectrum.rc", "@BBSpectrumInternal Edit", false);
	// ...an "About" box is always nice, even though we now have the regular plugin info function...
	MakeMenuItem(pMenu, "About BBSpectrum", "@BBSpectrumInternal About", false);

	// And finally, we show the menu...
	ShowMenu(pMenu);
}

//===========================================================================

void set_setting(bool *flag, const char *toolbar_message, const char *rc_string)
{
	// send a display sting to the toolbar
	char msg[80];
	sprintf(msg, "BBSpectrumPlugin -> %s %sabled!",
		toolbar_message, *flag ? "en" : "dis");
	SendMessage(hwndBlackbox, BB_SETTOOLBARLABEL, 0, (LPARAM)msg);

	// write the new setting to the rc - file
	WriteBool(rcpath, rc_string, *flag);

	// apply new settings
	set_window_modes();

	// and update the menu checkmarks
	ShowMyMenu(false);
}

void toggle_setting(bool *flag, const char *toolbar_message, const char *rc_string)
{
	// toggle the setting
	*flag = false == *flag;

	// send a display sting to the toolbar
	char msg[80];
	sprintf(msg, "BBSpectrumPlugin -> %s %sabled!",
		toolbar_message, *flag ? "en" : "dis");
	SendMessage(hwndBlackbox, BB_SETTOOLBARLABEL, 0, (LPARAM)msg);

	// write the new setting to the rc - file
	WriteBool(rcpath, rc_string, *flag);

	// apply new settings
	set_window_modes();

	// and update the menu checkmarks
	ShowMyMenu(false);
}

//===========================================================================

//===========================================================================

void MakeBuffer(){
	PAINTSTRUCT ps; RECT r;
	HDC hdc = BeginPaint(hwndPlugin, &ps);
	
	//get window rectangle
	GetClientRect(hwndPlugin, &r);
	
	//first get a new 'device context'
	buf = CreateCompatibleDC(NULL);
	
	//then create a buffer in memory with the window size
	bufbmp = CreateCompatibleBitmap(hdc, r.right, r.bottom);
	
	//select it into the DC and store, whatever there was before.
	otherbmp = SelectObject(buf, bufbmp);
	
	EndPaint(hwndPlugin, &ps);	
}

void resizeWindow(){
	DeleteObject(SelectObject(buf, otherbmp));
	DeleteDC(buf);
	MakeBuffer();
	
	InvalidateRect(hwndPlugin, NULL, FALSE);
}

LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	MINMAXINFO *minmax;
	RECT *r;
	char szTemp[MAX_PATH];

	// Blackbox messages we want to "subscribe" to:
	// BB_RECONFIGURE -> Sent when changing style and on reconfigure
	// BB_BROADCAST -> Broadcast message (bro@m)
	static int msgs[] = { BB_RECONFIGURE, BB_BROADCAST, 0};

	switch (message)
	{       
		// ==========
		case WM_CREATE:
			// set the global variable
			hwndPlugin = hwnd;
			
			// Register to receive Blackbox messages...
			SendMessage(hwndBlackbox, BB_REGISTERMESSAGE, (WPARAM)hwnd, (LPARAM)msgs);
			
			// make the window visible on all worspaces
			MakeSticky(hwnd);
			
			MakeBuffer();
			
			set_window_modes();
			
			break;

		// ==========
		case WM_DESTROY:
			//remove us from the sticky-windows-list
			RemoveSticky(hwnd);
			
			//restore the first previous whatever to the dc,
			//get in exchange back our bitmap, and delete it.
			DeleteObject(SelectObject(buf, otherbmp));

			//delete the memory - 'device context'
			DeleteDC(buf);

			// Unregister Blackbox messages...
			SendMessage(hwndBlackbox, BB_UNREGISTERMESSAGE, (WPARAM)hwnd, (LPARAM)msgs);
			break;

		// ==========
		// If Blackbox sends a reconfigure message, load the new style settings and
		// update the window...

		case BB_RECONFIGURE:
			if(InRecord){
				stopRecording();
				ReadRCSettings();
				GetStyleSettings();
				set_window_modes();
				startRecording();
			}else{
				ReadRCSettings();
				GetStyleSettings();
				set_window_modes();
			}
			break;

		// ==========
		// Window redraw processing...
		case WM_PAINT:
		{
			PAINTSTRUCT ps; RECT r;

			//get the screen-buffer
			HDC hdc = BeginPaint(hwnd, &ps);

			//get window rectangle
			GetClientRect(hwnd, &r);
			
			// use new api
			// Paint background according to the current style on the bitmap,
			// using the default border from style (true)
			
			if(!transback){
				// Draw gradient background.
				MakeStyleGradient(buf, &r, &myStyleItem, showBorder);
			}else{
				// Draw the transparent pink bib.
				HBRUSH hbbrush = CreateSolidBrush(TRANSCOLOUR);
				FillRect(buf, &r, hbbrush);
				DeleteObject(hbbrush);
			}
			
			//select it into the DC and store, whatever there was before
			HGDIOBJ otherfont = SelectObject(buf, hFont);

			//draw transparently on our background-gradient
			SetBkMode(buf, TRANSPARENT);
			
			//draw the wave energy history.
				char old = dat[0];
				int n;
				HPEN         hPen;
				HPEN         hPenOld;
				
				hPen = CreatePen(PS_SOLID, 1, PeakColor);
				hPenOld = (HPEN)SelectObject(buf, hPen);
				
				if(strcmp(scopeType, "h") == 0){
					int top = r.top + height - (bevelWidth + borderWidth);
					int left = r.left + bevelWidth + borderWidth;
					float scale = width - ((bevelWidth + borderWidth)*2);
					scale = scale/freqsize;
					char *data;
					RECT bar;
					HBRUSH hbbrush = CreateSolidBrush(ScopeColor);
					if(STREQ(peaktype, "line"))MoveToEx(buf, left, top - peak[wave_left], NULL);
					if(dat != NULL){
						int i;
						for(n = wave_left; n < wave_right; n++){
							i = n - wave_left;
							bar.top = top;
							bar.left = left + (int)(scale*(float)i);
							bar.bottom = top - dat[n];
							bar.right = left + (int)(scale*(float)(i + 1));
							FillRect(buf, &bar, hbbrush);
							if(peaks){
								if(STREQ(peaktype, "dot")){
									MoveToEx(buf, left + (int)(scale*(float)i), top - peak[n], NULL);
									LineTo(buf, left + (int)(scale*(float)(i+1)), top - peak[n]);
								}else if(n > 0){
									LineTo(buf, left + (int)(scale*(float)(i+1)), top - peak[n]);
								}
							}
						}
					}
					DeleteObject(hbbrush);
				}else{
					int top = r.top + bevelWidth + borderWidth;
					int left = r.left + (bevelWidth + borderWidth);
					float scale = height - ((bevelWidth + borderWidth)*2);
					scale = scale/freqsize;
					char *data;
					RECT bar;
					HBRUSH hbbrush = CreateSolidBrush(ScopeColor);
					if(STREQ(peaktype, "line"))MoveToEx(buf, left + peak[wave_left], top, NULL);
					if(dat != NULL){
						int i;
						for(n = wave_left; n < wave_right; n++){
							i = n - wave_left;
							bar.top = top + (int)(scale*(float)i);
							bar.left = left;
							bar.bottom = top + (int)(scale*(float)(i + 1));
							bar.right = left + dat[n];
							FillRect(buf, &bar, hbbrush);
							if(peaks){
								if(STREQ(peaktype, "dot")){
									MoveToEx(buf, left + peak[n], top + (int)(scale*(float)i), NULL);
									LineTo(buf, left + peak[n], top + (int)(scale*(float)(i+1)));
								}else if(n > 0){
									LineTo(buf, left + peak[n], top + (int)(scale*(float)(i+1)));
								}
							}
						}
					}
					DeleteObject(hbbrush);
				}
				
				SelectObject(buf, hPenOld);
				DeleteObject(hPen);
				
				if(transback){
					// Draw the handle...
					
					hPen = CreatePen(PS_SOLID, borderWidth, borderColor);
					hPenOld = (HPEN)SelectObject(buf, hPen);
					
					HBRUSH hbrush = CreateSolidBrush(myStyleItem.Color);
					HBRUSH hbrushOld = (HBRUSH)SelectObject(buf, hbrush);
					
					POINT pnts[3];
					if(strcmp(tabPos, "br") == 0){
						pnts[0].x = r.right- borderWidth; 	pnts[0].y = r.bottom- borderWidth;
						pnts[1].x = r.right- 10; 		pnts[1].y = r.bottom- borderWidth;
						pnts[2].x = r.right- borderWidth; 	pnts[2].y = r.bottom- 10;
					}else if(strcmp(tabPos, "tr") == 0){
						pnts[0].x = r.right- borderWidth; 	pnts[0].y = r.top+ borderWidth;
						pnts[1].x = r.right- 10; 		pnts[1].y = r.top+ borderWidth;  
						pnts[2].x = r.right- borderWidth; 	pnts[2].y = r.top+ 10;
					}else if(strcmp(tabPos, "bl") == 0){
						pnts[0].x = r.left+ borderWidth; 	pnts[0].y = r.bottom- borderWidth;
						pnts[1].x = r.left+ 10; 		pnts[1].y = r.bottom- borderWidth;
						pnts[2].x = r.left+ borderWidth; 	pnts[2].y = r.bottom- 10;
					}else if(strcmp(tabPos, "tl") == 0){
						pnts[0].x = r.left+ borderWidth; 	pnts[0].y = r.top+ borderWidth;
						pnts[1].x = r.left+ 10; 		pnts[1].y = r.top+ borderWidth;
						pnts[2].x = r.left+ borderWidth; 	pnts[2].y = r.top+ 10;
					}else if(strcmp(tabPos, "ht") == 0){
						// Do nothing...
					}else{
						pnts[0].x = r.right- borderWidth; 	pnts[0].y = r.bottom- borderWidth;
						pnts[1].x = r.right- 10; 		pnts[1].y = r.bottom- borderWidth;
						pnts[2].x = r.right- borderWidth; 	pnts[2].y = r.bottom- 10;
					}
					
					// Draw it...
					if(strcmp(tabPos, "ht") != 0){
						Polygon(buf, pnts, 3);
					}
					
					// Clear up.
					SelectObject(hdc, hbrushOld);
					DeleteObject(hbrush);
					
					SelectObject(buf, hPenOld);
					DeleteObject(hPen);
				}
			
			//restore the second previous whatever to the dc,

			SelectObject(buf, otherfont);

			//now copy the drawing from the paint buffer to the window...
			if(motionblur && !transback){
				AlphaBlend(	hdc,
						ps.rcPaint.left,
						ps.rcPaint.top,
						ps.rcPaint.right  - ps.rcPaint.left,
						ps.rcPaint.bottom - ps.rcPaint.top,
						buf,
						ps.rcPaint.left,
						ps.rcPaint.top,
						ps.rcPaint.right  - ps.rcPaint.left,
						ps.rcPaint.bottom - ps.rcPaint.top,
						blend
						);
			}else{
				BitBlt(		hdc,
						ps.rcPaint.left,
						ps.rcPaint.top,
						ps.rcPaint.right  - ps.rcPaint.left,
						ps.rcPaint.bottom - ps.rcPaint.top,
						buf,
						ps.rcPaint.left,
						ps.rcPaint.top,
						SRCCOPY
						);
			}

			//done
			EndPaint(hwnd, &ps);
			break;
		}

		// ==========
		// Broadcast messages (bro@m -> the bang killah! :D <vbg>)
		case BB_BROADCAST:
		{
			const char *msg_string = (LPCSTR)lParam;

			// First we check for the two "global" bro@ms,
			if (!stricmp(msg_string, "@BBShowPlugins"))
			{
				// @BBShowPlugins: Show window (global bro@m)
				if(!inSlit){
					hidden = false;
					set_window_modes();
				}
				break;
			}
			if (!stricmp(msg_string, "@BBHidePlugins"))
			{
				// @BBHidePlugins: Hide window (global bro@m)
				if(!inSlit){
					hidden = true;
					set_window_modes();
				}
				break;
			}
			
			// ==========
			// ...then we check for the plugin bro@ms that are available to
			// the end-user (ie. listed in the documentation! <g>)...
			if (!stricmp(msg_string, "@BBSpectrum SetStyle 1"))
			{
				strcpy(ScopeColorTxt, "toolbar.windowLabel.color:");
				WriteString(rcpath, "BBSpectrum.plugin.style.ScopeColor:", "toolbar.windowLabel.color");
				
				strcpy(PeakColorTxt, "window.label.focus.textColor:");
				WriteString(rcpath, "BBSpectrum.plugin.style.PeakColor:", "window.label.focus.textColor");
				
				GetStyleSettings();
				break;
			}
			if (!stricmp(msg_string, "@BBSpectrum SetStyle 2"))
			{
				strcpy(ScopeColorTxt, "window.label.focus.color:");
				WriteString(rcpath, "BBSpectrum.plugin.style.ScopeColor:", "window.label.focus.color");
				
				strcpy(PeakColorTxt, "menu.frame.color:");
				WriteString(rcpath, "BBSpectrum.plugin.style.PeakColor:", "menu.frame.color");
				
				GetStyleSettings();
				break;
			}
			if (!stricmp(msg_string, "@BBSpectrum SetStyle 3"))
			{
				strcpy(ScopeColorTxt, "menu.frame.color:");
				WriteString(rcpath, "BBSpectrum.plugin.style.ScopeColor:", "menu.frame.color");
				
				strcpy(PeakColorTxt, "window.label.focus.color:");
				WriteString(rcpath, "BBSpectrum.plugin.style.PeakColor:", "window.label.focus.color");
				
				GetStyleSettings();
				break;
			}
			if (!stricmp(msg_string, "@BBSpectrum SetStyle 4"))
			{
				strcpy(ScopeColorTxt, "menu.hilite.color:");
				WriteString(rcpath, "BBSpectrum.plugin.style.ScopeColor:", "menu.hilite.color");
				
				strcpy(PeakColorTxt, "menu.frame.color:");
				WriteString(rcpath, "BBSpectrum.plugin.style.PeakColor:", "menu.frame.color");
				
				GetStyleSettings();
				break;
			}

			// ==========
			// Finally, we check for our "internal" bro@m, which is used for
			// all plugin internal commands, for example the menu commands...

			static const char Example_Broam_ID[] = "@BBSpectrumInternal";

			if (!strnicmp(msg_string, Example_Broam_ID, sizeof Example_Broam_ID - 1))
			{
				msg_string += sizeof Example_Broam_ID - 1;
				while (*msg_string == ' ') msg_string ++;
				
				if (!stricmp(msg_string, "SlitToggle"))
				{
					if(inSlit && hSlit)
					{
						// We are in the slit, so lets unload and get out..
						SendMessage(hSlit, SLIT_REMOVE, NULL, (LPARAM)hwndPlugin);

						// Here you can move to where ever you want ;)
						SetWindowPos(hwndPlugin, NULL, xpos, ypos, 0, 0, SWP_NOZORDER | SWP_NOSIZE);
						inSlit = false;
						set_window_modes();
					}
					/* Make sure before you try and load into the slit that you have
					 * the HWND of the slit ;)
					 */
					else if(hSlit)
					{
						// (Back) into the slit..
						inSlit = true;
						set_window_modes();
						SendMessage(hSlit, SLIT_ADD, NULL, (LPARAM)hwndPlugin);
					}
					
					set_setting(&inSlit,        // the * to the bool to toggle
						msg_string,                     // the message to display on toolbar
						"BBSpectrum.plugin.inSlit:"   // the rc-setting to write
						);
				}
				
				//...and check for the second part of the message-string
				if (!stricmp(msg_string, "MonitorStartUp"))
				{
					// make use of our frienly ghost from above
					toggle_setting(&startInRecord,        // the * to the bool to toggle
						msg_string,                     // the message to display on toolbar
						"BBSpectrum.plugin.StartWaveMonitorOnStartUp:"   // the rc-setting to write
						);
					break;
				}
				if (!stricmp(msg_string, "PeakMarkers"))
				{
					// make use of our frienly ghost from above
					toggle_setting(&peaks,        // the * to the bool to toggle
						msg_string,                     // the message to display on toolbar
						"BBSpectrum.plugin.wave.peaks:"   // the rc-setting to write
						);
					break;
				}
				if (!stricmp(msg_string, "AlwaysOnTop"))
				{
					// make use of our frienly ghost from above
					toggle_setting(&alwaysOnTop,        // the * to the bool to toggle
						msg_string,                     // the message to display on toolbar
						"BBSpectrum.plugin.alwaysontop:"   // the rc-setting to write
						);
					break;
				}
				if (!stricmp(msg_string, "SnapWindowToEdge"))
				{
					toggle_setting(&snapWindow,
						msg_string,
						"BBSpectrum.plugin.snapwindow:"
						);
					break;
				}
				if (!stricmp(msg_string, "TransparencyEnabled"))
				{
					toggle_setting(&transparencyEnabled,
						msg_string,
						"BBSpectrum.plugin.transparency.enabled:"
						);
					break;
				}
				if (!stricmp(msg_string, "ToggleBorder"))
				{
					// make use of our frienly ghost from above
					toggle_setting(&showBorder,        // the * to the bool to toggle
						msg_string,                     // the message to display on toolbar
						"BBSpectrum.plugin.showBorder:"   // the rc-setting to write
						);
					break;
				}
				if (!stricmp(msg_string, "TransbackEnabled"))
				{
					toggle_setting(&transback,
						msg_string,
						"BBSpectrum.plugin.transback.enabled:"
						);
					break;
				}

				{
					static char trans_alpha_id[] = "TransparencyAlpha ";

					// If the original command was
					//      "@ExampleInternal TransparencyAlpha"

					// the integer item sends a message like this:
					//      "@ExampleInternal TransparencyAlpha 45"

					if (!memicmp(msg_string, trans_alpha_id, sizeof(trans_alpha_id)-1))
					{
						transparencyAlpha = atoi(msg_string + sizeof(trans_alpha_id)-1);
						set_window_modes();
						WriteInt(rcpath, "BBSpectrum.plugin.transparency.alpha:", transparencyAlpha);
						break;
					}
				}

				if (!stricmp(msg_string, "ToggleWithPlugins"))
				{
					toggle_setting(&pluginToggle,
						msg_string,
						"BBSpectrum.plugin.togglewithplugins:"
						);
					break;
				}
				if (!stricmp(msg_string, "Edit"))
				{
					GetBlackboxEditor(szTemp);
					BBExecute(NULL, NULL, szTemp, rcpath, NULL, SW_SHOWNORMAL, false);
					break;
				}
				if (!stricmp(msg_string, "About"))
				{
					sprintf(szTemp,
						"%s\n\n"
						"Author : Kaloth\n"
						"Web: http://www.geocities.com/madant8\n"
						"Email: mulletwarriorextreem@hotmail.com",
						szVersion
						);
					MessageBox(
						NULL, szTemp, "About this plugin...",
						MB_OK | MB_ICONINFORMATION);
					break;
				}
				if (!stricmp(msg_string, "ToggleRecord"))
				{
					if(InRecord){stopRecording();}else{startRecording();}
					break;
				}
				
				if (!stricmp(msg_string, "ToggleBlur"))
				{
					toggle_setting(&motionblur,
						msg_string,
						"BBSpectrum.plugin.motionblur:"
						);
					break;
				}
				
				if (strstr(msg_string, "SetBlurAlpha") != NULL)
				{
					const char *temp = strchr(msg_string, ' ');
					blend.SourceConstantAlpha = atoi(temp);
					WriteInt(rcpath, "BBSpectrum.plugin.motionbluralpha:", blend.SourceConstantAlpha);
					break;
				}
				
				if (strstr(msg_string, "SetKHz") != NULL)
				{
					const char *temp = strchr(msg_string, ' ');
					KHz = atoi(temp);
					WriteInt(rcpath, "BBSpectrum.plugin.wave.KHz:", KHz);
					if(InRecord){
						stopRecording();
						startRecording();
					}
					break;
				}
				
				if (strstr(msg_string, "SetFreqMin") != NULL)
				{
					const char *temp = strchr(msg_string, ' ');
					wave_left = atoi(temp);
					freqsize = wave_right - wave_left;
					WriteInt(rcpath, "BBSpectrum.plugin.wave.freqmin:", wave_left);
					break;
				}
				
				if (strstr(msg_string, "SetFreqMax") != NULL)
				{
					const char *temp = strchr(msg_string, ' ');
					wave_right = atoi(temp);
					freqsize = wave_right - wave_left;
					WriteInt(rcpath, "BBSpectrum.plugin.wave.freqmax:", wave_right);
					break;
				}
				
				if (strstr(msg_string, "SetBuff") != NULL)
				{
					const char *temp = strchr(msg_string, ' ');
					buffsize = atoi(temp);
					if(wave_right > (buffsize - 1)){
						wave_right = buffsize-1;
					}
					if(wave_left > wave_right){
						wave_left = 0;
						wave_right = buffsize-1;
					}
					freqsize = wave_right - wave_left;
					WriteInt(rcpath, "BBSpectrum.plugin.wave.buffsize:", buffsize);
					if(InRecord){
						stopRecording();
						startRecording();
					}
					break;
				}
				
				if (strstr(msg_string, "SetBit") != NULL)
				{
					const char *temp = strchr(msg_string, ' ');
					bit = atoi(temp);
					WriteInt(rcpath, "BBSpectrum.plugin.wave.bit:", bit);
					if(InRecord){
						stopRecording();
						startRecording();
					}
					break;
				}
				
				if (strstr(msg_string, "SetFPS") != NULL)
				{
					const char *temp = strchr(msg_string, ' ');
					wait = atoi(temp);
					WriteInt(rcpath, "BBSpectrum.plugin.wave.fps:", wait);
					wait = 1000/wait;
					if(InRecord){
						stopRecording();
						startRecording();
					}
					break;
				}
				
				if (!stricmp(msg_string, "SetTabHT"))
				{
					strcpy(tabPos, "ht");
					WriteString(rcpath, "BBSpectrum.plugin.style.tabPosition:", tabPos);
					break;
				}
				if (!stricmp(msg_string, "SetTabTR"))
				{
					strcpy(tabPos, "tr");
					WriteString(rcpath, "BBSpectrum.plugin.style.tabPosition:", tabPos);
					break;
				}
				if (!stricmp(msg_string, "SetTabBL"))
				{
					strcpy(tabPos, "bl");
					WriteString(rcpath, "BBSpectrum.plugin.style.tabPosition:", tabPos);
					break;
				}
				if (!stricmp(msg_string, "SetTabBR"))
				{
					strcpy(tabPos, "br");
					WriteString(rcpath, "BBSpectrum.plugin.style.tabPosition:", tabPos);
					break;
				}
				if (!stricmp(msg_string, "SetTabTL"))
				{
					strcpy(tabPos, "tl");
					WriteString(rcpath, "BBSpectrum.plugin.style.tabPosition:", tabPos);
					break;
				}

				if (!stricmp(msg_string, "SetScopeV"))
				{
					strcpy(scopeType, "v");
					WriteString(rcpath, "BBSpectrum.plugin.style.scopeType:", scopeType);
					break;
				}
				if (!stricmp(msg_string, "SetScopeH"))
				{
					strcpy(scopeType, "h");
					WriteString(rcpath, "BBSpectrum.plugin.style.scopeType:", scopeType);
					break;
				}
				
				if (!stricmp(msg_string, "SetPeakDot"))
				{
					strcpy(peaktype, "dot");
					WriteString(rcpath, "BBSpectrum.plugin.style.peakType:", peaktype);
					break;
				}
				if (!stricmp(msg_string, "SetPeakLine"))
				{
					strcpy(peaktype, "line");
					WriteString(rcpath, "BBSpectrum.plugin.style.peakType:", peaktype);
					break;
				}
				if (strstr(msg_string, "SetPeakDrop") != NULL)
				{
					const char *temp = strchr(msg_string, ' ');
					peakdrop = ((float)atoi(temp)) / 10.0;
					WriteInt(rcpath, "BBSpectrum.plugin.style.peakDrop:", (int)(peakdrop * 10));
					break;
				}
				
				if (strstr(msg_string, "SetMaxHeight") != NULL)
				{
					const char *temp = strchr(msg_string, ' ');
					maxheight = atoi(temp);
					WriteInt(rcpath, "BBSpectrum.plugin.style.maxBarHeight:", maxheight);
					break;
				}
				
				if (!stricmp(msg_string, "SetStyle1"))
				{
					strcpy(ScopeColorTxt, "toolbar.windowLabel.color:");
					WriteString(rcpath, "BBSpectrum.plugin.style.ScopeColor:", "toolbar.windowLabel.color");
					
					strcpy(PeakColorTxt, "window.label.focus.textColor:");
					WriteString(rcpath, "BBSpectrum.plugin.style.PeakColor:", "window.label.focus.textColor");
					
					GetStyleSettings();
					break;
				}
				if (!stricmp(msg_string, "SetStyle2"))
				{
					strcpy(ScopeColorTxt, "window.label.focus.color:");
					WriteString(rcpath, "BBSpectrum.plugin.style.ScopeColor:", "window.label.focus.color");
					
					strcpy(PeakColorTxt, "menu.frame.color:");
					WriteString(rcpath, "BBSpectrum.plugin.style.PeakColor:", "menu.frame.color");
					
					GetStyleSettings();
					break;
				}
				if (!stricmp(msg_string, "SetStyle3"))
				{
					strcpy(ScopeColorTxt, "menu.frame.color:");
					WriteString(rcpath, "BBSpectrum.plugin.style.ScopeColor:", "menu.frame.color");
					
					strcpy(PeakColorTxt, "window.label.focus.color:");
					WriteString(rcpath, "BBSpectrum.plugin.style.PeakColor:", "window.label.focus.color");
					
					GetStyleSettings();
					break;
				}
				if (!stricmp(msg_string, "SetStyle4"))
				{
					strcpy(ScopeColorTxt, "menu.hilite.color:");
					WriteString(rcpath, "BBSpectrum.plugin.style.ScopeColor:", "menu.hilite.color");
					
					strcpy(PeakColorTxt, "menu.frame.color:");
					WriteString(rcpath, "BBSpectrum.plugin.style.PeakColor:", "menu.frame.color");
					
					GetStyleSettings();
					break;
				}
			}
			break;
		}
		// ==========

		case WM_NCRBUTTONUP:
			ShowMyMenu(true);
			break;
			
		case WM_RBUTTONUP:
			ShowMyMenu(true);
			break;

		// ==========
		// The window is moving
		case WM_WINDOWPOSCHANGING:
			// Is SnapWindowToEdge enabled? - and the plugin not hidden?
			if (!inSlit && snapWindow && IsWindowVisible(hwnd))
			{
				// ...then snap window to screen edges, if the distance is
				// less than 10 pixel (if the last bool is false it uses the
				// current DesktopArea)
				SnapWindowToEdge((WINDOWPOS*)lParam, 10, true);
			}
			break;

		// ==========
		// Save window position if it changed...
		case WM_WINDOWPOSCHANGED:
		{
			if(!inSlit){
				WINDOWPOS* wp = (WINDOWPOS*)lParam;
				xpos = wp->x;
				ypos = wp->y;
				if (wp->cx < 16) wp->cx = 16;
				if (wp->cy < 16) wp->cy = 16;
				width = wp->cx; height = wp->cy;
				WriteInt(rcpath, "BBSpectrum.plugin.x:", xpos);
				WriteInt(rcpath, "BBSpectrum.plugin.y:", ypos);
				WriteInt(rcpath, "BBSpectrum.plugin.width:", width);
				WriteInt(rcpath, "BBSpectrum.plugin.height:", height);
			}
			break;
		}

		// ==========
		// Allow window to move...
		case WM_NCHITTEST:
			if(!inSlit){
				return HTCAPTION;
			}
			return HTCLIENT;
		
		case WM_SIZE:
			{int w = LOWORD(lParam);
			int h = HIWORD(lParam);
			if (w < 16) w = 16;
			if (h < 16) h = 16;
			width = w; height = h;
			WriteInt(rcpath, "BBSpectrum.plugin.width:", width);
			WriteInt(rcpath, "BBSpectrum.plugin.height:", height);}
			resizeWindow();
			break;
		
		case WM_SIZING:
			r = (RECT *)lParam;
			width = r->right - r->left;
			height = r->bottom - r->top;
			resizeWindow();
			break;
		
		case WM_NCLBUTTONDOWN:
			// initialize moving, when alt-key is held down
			if (GetAsyncKeyState(VK_MENU) & 0x8000){
				PostMessage(hwnd, WM_SYSCOMMAND, 0xf008, 0); // bottom right corner - resize
			}else{
				PostMessage(hwnd, WM_SYSCOMMAND, 0xf012, 0); // move
			}
			break;
		
		//Set minimum window size...
		case WM_GETMINMAXINFO:
			minmax = (MINMAXINFO *)lParam;
			minmax->ptMinTrackSize.x = 5;
			minmax->ptMinTrackSize.y = 5;
			break;

		// ==========
		// Prevent closing with alt-f4
		case WM_CLOSE:
			break;

		case WM_MOUSEWHEEL:
			if(GET_WHEEL_DELTA_WPARAM(wParam) > 0){
				if(maxheight < 128)maxheight++;
			}else{
				if(maxheight > 1)maxheight--;
			}
			WriteInt(rcpath, "BBSpectrum.plugin.style.maxBarHeight:", maxheight);
			break;
		
		// ==========
		// Let windows do everything else
		default:
			return DefWindowProc(hwnd,message,wParam,lParam);
	}
	// return zero, except from the hit-test message
	return 0;
}

//===========================================================================

void ReadRCSettings()
{
	char defaultpath[MAX_PATH]; int nLen, i;
	// First we look for the config file in the same folder as the plugin...
	GetModuleFileName(hInstance, rcpath, sizeof(rcpath));
	for (i=0;;i++)
	{
		nLen = strlen(rcpath); while (nLen && rcpath[nLen-1] != '\\') nLen--;
		// ...check the two possible filenames 'examplerc' and 'example.rc'...
		strcpy(rcpath+nLen, "BBSpectrumrc");  if (FileExists(rcpath)) break;
		strcpy(rcpath+nLen, "BBSpectrum.rc"); if (FileExists(rcpath)) break;
		if (0 == i)
		{   //...if not found aside the plugin, we keep this as the default path...
			strcpy(defaultpath, rcpath);
			//...and try the Blackbox directory.
			GetBlackboxPath(rcpath, sizeof(rcpath));
			continue;
		}
		//if again no config file was found, we use the default path...
		strcpy(rcpath, defaultpath);
		break; // ...and quit the loop
	}
	// If a config file was found we read the plugin settings from it...
	// ...if not, the ReadXXX functions give us just the defaults.
	xpos = ReadInt(rcpath, "BBSpectrum.plugin.x:", 10);
	ypos = ReadInt(rcpath, "BBSpectrum.plugin.y:", 10);
	inSlit	   	    = ReadBool(rcpath, "BBSpectrum.plugin.inSlit:", false);
	motionblur 	    = ReadBool(rcpath, "BBSpectrum.plugin.motionblur:", false);
	blend.SourceConstantAlpha = ReadInt(rcpath,  "BBSpectrum.plugin.motionbluralpha:", 120);
	transparencyEnabled = ReadBool(rcpath, "BBSpectrum.plugin.transparency.enabled:", false);
	transback	    = ReadBool(rcpath, "BBSpectrum.plugin.transback.enabled:", false);
	transparencyAlpha   = ReadInt(rcpath,  "BBSpectrum.plugin.transparency.alpha:", 60);
	alwaysOnTop         = ReadBool(rcpath, "BBSpectrum.plugin.alwaysontop:", true);
	snapWindow          = ReadBool(rcpath, "BBSpectrum.plugin.snapwindow:", true);
	pluginToggle        = ReadBool(rcpath, "BBSpectrum.plugin.togglewithplugins:", true);
	strcpy(updateURL, ReadString(rcpath,"BBSpectrum.plugin.update.url:", ""));
	
	// Animation Settings...
	width 		    = ReadInt(rcpath, "BBSpectrum.plugin.width:", 100);
	height		    = ReadInt(rcpath, "BBSpectrum.plugin.height:", 25);
	startInRecord       = ReadBool(rcpath, "BBSpectrum.plugin.wave.StartWaveMonitorOnStartUp:", true);
	buffsize	    = ReadInt(rcpath, "BBSpectrum.plugin.wave.buffsize:", 64);
	if(buffsize > 256){buffsize = 256;}
	if(buffsize < 32){buffsize = 32;}
	wave_left	    = ReadInt(rcpath, "BBSpectrum.plugin.wave.freqmin:", 0);
	wave_right	    = ReadInt(rcpath, "BBSpectrum.plugin.wave.freqmax:", (buffsize >> 1));
	if(wave_left > wave_right){
		wave_left = 0;
		wave_right = buffsize-1;
	}
	if(wave_right > (buffsize - 1)){
		wave_right = buffsize-1;
	}
	freqsize = wave_right - wave_left;
	wait		    = ReadInt(rcpath, "BBSpectrum.plugin.wave.fps:", 25);
	if(wait <= 0){wait = 1;}
	wait = 1000/wait;
	KHz		    = ReadInt(rcpath, "BBSpectrum.plugin.wave.KHz:", 16);
	bit		    = ReadInt(rcpath, "BBSpectrum.plugin.wave.bit:", 16);
	peaks       	    = ReadBool(rcpath, "BBSpectrum.plugin.wave.peaks:", false);
	strcpy(peaktype, ReadString(rcpath,"BBSpectrum.plugin.style.peakType:", "dot"));
	peakdrop	    = (float)ReadInt(rcpath, "BBSpectrum.plugin.style.peakDrop:", 1.0) / 10.0;
	maxheight	    = ReadInt(rcpath, "BBSpectrum.plugin.style.maxBarHeight:", 64);
	
	// Style Settings...
	strcpy(ScopeColorTxt, ReadString(rcpath,"BBSpectrum.plugin.style.ScopeColor:", "toolbar.textColor"));
	strcat(ScopeColorTxt, ":");
	strcpy(PeakColorTxt, ReadString(rcpath,"BBSpectrum.plugin.style.PeakColor:", "borderColor"));
	strcat(PeakColorTxt, ":");
	strcpy(tabPos, ReadString(rcpath,"BBSpectrum.plugin.style.tabPosition:", "br"));
	strcpy(scopeType, ReadString(rcpath,"BBSpectrum.plugin.style.scopeType:", "h"));
	showBorder	    = ReadBool(rcpath, "BBSpectrum.plugin.showBorder:", true);
}

//===========================================================================
void GetStyleSettings()
{

	if (hFont)
		DeleteObject(hFont), hFont = NULL;
	
	// -------------------------------------------------------------------
	// the direct way:
	// ---------------------

	myStyleItem = *(StyleItem*)GetSettingPtr(SN_TOOLBAR);

	bevelWidth  = *(int*)GetSettingPtr(SN_BEVELWIDTH);
	borderWidth = *(int*)GetSettingPtr(SN_BORDERWIDTH);
	borderColor = *(COLORREF*)GetSettingPtr(SN_BORDERCOLOR);
	
	// Read Custom style mods...
	
	const char *stylepath = stylePath();
	
	ScopeColor = ReadColor(stylepath, ScopeColorTxt, "#FFFFFF");
	PeakColor = ReadColor(stylepath, PeakColorTxt, "#000000");

	hFont = CreateStyleFont(&myStyleItem);
	

	/*// -------------------------------------------------------------------
	// the individual way:
	// -------------------------

	// Get the path to the current style file from Blackbox...
	const char *stylepath = stylePath();
	const char *temp;

	// Get the applicable color settings from the current style...
	myStyleItem.Color = ReadColor(stylepath, Color, "#000000");
	myStyleItem.ColorTo = ReadColor(stylepath, ColorTo, "#000000");
	myStyleItem.TextColor = ReadColor(stylepath, TextColor, "#FFFFFF");
	
	// ...gradient type, bevel etc. (using a StyleItem)...
	temp = ReadString(stylepath, "toolbar.label:", "Flat Gradient Vertical");
	ParseItem(temp, &myStyleItem);

	// ...font settings...
	strcpy(myStyleItem.Font, ReadString(stylepath, "toolbar.font:", "tahoma"));
	myStyleItem.FontHeight = ReadInt(stylepath, "toolbar.fontHeight:", 12);

	temp = ReadString(stylepath, "toolbar.fontHeight:", "normal");
	myStyleItem.FontWeight = stricmp(temp, "bold") ? FW_BOLD : FW_NORMAL;

	// ...and some additional parameters
	bevelWidth  = ReadInt(stylepath, "bevelWidth:",  1);
	borderWidth = ReadInt(stylepath, "borderWidth:", 1);
	borderColor = ReadColor(stylepath, BorderColor, "#000000");

	//make the font
	hFont = CreateFont(
		myStyleItem.FontHeight, 0, 0, 0, myStyleItem.FontWeight,
		false, false, false,
		DEFAULT_CHARSET,
		OUT_DEFAULT_PRECIS,
		CLIP_DEFAULT_PRECIS,
		DEFAULT_QUALITY,
		DEFAULT_PITCH,
		myStyleItem.Font
		);*/
}

//===========================================================================

// Many thanks to the peeps at www.musicdsp.com and Toth Laszlo for this FFT library ;)

/////////////////////////////////////////////////////////
//	                 FFT library
//
//  (one-dimensional complex and real FFTs for array 
//  lengths of 2^n)
//
//	Author: Toth Laszlo (tothl@inf.u-szeged.hu)
//  
//	Research Group on Artificial Intelligence
//  H-6720 Szeged, Aradi vertanuk tere 1, Hungary
//	
//	Last modified: 97.05.29
//      Last Shuffled about by Kal for use in BBSpectrum: 19/04/2004
/////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////
// Decimation-in-freq radix-2 in-place butterfly
// data: array of doubles:
// re(0),im(0),re(1),im(1),...,re(size-1),im(size-1)
// it means that size=array_length/2 !!
// 
// suggested use:
// intput in normal order
// output in bit-reversed order
//
// Source: Rabiner-Gold: Theory and Application of DSP,
// Prentice Hall,1978

void dif_butterfly(double *data,long size){

	long angle,astep,dl;
	double xr,yr,xi,yi,wr,wi,dr,di,ang;
	double *l1, *l2, *end, *ol2;

     astep=1;
	 end=data+size+size;
     for(dl=size;dl>1;dl>>=1,astep+=astep){
					  l1=data;
					  l2=data+dl;
                      for(;l2<end;l1=l2,l2=l2+dl){
							ol2=l2;
                            for(angle=0;l1<ol2;l1+=2,l2+=2){
                               ang=2*pi*angle/size;
                               wr=cos(ang);
                               wi=-sin(ang);
                               xr=*l1+*l2;
                               xi=*(l1+1)+*(l2+1);
                               dr=*l1-*l2;
                               di=*(l1+1)-*(l2+1);
                               yr=dr*wr-di*wi;
                               yi=dr*wi+di*wr;
                               *(l1)=xr;*(l1+1)=xi;
                               *(l2)=yr;*(l2+1)=yi;
                               angle+=astep;
							   }
					  }
	 }
}

/////////////////////////////////////////////////////////
// data shuffling into bit-reversed order
// data: array of doubles:
// re(0),im(0),re(1),im(1),...,re(size-1),im(size-1)
// it means that size=array_length/2 !!
//
// Source: Rabiner-Gold: Theory and Application of DSP,
// Prentice Hall,1978 

void unshuffle(double *data, long size){

	long i,j,k,l,m;
	double re,im;


	l=size-1;
	m=size>>1;
	for (i=0,j=0; i<l ; i++){
		  if (i<j){
					re=data[j+j]; im=data[j+j+1];
					data[j+j]=data[i+i]; data[j+j+1]=data[i+i+1];
					data[i+i]=re; data[i+i+1]=im;
					}
		  k=m;
		  while (k<=j){
					j-=k;
					k>>=1;	
					}
		  j+=k;
      }
}

/////////////////////////////////////////////////////////
// in-place Radix-2 FFT for complex values
// data: array of doubles:
// re(0),im(0),re(1),im(1),...,re(size-1),im(size-1)
// it means that size=array_length/2 !!
// 
// output is in similar order, normalized by array length
//
// Source: see the routines it calls ...

void fft(double *data, long size){

	double *l, *end;

	dif_butterfly(data,size);
	unshuffle(data,size);

	end=data+size+size;
	for(l=data;l<end;l++){*l=*l/size;};
}
