/*
 ============================================================================
 Blackbox for Windows: Plugin SDK example used for making this plugin
 ============================================================================
 Copyright  2001-2002 The Blackbox for Windows Development Team
 http://desktopian.org/bb/ - #bb4win on irc.freenode.net
 ============================================================================

  Blackbox for Windows is free software, released under the
  GNU General Public License (GPL version 2 or later), with an extension
  that allows linking of proprietary modules under a controlled interface.
  What this means is that plugins etc. are allowed to be released
  under any license the author wishes. Please note, however, that the
  original Blackbox gradient math code used in Blackbox for Windows
  is available under the BSD license.

  http://www.fsf.org/licenses/gpl.html
  http://www.fsf.org/licenses/gpl-faq.html#LinkingOverControlledInterface
  http://www.xfree86.org/3.3.6/COPYRIGHT2.html#5

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  For additional license information, please read the included license.html
  

 ============================================================================
*/

#include "bbsysmon.h"

//needed precompiled libraries.
//#pragma comment(lib, "Blackbox.lib")
//#pragma comment(lib, "Psapi.Lib")
//#pragma comment(lib, "WS2_32.Lib")
//#pragma comment(lib, "wsock32.lib")
//#pragma warning(disable: 4786) // disable STL naming warnings when compiling

LPSTR szAppName = "BBSysmon";		// The name of our window class, etc.
LPSTR szVersion = "BBSysMon v0.8.1";	// Used in MessageBox titlebars

LPSTR szInfoVersion = "0.8.1";
LPSTR szInfoAuthor = "Mortar";
LPSTR szInfoRelDate = "2003-12-11";
LPSTR szInfoLink = "http://mortar.crackmonkey.us/\n#bb4win on irc.freenode.net";
LPSTR szInfoEmail = "mortar@crackmonkey.us";
char szBroamList[MAX_LINE_LENGTH];

//===========================================================================

int beginSlitPlugin(HINSTANCE hMainInstance, HWND hBBSlit)
{
	/* Since we were loaded in the slit we need to remember the Slit
	 * HWND and make sure we remember that we are in the slit ;)
	 */
	inSlit = true;
	hSlit = hBBSlit;

	// Start the plugin like normal now..
	return beginPlugin(hMainInstance);
}

//===========================================================================

//new xoblite "slit-aware" begin plugin function.  Use the same as beginSlitPlugin.
int beginPluginEx(HINSTANCE hMainInstance, HWND hBBSlit)
{
	/* Since we were loaded in the slit we need to remember the Slit
	 * HWND and make sure we remember that we are in the slit ;)
	 */
	inSlit = true;
	hSlit = hBBSlit;

	// Start the plugin like normal now..
	return beginPlugin(hMainInstance);
}

//===========================================================================

int beginPlugin(HINSTANCE hPluginInstance)
{
	WNDCLASS wc;
	hwndBlackbox = GetBBWnd();
	hInstance = hPluginInstance;

	// Register the window class...
	ZeroMemory(&wc,sizeof(wc));
	wc.lpfnWndProc = WndProc;			// our window procedure
	wc.hInstance = hPluginInstance;		// hInstance of .dll
	wc.lpszClassName = szAppName;		// our window class name
	if (!RegisterClass(&wc)) 
	{
		MessageBox(hwndBlackbox, "Error registering window class", szAppName, MB_OK | MB_ICONERROR | MB_TOPMOST);
		return 1;
	}

	// Get plugin and style settings...
	ReadRCSettings();
	if(!hSlit) inSlit = false;
	else inSlit = wantInSlit;
	GetStyleSettings();
	
	// Create the window...
	hwndBBSysMon = CreateWindowEx(
						WS_EX_TOOLWINDOW,								// window style
						szAppName,										// our window class name
						NULL,											// NULL -> does not show up in task manager!
						WS_POPUP | WS_CLIPCHILDREN | WS_CLIPSIBLINGS,	// window parameters
						xpos,											// x position
						ypos,											// y position
						width,											// window width
						height,											// window height
						NULL,											// parent window
						NULL,											// no menu
						hPluginInstance,								// hInstance of .dll
						NULL);
	if (!hwndBBSysMon)
	{						   
		MessageBox(0, "Error creating window", szAppName, MB_OK | MB_ICONERROR | MB_TOPMOST);
		// Unregister Blackbox messages...
		SendMessage(hwndBlackbox, BB_UNREGISTERMESSAGE, (WPARAM)hwndBBSysMon, (LPARAM)msgs);
		return 1;
	}

	if(inSlit && hSlit)// Yes, so Let's let BBSlit know.
		SendMessage(hSlit, SLIT_ADD, NULL, (LPARAM)hwndBBSysMon);
	else inSlit = false;
	// Initialize the Performance Monitor stuff
	InitBBSysMon();
	//check for windows 2000 or higher before using transparency
	if(!inSlit)
	{
		if (transparency)
		{
			SetWindowLong(hwndBBSysMon, GWL_EXSTYLE, WS_EX_TOOLWINDOW | WS_EX_LAYERED);
			BBSetLayeredWindowAttributes(hwndBBSysMon, NULL, (unsigned char)alpha, LWA_ALPHA);
		}
	}
	
	// Register to receive Blackbox messages...
	SendMessage(hwndBlackbox, BB_REGISTERMESSAGE, (WPARAM)hwndBBSysMon, (LPARAM)msgs);
	// Set magicDWord to make the window sticky (same magicDWord that is used by LiteStep)...
	SetWindowLong(hwndBBSysMon, GWL_USERDATA, magicDWord);
	// Make the window AlwaysOnTop?
	if (alwaysOnTop) SetWindowPos(hwndBBSysMon, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOACTIVATE | SWP_NOSIZE | SWP_NOMOVE);
	// Show the window and force it to update...
	ShowWindow(hwndBBSysMon, SW_SHOW);
	InvalidateRect(hwndBBSysMon, NULL, true);

	return 0;
}

//===========================================================================

void InitBBSysMon()
{
	UINT uResult;               // SetTimer's return value 

    dwId = 0;
    dwMajorVer = 0;
    dwMinorVer = 0;

	//Get Platform type
	_GetPlatformId (&dwId, &dwMajorVer, &dwMinorVer);
    
    if (dwId == VER_PLATFORM_WIN32_WINDOWS)
    {
        m_eOpSys = OPSYS_WIN_9X;
        m_pStatsObj = new CGet9XStats();
    }
    else if (dwId == VER_PLATFORM_WIN32_NT)
    {
        if (dwMajorVer <= 4)
        {
            m_eOpSys = OPSYS_WIN_NT4;
            m_pStatsObj = new CGetNTStats();
        }
        else if (dwMajorVer > 4)
        {
            m_eOpSys = OPSYS_WIN_NT5;
            m_pStatsObj = new CGetNTStats();
        }
    }
	uResult = SetTimer(hwndBBSysMon,		// handle to main window 
						IDT_TIMER,			// timer identifier 
						1000,				// 1-second interval 
						(TIMERPROC) NULL);	// no timer callback 

	uResult = SetTimer(hwndBBSysMon,		// handle to main window 
						IDT_DISK_TIMER,		// timer identifier 
						20000,				// 20-second interval 
						(TIMERPROC) NULL);	// no timer callback 

	uResult = SetTimer(hwndBBSysMon,		// handle to main window 
						IDT_UPTIME_TIMER,	// timer identifier 
						60000,				// 1 minute interval 
						(TIMERPROC) NULL);	// no timer callback

	ScreenWidth = GetSystemMetrics(SM_CXVIRTUALSCREEN);
	ScreenHeight = GetSystemMetrics(SM_CXVIRTUALSCREEN);

	getStaticCounters();	//always do this before processing the input string

	//process strFormat from the rc
	initParser();
	processInputString();
	processExtendedChar();
	setDriveSpaces();		//seed drive space values
	setUptime();			//seed uptime values
	setMainTimerValues();	//seed all main timer values
}

//===========================================================================

void endPlugin(HINSTANCE hPluginInstance)
{
	// Write the current plugin settings to the config file...
	WriteRCSettings();
	// Delete used StyleItems...
	if (myStyleItem) delete myStyleItem;
	if (myStyleItem2) delete myStyleItem2;
	// Delete the main plugin menu if it exists (PLEASE NOTE that this takes care of submenus as well!)
	if (myMenu){ DelMenu(myMenu); myMenu = NULL;}
		
	// Release our timer resources
	KillTimer(hwndBBSysMon, IDT_TIMER); 
	KillTimer(hwndBBSysMon, IDT_DISK_TIMER);
	KillTimer(hwndBBSysMon, IDT_UPTIME_TIMER);

	// Unregister Blackbox messages...
	SendMessage(hwndBlackbox, BB_UNREGISTERMESSAGE, (WPARAM)hwndBBSysMon, (LPARAM)msgs);
	
	if(inSlit && hSlit)
		SendMessage(hSlit, SLIT_REMOVE, NULL, (LPARAM)hwndBBSysMon);
	
	// Destroy our window...
	DestroyWindow(hwndBBSysMon);
	// Unregister window class...
	UnregisterClass(szAppName, hPluginInstance);
	//clean our parser
	cleanParser();
}

//===========================================================================

LPCSTR pluginInfo(int field)
{
	// pluginInfo is used by Blackbox for Windows to fetch information about
	// a particular plugin. At the moment this information is simply displayed
	// in an "About loaded plugins" MessageBox, but later on this will be
	// expanded into a more advanced plugin handling system. Stay tuned! :)

	switch (field)
	{
		case PLUGIN_NAME:
			return szAppName;		// Plugin name
		case PLUGIN_VERSION:
			return szInfoVersion;	// Plugin version
		case PLUGIN_AUTHOR:
			return szInfoAuthor;	// Author
		case PLUGIN_RELEASEDATE:
			return szInfoRelDate;	// Release date, preferably in yyyy-mm-dd format
		case PLUGIN_LINK:
			return szInfoLink;		// Link to author's website
		case PLUGIN_EMAIL:
			return szInfoEmail;		// Author's email

		// ==========
		//new xoblite code
		case PLUGIN_BROAMS:
		{
			strcpy(szBroamList, "@BBSysMonTransparent");
			strcat(szBroamList, "@BBSysMonStyleWindowLabel");
			strcat(szBroamList, "@BBSysMonStyleLabel");
			strcat(szBroamList, "@BBSysMonStyleClock");
			strcat(szBroamList, "@BBSysMonSnapToEdge");
			strcat(szBroamList, "@BBSysMonSlit");
			strcat(szBroamList, "@BBSysmonSaveSettings");
			strcat(szBroamList, "@BBSysmonReloadSettings");
			strcat(szBroamList, "@BBSysMonPluginToggle");
			strcat(szBroamList, "@BBSysMonPadPercentages");
			strcat(szBroamList, "@BBSysMonOnTop");
			strcat(szBroamList, "@BBSysMonJustifyRight");
			strcat(szBroamList, "@BBSysMonJustifyLeft");
			strcat(szBroamList, "@BBSysMonJustifyCenter");
			strcat(szBroamList, "@BBSysmonEditRC");
			strcat(szBroamList, "@BBSysMonAutoResize");
			strcat(szBroamList, "@BBSysMonAbout");
			return szBroamList;
		}
		//end new xoblite code

		// ==========

		default:
			return szVersion; // Fallback: Plugin name + version, e.g. "MyPlugin 1.0"
	}
}

//===========================================================================

LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
		// Window update process...
		case WM_PAINT:
		{
			makeDisplayString();
			drawTextMode(hwnd);
			return 0;
		}
		break;

		// ==========

		// If Blackbox sends a reconfigure message, load the new style settings and update the window...
		case BB_RECONFIGURE:
		{
			if (myMenu){ DelMenu(myMenu); myMenu = NULL;}
			GetStyleSettings();
			//setScreenPosition();
			InvalidateRect(hwndBBSysMon, NULL, true);
		}
		break;

		case WM_DISPLAYCHANGE:
		{
			if(!inSlit)
			{
				// IntelliMove(tm)... <g>
				// (c) 2003 qwilk
				//should make this a function so it can be used on startup in case resolution changed since
				//the last time blackbox was used.
				int relx, rely;
				int oldscreenwidth = ScreenWidth;
				int oldscreenheight = ScreenHeight;
				ScreenWidth = GetSystemMetrics(SM_CXVIRTUALSCREEN);
				ScreenHeight = GetSystemMetrics(SM_CXVIRTUALSCREEN);
				if (xpos > oldscreenwidth / 2)
				{
					relx = oldscreenwidth - xpos;
					xpos = ScreenWidth - relx;
				}
				if (ypos > oldscreenheight / 2)
				{
					rely = oldscreenheight - ypos;
					ypos = ScreenHeight - rely;
				}
				MoveWindow(hwndBBSysMon, xpos, ypos, width, height, true);
			}
		}
		break;

		// ==========

		// Broadcast messages (bro@m -> the bang killah! :D <vbg>)
/*		case BB_BROADCAST:		// original plugin code
		{
			char temp[MAX_LINE_LENGTH];
			strcpy(temp, (LPCSTR)lParam);

			if (!_stricmp(temp, "@BBShowPlugins")) // @BBShowPlugins: Show window and force update (global bro@m)
			{
				ShowWindow(hwndBBSysMon, SW_SHOW);
				InvalidateRect(hwndBBSysMon, NULL, true);
			}
			else if (!_stricmp(temp, "@BBHidePlugins")) // @BBHidePlugins: Hide window (global bro@m)
			{
				ShowWindow(hwndBBSysMon, SW_HIDE);
			}
			else if (!_stricmp(temp, "@myBroam")) // @myBroam is "our own" bro@m to set the temporary toolbar display
			{
				SendMessage(GetBBWnd(), BB_SETTOOLBARLABEL, 0, (LPARAM)"plugin sdk example: using the temporary toolbar display");
			}
		}
		break;
*/
		//====================

		// Broadcast messages (bro@m -> the bang killah! :D <vbg>)
		case BB_BROADCAST:
		{
			//===================
			//non-bbsysmon specific broams

			strcpy(szTemp, (LPCSTR)lParam);

			if (!_stricmp(szTemp, "@BBShowPlugins") &&  BBSysMonPluginToggle && !inSlit)
			{
				// Show window and force update...
				ShowWindow( hwndBBSysMon, SW_SHOW);
				InvalidateRect( hwndBBSysMon, NULL, true);
			}
			else if (!_stricmp(szTemp, "@BBHidePlugins") &&  BBSysMonPluginToggle && !inSlit)
			{
				// Hide window...
				ShowWindow( hwndBBSysMon, SW_HIDE);
			}

			//===================
			//bbsysmon specific

			// break out completely if the br0@m is not intended for bbsysmon
			else if(_stricmp(szTemp, "@BBSysMon") < 0)
				break;

			//===================

			else if (!_stricmp(szTemp, "@BBSysMonAbout"))
			{
				sprintf(szTemp, "%s\n\n 2003 %s\n\nSpecial thanks to:\nzenlc2000@yahoo.com\n\n%s",
						szVersion, szInfoEmail, szInfoLink);

				MSGBOXPARAMS msgbox;
				ZeroMemory(&msgbox, sizeof(msgbox));
				msgbox.cbSize = sizeof(msgbox);
				msgbox.hwndOwner = GetBBWnd();
				msgbox.hInstance = hInstance;
				msgbox.lpszText = szTemp;
				msgbox.lpszCaption = szAppName;
				msgbox.dwStyle = MB_OK | MB_SETFOREGROUND | MB_USERICON;
				msgbox.lpszIcon = MAKEINTRESOURCE(IDI_ICON1);
				msgbox.dwContextHelpId = 0;
				msgbox.lpfnMsgBoxCallback = NULL;
				msgbox.dwLanguageId = LANG_NEUTRAL;
				MessageBoxIndirect(&msgbox);
			}
			else if (!_stricmp(szTemp, "@BBSysMonStyleLabel"))
			{
				// Set the label attributes to the window style
				strcpy(windowStyle, "label");
				GetStyleSettings();
				InvalidateRect(hwndBBSysMon, NULL, true);
			}
			else if (!_stricmp(szTemp, "@BBSysMonStyleWindowLabel"))
			{
				// Set the windowLabel attributes to the window style
				strcpy(windowStyle, "windowlabel");
				GetStyleSettings();
				InvalidateRect(hwndBBSysMon, NULL, true);
			}
			else if (!_stricmp(szTemp, "@BBSysMonStyleClock"))
			{
				// Set the clock attributes to the window style
				strcpy(windowStyle, "clock");
				GetStyleSettings();
				InvalidateRect(hwndBBSysMon, NULL, true);
			}
			else if (!_stricmp(szTemp, "@BBSysMonJustifyLeft"))
			{
				justify = DT_LEFT;
				InvalidateRect(hwndBBSysMon, NULL, true);
			}
			else if (!_stricmp(szTemp, "@BBSysMonJustifyCenter"))
			{
				justify = DT_CENTER;
				InvalidateRect(hwndBBSysMon, NULL, true);
			}
			else if (!_stricmp(szTemp, "@BBSysMonJustifyRight"))
			{
				justify = DT_RIGHT;
				InvalidateRect(hwndBBSysMon, NULL, true);
			}
			else if (!_stricmp(szTemp, "@BBSysMonAutoResize"))
			{
				if (autoResize)
					 autoResize = false;
				else
					 autoResize = true;
			}
			else if (!_stricmp(szTemp, "@BBSysMonPadPercentages"))
			{
				if (padPercentages)
					 padPercentages = false;
				else
					 padPercentages = true;
			}
			else if (!_stricmp(szTemp, "@BBSysMonPluginToggle"))
			{
				// Hide window...
				if (BBSysMonPluginToggle)
					 BBSysMonPluginToggle = false;
				else
					 BBSysMonPluginToggle = true;
			}
			else if (!_stricmp(szTemp, "@BBSysMonOnTop"))
			{
				// Hide window...
				if (alwaysOnTop)
					 alwaysOnTop = false;
				else
					 alwaysOnTop = true;
				if(!inSlit)
				{
					if (alwaysOnTop) SetWindowPos( hwndBBSysMon, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOACTIVATE | SWP_NOSIZE | SWP_NOMOVE);
					else SetWindowPos( hwndBBSysMon, HWND_NOTOPMOST, 0, 0, 0, 0, SWP_NOACTIVATE | SWP_NOSIZE | SWP_NOMOVE);
				}
			}
			else if (!_stricmp(szTemp, "@BBSysMonSlit"))
			{
				// Does user want it in the slit...
				if (wantInSlit)
					 wantInSlit = false;
				else
					 wantInSlit = true;

				inSlit = wantInSlit;
				if(wantInSlit && hSlit)
					SendMessage(hSlit, SLIT_ADD, NULL, (LPARAM)hwndBBSysMon);
				else if(!wantInSlit && hSlit)
					SendMessage(hSlit, SLIT_REMOVE, NULL, (LPARAM)hwndBBSysMon);
				else
					inSlit = false;	

				if(!inSlit)
				{
					//check for windows 2000 or higher before using transparency
					if (transparency && (dwId == VER_PLATFORM_WIN32_NT)&&(dwMajorVer > 4))
					{
						SetWindowLong(hwndBBSysMon, GWL_EXSTYLE, WS_EX_TOOLWINDOW | WS_EX_LAYERED);
						BBSetLayeredWindowAttributes(hwndBBSysMon, NULL, (unsigned char)alpha, LWA_ALPHA);
					}
					else
						SetWindowLong(hwndBBSysMon, GWL_EXSTYLE, WS_EX_TOOLWINDOW);

					//set window on top is alwaysontop: is true
					if ( alwaysOnTop) SetWindowPos( hwndBBSysMon, HWND_TOPMOST, xpos, ypos, width, height, SWP_NOACTIVATE);
					else SetWindowPos( hwndBBSysMon, HWND_NOTOPMOST, xpos, ypos, width, height, SWP_NOACTIVATE);
				}
				else
					if(transparency) SetWindowLong(hwndBBSysMon, GWL_EXSTYLE, WS_EX_TOOLWINDOW);

				GetStyleSettings();
				//update window
				InvalidateRect(hwndBBSysMon, NULL, true);
			}
			else if (!_stricmp(szTemp, "@BBSysMonTransparent"))
			{
				// Set the transparent attributes to the window
				if (transparency)
					 transparency = false;
				else
					 transparency = true;
				
				//check for windows 2000 or higher before using transparency
				if(!inSlit)
				{
					if (transparency && (dwId == VER_PLATFORM_WIN32_NT)&&(dwMajorVer > 4))
					{
						SetWindowLong(hwndBBSysMon, GWL_EXSTYLE, WS_EX_TOOLWINDOW | WS_EX_LAYERED);
						BBSetLayeredWindowAttributes(hwndBBSysMon, NULL, (unsigned char)alpha, LWA_ALPHA);
					}
					else
						SetWindowLong(hwndBBSysMon, GWL_EXSTYLE, WS_EX_TOOLWINDOW);
				}
			}
			else if (!_stricmp(szTemp, "@BBSysMonSnapToEdge"))
			{
				// Set the transparent attributes to the window
				if (snapWindow)
					 snapWindow = false;
				else
					 snapWindow = true;
			}
			/*
			else if (!_stricmp(szTemp, "@BBSysMonTopLeft"))
			{
				strcpy(BBSysMonPlacement, "TopLeft");
				//ReadRCSettings();
				setScreenPosition();
				MoveWindow( hwndBBSysMon,  xpos,  ypos,  width,  height, true);
				bBackgroundExists = false;
				InvalidateRect( hwndBBSysMon, NULL, true);
			}
			else if (!_stricmp(szTemp, "@BBSysMonTopCenter"))
			{
				strcpy(BBSysMonPlacement, "TopCenter");
				//ReadRCSettings();
				setScreenPosition();
				MoveWindow( hwndBBSysMon,  xpos,  ypos,  width,  height, true);
				bBackgroundExists = false;
				InvalidateRect( hwndBBSysMon, NULL, true);
			}
			else if (!_stricmp(szTemp, "@BBSysMonTopRight"))
			{
				strcpy(BBSysMonPlacement, "TopRight");
				//ReadRCSettings();
				setScreenPosition();
				MoveWindow( hwndBBSysMon,  xpos,  ypos,  width,  height, true);
				bBackgroundExists = false;
				InvalidateRect( hwndBBSysMon, NULL, true);
			}
			else if (!_stricmp(szTemp, "@BBSysMonBottomLeft"))
			{
				strcpy(BBSysMonPlacement, "BottomLeft");
				//ReadRCSettings();
				setScreenPosition();
				MoveWindow( hwndBBSysMon,  xpos,  ypos,  width,  height, true);
				bBackgroundExists = false;
				InvalidateRect( hwndBBSysMon, NULL, true);
			}
			else if (!_stricmp(szTemp, "@BBSysMonBottomCenter"))
			{
				strcpy(BBSysMonPlacement, "BottomCenter");
				//ReadRCSettings();
				setScreenPosition();
				MoveWindow( hwndBBSysMon,  xpos,  ypos,  width,  height, true);
				bBackgroundExists = false;
				InvalidateRect( hwndBBSysMon, NULL, true);
			}
			else if (!_stricmp(szTemp, "@BBSysMonBottomRight"))
			{
				strcpy(BBSysMonPlacement, "BottomRight");
				//ReadRCSettings();
				setScreenPosition();
				MoveWindow( hwndBBSysMon,  xpos,  ypos,  width,  height, true);
				bBackgroundExists = false;
				InvalidateRect( hwndBBSysMon, NULL, true);
			}*/
			else if (!_stricmp(szTemp, "@BBSysmonEditRC"))
			{
				//char extensionsrc[MAX_LINE_LENGTH];
				//char preferredEditor[MAX_LINE_LENGTH];

				//strcpy(extensionsrc, extensionsrcPath());
				//strcpy(preferredEditor, ReadString(extensionsrcPath(), "blackbox.editor:", "notepad.exe"));
				BBExecute(GetDesktopWindow(), NULL,
					ReadString(extensionsrcPath(), "blackbox.editor:", "notepad.exe"),
					rcpath, NULL, SW_SHOWNORMAL, false);
			}
			else if (!_stricmp(szTemp, "@BBSysmonReloadSettings"))
			{
				//remove from slit before resetting window attributes
				bool wasInSlit = false;
				if(inSlit && hSlit)
					wasInSlit = true;
					//SendMessage(hSlit, SLIT_REMOVE, NULL, (LPARAM)hwndBBSysMon);

				ReadRCSettings();
				inSlit = wantInSlit;
				GetStyleSettings();
				

				getStaticCounters();	//always do this before processing the input string
				cleanParser();
				processInputString();
				processExtendedChar();
				setDriveSpaces();		//seed drive values
				setUptime();			//seed uptime values
				setMainTimerValues();	//seed all main timer values
				
				if(wasInSlit && inSlit && hSlit)
				{
					SetWindowLong(hwndBBSysMon, GWL_EXSTYLE, WS_EX_TOOLWINDOW);
					SetWindowPos(hwndBBSysMon, 0, 0, 0, width, height, SWP_NOACTIVATE | SWP_NOZORDER | SWP_NOMOVE);
					SendMessage(hSlit, SLIT_UPDATE, NULL, NULL);
				}
				else if(wasInSlit && (!inSlit || !hSlit))
					SendMessage(hSlit, SLIT_REMOVE, NULL, (LPARAM)hwndBBSysMon);
				else if(inSlit && hSlit)
				{
					SetWindowLong(hwndBBSysMon, GWL_EXSTYLE, WS_EX_TOOLWINDOW);
					SetWindowPos(hwndBBSysMon, 0, 0, 0, width, height, SWP_NOACTIVATE | SWP_NOZORDER | SWP_NOMOVE);
					SendMessage(hSlit, SLIT_ADD, NULL, (LPARAM)hwndBBSysMon);
				}
				else inSlit = false;

				//check for windows 2000 or higher before using transparency
				if(!inSlit)
				{
					if (transparency && (dwId == VER_PLATFORM_WIN32_NT)&&(dwMajorVer > 4))
					{
						SetWindowLong(hwndBBSysMon, GWL_EXSTYLE, WS_EX_TOOLWINDOW | WS_EX_LAYERED);
						BBSetLayeredWindowAttributes(hwndBBSysMon, NULL, (unsigned char)alpha, LWA_ALPHA);
					}
					else
						SetWindowLong(hwndBBSysMon, GWL_EXSTYLE, WS_EX_TOOLWINDOW);

					if (alwaysOnTop) SetWindowPos( hwndBBSysMon, HWND_TOPMOST, xpos, ypos, width, height, SWP_NOACTIVATE);
					else SetWindowPos( hwndBBSysMon, HWND_NOTOPMOST, xpos, ypos, width, height, SWP_NOACTIVATE);
				}

				InvalidateRect(hwndBBSysMon, NULL, true);
			}
			else if (!_stricmp(szTemp, "@BBSysmonSaveSettings"))
			{
				WriteRCSettings();
			}
			/*else if (!_stricmp(szTemp, "@BBSysmonResizeWindow"))
			{
				resizeWindow();
			}*/
		}
		break;

		// ==========

		case WM_WINDOWPOSCHANGING:
		{
			// Is SnapWindowToEdge enabled?
			if (!inSlit && snapWindow)
			{
				// Snap window to screen edges (if the last bool is false it uses the current DesktopArea)
				if(IsWindowVisible(hwnd)) SnapWindowToEdge((WINDOWPOS*)lParam, 10, true);
			}
		}
		break;

		// ==========

		// Save window position if it changes...
		case WM_WINDOWPOSCHANGED:
		{
				WINDOWPOS* windowpos = (WINDOWPOS*)lParam;
				xpos = windowpos->x;
				ypos = windowpos->y;
		}
		break;

		// ==========

		// Allow window to move...
		case WM_NCHITTEST:
		{
			//strcpy(BBSysMonPlacement, "\0");
			return HTCAPTION;
		}
		break;

		case WM_NCLBUTTONDOWN:
		{
			/* Please do not allow plugins to be moved in the slit.
			 * That's not a request..  Okay, so it is.. :-P
			 * I don't want to hear about people losing their plugins
			 * because they loaded it into the slit and then moved it to
			 * the very edge of the slit window and can't get it back..
			 */
			if(!inSlit)
				return DefWindowProc(hwnd,message,wParam,lParam);
		}
		break;

		case WM_NCLBUTTONUP: {} break;

		// ==========

		// Right mouse button clicked?
		case WM_NCRBUTTONUP:
		{	
			ShowMyMenu();
		}
		break;

		case WM_NCRBUTTONDOWN: {} break;

		//case WM_NCLBUTTONUP:
		//{
		//	strcpy(BBSysMonPlacement, "\0");
		//}
		//break;

		case WM_TIMER:
			{
				switch (wParam)
				{
				case IDT_TIMER:
					{
						setMainTimerValues();
						//redraw the window
						InvalidateRect(hwndBBSysMon, NULL, false);
					} break;

				case IDT_DISK_TIMER:
					{
						setDriveSpaces();
					} break;
				case IDT_UPTIME_TIMER:
					{
						setUptime();
					}break;
				}
			}
			break;
		
		//prevents alt+F4 to work outside of slit
		case WM_CLOSE:
			{
				return 0;
			}
			break;
 		// ==========
		default:
			return DefWindowProc(hwnd,message,wParam,lParam);
	}
	return 0;
}

//===========================================================================

void ShowMyMenu()
{
	bool tempBool = false;
	if (myMenu){ DelMenu(myMenu); myMenu = NULL;}

	windowStyleSubmenu = MakeMenu("Window Style");
	if(StrStrI(windowStyle, "label") != NULL && strlen(windowStyle) < 6) tempBool = true;
	MakeMenuItem(windowStyleSubmenu, "toolbar.label:", "@BBSysMonStyleLabel", tempBool);
	tempBool = false;
	if(StrStrI(windowStyle, "windowlabel") != NULL) tempBool = true;
	MakeMenuItem(windowStyleSubmenu, "toolbar.windowLabel:", "@BBSysMonStyleWindowLabel", tempBool);
	tempBool = false;
	if(StrStrI(windowStyle, "clock") != NULL) tempBool = true;
	MakeMenuItem(windowStyleSubmenu, "toolbar.clock:", "@BBSysMonStyleClock", tempBool);
	tempBool = false;
	
	justifySubmenu = MakeMenu("Justification");
	if(justify == DT_LEFT) tempBool = true;
	MakeMenuItem(justifySubmenu, "left", "@BBSysMonJustifyLeft", tempBool);
	tempBool = false;
	if(justify == DT_CENTER) tempBool = true;
	MakeMenuItem(justifySubmenu, "center", "@BBSysMonJustifyCenter", tempBool);
	tempBool = false;
	if(justify == DT_RIGHT) tempBool = true;
	MakeMenuItem(justifySubmenu, "right", "@BBSysMonJustifyRight", tempBool);
	tempBool = false;

	configSubmenu = MakeMenu("Configuration");
	MakeMenuItem(configSubmenu, "Auto Resize Window", "@BBSysMonAutoResize", autoResize);
	MakeMenuItem(configSubmenu, "Toggle With Plugins", "@BBSysMonPluginToggle", BBSysMonPluginToggle);
	MakeMenuItem(configSubmenu, "Pad % Values", "@BBSysMonPadPercentages", padPercentages);
	MakeMenuItem(configSubmenu, "Always on Top", "@BBSysMonOnTop", alwaysOnTop);
	if ((dwId == VER_PLATFORM_WIN32_NT)&&(dwMajorVer > 4))
		MakeMenuItem(configSubmenu, "Transparency", "@BBSysMonTransparent", transparency);
	if(hSlit) MakeMenuItem(configSubmenu, "In Slit", "@BBSysMonSlit", wantInSlit);
	MakeMenuItem(configSubmenu, "Snap Window To Edge", "@BBSysMonSnapToEdge", snapWindow);

	settingsSubmenu = MakeMenu("Settings");
	MakeMenuItem(settingsSubmenu, "Edit Settings", "@BBSysmonEditRC", false);
	MakeMenuItem(settingsSubmenu, "Reload Settings", "@BBSysmonReloadSettings", false);
	MakeMenuItem(settingsSubmenu, "Save Settings", "@BBSysmonSaveSettings", false);
	
	//placementSubmenu = MakeMenu("Placement");
	//MakeMenuItem(placementSubmenu, "Top Left", "@BBSysMonTopLeft", !strcmp(BBSysMonPlacement, "TopLeft"));
	//MakeMenuItem(placementSubmenu, "Top Center", "@BBSysMonTopCenter", !strcmp(BBSysMonPlacement, "TopCenter"));
	//MakeMenuItem(placementSubmenu, "Top Right", "@BBSysMonTopRight", !strcmp(BBSysMonPlacement, "TopRight"));
	//MakeMenuItem(placementSubmenu, "Bottom Left", "@BBSysMonBottomLeft", !strcmp(BBSysMonPlacement, "BottomLeft"));
	//MakeMenuItem(placementSubmenu, "Bottom Center", "@BBSysMonBottomCenter", !strcmp(BBSysMonPlacement, "BottomCenter"));
	//MakeMenuItem(placementSubmenu, "Bottom Right", "@BBSysMonBottomRight", !strcmp(BBSysMonPlacement, "BottomRight"));
	
	myMenu = MakeMenu("BBSysMon");
	MakeMenuItem(myMenu, "About", "@BBSysMonAbout", false);
	//MakeMenuItem(myMenu, "Resize Window", "@BBSysMonResizeWindow", false);
	
	//MakeMenuNOP(myMenu, NULL);
	MakeSubmenu(myMenu, windowStyleSubmenu, "Window Style");
	MakeSubmenu(myMenu, justifySubmenu, "Justification");
	MakeSubmenu(myMenu, configSubmenu, "Configuration");
	MakeSubmenu(myMenu, settingsSubmenu, "Settings");
	//zMakeSubmenu(myMenu, placementSubmenu, "Placement");
	ShowMenu(myMenu);
}
//===========================================================================

void GetStyleSettings()
{
	// Get the path to the current style file from Blackbox, if needed...
	if(IsInString(GetBBVersion(), "bb")) //...new xoblite code for the "NULL aware" thing.
		strcpy(stylepath, "");
	else
		strcpy(stylepath, stylePath());

	// ...font settings...
	if(useRCFont) strcpy(fontFace, rcFont);
	else
	{
		strcpy(fontFace, ReadString(stylepath, "toolbar.font:", ""));
		if (!_stricmp(fontFace, "")) strcpy(fontFace, ReadString(stylepath, "*font:", "Tahoma"));
	}

	if(useRCFontHeight) fontSize = fontHeight;
	else
	{
		fontSize = ReadInt(stylepath, "toolbar.fontheight:", 0);
		if (fontSize <= 0) fontSize = ReadInt(stylepath, "*fontHeight:", 12);
	}

	//fontSize = 12;

	// ...and some additional parameters
	bevelWidth = ReadInt(stylepath, "bevelWidth:", 2);
	borderWidth = ReadInt(stylepath, "borderWidth:", 1);

	// Get the applicable color settings from the current style...
	backColor = ReadColor(stylepath, "toolbar.color:", "#000000");
	backColorTo = ReadColor(stylepath, "toolbar.colorTo:", "#FFFFFF");
	borderColor = ReadColor(stylepath, "borderColor:", "#000000");

	// ...gradient type, bevel etc. from toolbar:(using a StyleItem)...
	char tempstyle[MAX_LINE_LENGTH];
	strcpy(tempstyle, ReadString(stylepath, "toolbar:", "Flat Gradient Vertical"));
	if (myStyleItem) delete myStyleItem;
	myStyleItem = new StyleItem;
	ParseItem(tempstyle, myStyleItem);

	//get inner rect style info, or set it to toolbar.color etc, if it is parent relative
	//check for plugin's window style
	char styleStr[50];
	char styleStrColor[50];
	char styleStrColorTo[50];
	char styleStrTextColor[50];
	if(StrStrI(windowStyle, "label") != NULL  && strlen(windowStyle) < 6)
	{
		strcpy(styleStr, "toolbar.label:");
		strcpy(styleStrColor, "toolbar.label.color:");
		strcpy(styleStrColorTo, "toolbar.label.colorto:");
		strcpy(styleStrTextColor, "toolbar.label.textColor:");
	}
	else if(StrStrI(windowStyle, "windowlabel") != NULL)
	{
		strcpy(styleStr, "toolbar.windowlabel:");
		strcpy(styleStrColor, "toolbar.windowlabel.color:");
		strcpy(styleStrColorTo, "toolbar.windowlabel.colorto:");
		strcpy(styleStrTextColor, "toolbar.windowlabel.textColor:");
	}
	else
	{
		strcpy(styleStr, "toolbar.clock:");
		strcpy(styleStrColor, "toolbar.clock.color:");
		strcpy(styleStrColorTo, "toolbar.clock.colorto:");
		strcpy(styleStrTextColor, "toolbar.clock.textColor:");
	}

	// ...gradient type, bevel etc. from toolbar.clock:(using a StyleItem)...
	char tempstyle2[MAX_LINE_LENGTH];
	strcpy(tempstyle2, ReadString(stylepath, styleStr, "parentrelative"));
	if (!IsInString(tempstyle2, "parentrelative"))
	{
		if (myStyleItem2) delete myStyleItem2;
		myStyleItem2 = new StyleItem;			
		ParseItem(tempstyle2, myStyleItem2);

		backColor2 = ReadColor(stylepath, styleStrColor, "#000000");
		backColorTo2 = ReadColor(stylepath, styleStrColorTo, "#000000");
		if(useRCFontColor) fontColor = textColor;
		else fontColor = ReadColor(stylepath, styleStrTextColor, "#FFFFFF");
		labelIsPR = false;
	}
	else //fallback
	{
		if (myStyleItem2) delete myStyleItem2;	//else use the the toolbar: settings
		myStyleItem2 = new StyleItem;
		ParseItem(tempstyle, myStyleItem2);	//use original tempstyle if "parentrelative"
		backColor2 = backColor;
		backColorTo2 = backColorTo;
		if(useRCFontColor) fontColor = textColor;
		else fontColor = ReadColor(stylepath, "toolbar.textColor:", "#FFFFFF");
		labelIsPR = true;
	}

/*
	// ...gradient type, bevel etc. from toolbar.clock:(using a StyleItem)...
	char tempstyle2[MAX_LINE_LENGTH];
	strcpy(tempstyle2, ReadString(stylepath, styleStr, "parentrelative"));
	//check to make sure the config exists && for parentrelative
	if (!IsInString("", tempstyle2)&&!IsInString(tempstyle2, "parentrelative"))
	{
		if (myStyleItem2) delete myStyleItem2;	//if everything is found in toolbar.clock: then make a new StyleItem
		myStyleItem2 = new StyleItem;			
		ParseItem(tempstyle2, myStyleItem2);

		//check that the config even exists, this prevents crashes with the BB0.8x Read inconsistencies
		if (!IsInString("", ReadString(stylepath, styleStrColor, "")))
			backColor2 = ReadColor(stylepath, styleStrColor, "#000000");
		else
   			backColor2 = ReadColor(stylepath, "toolbar.color:", "#FFFFFF");

		if (!IsInString("", ReadString(stylepath, styleStrColorTo, "")))
			backColorTo2 = ReadColor(stylepath, styleStrColorTo, "#000000");
		else
			backColorTo2 = ReadColor(stylepath, "toolbar.colorTo:", "#000000");

		fontColor = ReadColor(stylepath, styleStrTextColor, "#FFFFFF");
		labelIsPR = false;
	}
	else //fallback
	{
		if (myStyleItem2) delete myStyleItem2;	//else use the the toolbar: settings
		myStyleItem2 = new StyleItem;
		ParseItem(tempstyle, myStyleItem2);	//use original tempstyle if "parentrelative"
		backColor2 = backColor;			//have to do this if parent relative found, it seems bb4win uses
		backColorTo2 = backColorTo;		//the toolbar.color if parent relative is found for toolbar.clock
		fontColor = ReadColor(stylepath, "toolbar.textColor:", "#FFFFFF");
		labelIsPR = true;
	}
*/
}

//===========================================================================

void ReadRCSettings()
{
	char temp[MAX_LINE_LENGTH], path[MAX_LINE_LENGTH], defaultpath[MAX_LINE_LENGTH];
	int nLen;

	// First we look for the config file in the same folder as the plugin...
	GetModuleFileName(hInstance, rcpath, sizeof(rcpath));
	nLen = strlen(rcpath) - 1;
	while (nLen >0 && rcpath[nLen] != '\\') nLen--;
	rcpath[nLen + 1] = 0;
	strcpy(temp, rcpath);
	strcpy(path, rcpath);
	strcat(temp, "BBSysmon.rc");
	strcat(path, "BBSysmonrc");
	// ...checking the two possible filenames BBSysmon.rc and BBSysmonrc ...
	if (FileExists(temp)) strcpy(rcpath, temp);
	else if (FileExists(path)) strcpy(rcpath, path);
	// ...if not found, we try the Blackbox directory...
	else
	{
		// ...but first we save the default path (BBSysmon.rc in the same
		// folder as the plugin) just in case we need it later (see below)...
		strcpy(defaultpath, temp);
		GetBlackboxPath(rcpath, sizeof(rcpath));
		strcpy(temp, rcpath);
		strcpy(path, rcpath);
		strcat(temp, "BBSysmon.rc");
		strcat(path, "BBSysmonrc");
		if (FileExists(temp)) strcpy(rcpath, temp);
		else if (FileExists(path)) strcpy(rcpath, path);
		else // If no config file was found, we use the default path and settings, and return
		{
			strcpy(rcpath, defaultpath);
			xpos = ypos = 10;
			width = 150;
			height = 30;
			alwaysOnTop = true;
			snapWindow = true;
			transparency = false;
			alpha = 160;
			BBSysMonPluginToggle = false;
			wantInSlit = true;
			autoResize = false;
			padPercentages = true;
			strcpy(windowStyle, "windowlabel");
			justify = DT_LEFT;
			strcpy(strFormat, "CPU: %C - RAM: %R");

			//textColor = NULL;
			//strcpy(BBSysMonPlacement, "\0");
			WriteRCSettings(); //write a file for editing later
			return;
		}
	}

	strcpy(temp, ReadString(rcpath, "bbsysmon.textColor:", ""));
	if (!_stricmp(temp, "")) useRCFontColor = false;
	else
	{
		textColor = ReadColor(rcpath, "bbsysmon.textColor:", "#000000");
		useRCFontColor = true;
	}

	strcpy(temp, ReadString(rcpath, "bbsysmon.font:", ""));
	if (!_stricmp(temp, "")) useRCFont = false;
	else
	{
		strcpy(rcFont, temp);
		useRCFont = true;
	}

	fontHeight = ReadInt(rcpath, "bbsysmon.fontHeight:", 0);
	if(fontHeight <= 0) useRCFontHeight = false;
	else useRCFontHeight = true;

	// If a config file was found we read the plugin settings from the file...
	xpos = ReadInt(rcpath, "bbsysmon.x:", 10);
	ypos = ReadInt(rcpath, "bbsysmon.y:", 10);
	if (xpos >= GetSystemMetrics(SM_CXVIRTUALSCREEN)) xpos = 10;
	if (ypos >= GetSystemMetrics(SM_CXVIRTUALSCREEN)) ypos = 10;
	width = ReadInt(rcpath, "bbsysmon.width:", 150);
	height = ReadInt(rcpath, "bbsysmon.height:", 30);
	alwaysOnTop = ReadBool(rcpath, "bbsysmon.alwaysontop:", true);
	transparency = ReadBool(rcpath, "bbsysmon.transparency:", false);
	alpha = ReadInt(rcpath, "bbsysmon.alpha:", 160);
	snapWindow = ReadBool(rcpath, "bbsysmon.snapwindow:", true);

	autoResize = ReadBool(rcpath, "bbsysmon.autoResize:", false);
	padPercentages = ReadBool(rcpath, "bbsysmon.padPercentages:", true);
	//BBSysMonHidden = ReadBool(rcpath, "bbsysmon.hidden:", false);
	//BBSysMonWidthPercent = ReadInt(rcpath, "bbsysmon.widthPercent:", 100);
	//strcpy(BBSysMonPlacement, ReadString(rcpath, "bbsysmon.placement:", "\0"));

	BBSysMonPluginToggle = ReadBool(rcpath, "bbsysmon.pluginToggle:", false);

	if(ReadString(rcpath, "bbsysmon.inSlit:", NULL) == NULL) wantInSlit = true;
	else wantInSlit = ReadBool(rcpath, "bbsysmon.inSlit:", true);

	strcpy(windowStyle, ReadString(rcpath, "bbsysmon.windowStyle:", "windowlabel"));
	if(((StrStrI(windowStyle, "label") == NULL) || ((StrStrI(windowStyle, "label") != NULL) && (strlen(windowStyle) > 5))) 
		&& (StrStrI(windowStyle, "windowlabel") == NULL) && (StrStrI(windowStyle, "clock") == NULL))
		strcpy(windowStyle, "windowLabel");

	strcpy(szTemp, ReadString(rcpath, "bbsysmon.justification:", "left"));
	if(StrStrI(szTemp, "right") != NULL)
		justify = DT_RIGHT;
	else if(StrStrI(szTemp, "center") != NULL)
		justify = DT_CENTER;
	else justify = DT_LEFT;

	strcpy(strFormat, ReadString(rcpath, "bbsysmon.strFormat:", "CPU: %C - RAM: %R"));

	//just in case IP changes.  User can reload to get the new one.
	mgetIp();
}

//===========================================================================

void WriteRCSettings()
{
	WriteInt(rcpath, "bbsysmon.x:", xpos);
	WriteInt(rcpath, "bbsysmon.y:", ypos);
	WriteInt(rcpath, "bbsysmon.width:", width);
	WriteInt(rcpath, "bbsysmon.height:", height);
	WriteBool(rcpath, "bbsysmon.alwaysontop:", alwaysOnTop);
	WriteBool(rcpath, "bbsysmon.transparency:", transparency);
	WriteInt(rcpath, "bbsysmon.alpha:", alpha);
	WriteBool(rcpath, "bbsysmon.snapWindow:", snapWindow);
	WriteBool(rcpath, "bbsysmon.pluginToggle:", BBSysMonPluginToggle);
	WriteBool(rcpath, "bbsysmon.inSlit:", wantInSlit);
	WriteString(rcpath, "bbsysmon.windowStyle:", windowStyle);

	if(justify == DT_LEFT)
		WriteString(rcpath, "bbsysmon.justification:", "left");
	else if(justify == DT_CENTER)
		WriteString(rcpath, "bbsysmon.justification:", "center");
	else
		WriteString(rcpath, "bbsysmon.justification:", "right");
	
	WriteBool(rcpath, "bbsysmon.autoResize:", autoResize);
	WriteBool(rcpath, "bbsysmon.padPercentages:", padPercentages);
	WriteString(rcpath, "bbsysmon.strFormat:", strFormat);

	if(useRCFontColor)
		WriteColor(rcpath, "bbsysmon.textColor:", textColor);
	else
		WriteString(rcpath, "bbsysmon.textColor:", "");

	if(useRCFontHeight)
		WriteInt(rcpath, "bbsysmon.fontHeight:", fontHeight);
	else
		WriteString(rcpath, "bbsysmon.fontHeight:", "");

	if(useRCFont)
		WriteString(rcpath, "bbsysmon.font:", rcFont);
	else
		WriteString(rcpath, "bbsysmon.font:", "");
	/*
	static char szTemp[MAX_LINE_LENGTH];
	static char temp[8];
	DWORD retLength = 0;
	
	// Write plugin settings to config file, using path found in ReadRCSettings()...
	HANDLE file = CreateFile(rcpath, GENERIC_WRITE, FILE_SHARE_READ, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
	if (file)
	{
		sprintf(szTemp, "bbsysmon.x: %d\r\n", xpos, temp);
		WriteFile(file, szTemp, strlen(szTemp), &retLength, NULL);

		sprintf(szTemp, "bbsysmon.y: %d\r\n", ypos, temp);
		WriteFile(file, szTemp, strlen(szTemp), &retLength, NULL);

		sprintf(szTemp, "bbsysmon.width: %d\r\n", width, temp);
		WriteFile(file, szTemp, strlen(szTemp), &retLength, NULL);

		sprintf(szTemp, "bbsysmon.height: %d\r\n", height, temp);
		WriteFile(file, szTemp, strlen(szTemp), &retLength, NULL);

		(alwaysOnTop) ? strcpy(temp, "true") : strcpy(temp, "false");
		sprintf(szTemp, "bbsysmon.alwaysontop: %s\r\n", temp);
 		WriteFile(file, szTemp, strlen(szTemp), &retLength, NULL);

		(transparency) ? strcpy(temp, "true") : strcpy(temp, "false");
		sprintf(szTemp, "bbsysmon.transparency: %s\r\n", temp);
 		WriteFile(file, szTemp, strlen(szTemp), &retLength, NULL);

		sprintf(szTemp, "bbsysmon.alpha: %d\r\n", alpha, temp);
		WriteFile(file, szTemp, strlen(szTemp), &retLength, NULL);

		(snapWindow) ? strcpy(temp, "true") : strcpy(temp, "false");
		sprintf(szTemp, "bbsysmon.snapwindow: %s\r\n", temp);
 		WriteFile(file, szTemp, strlen(szTemp), &retLength, NULL);
	
		//sprintf(szTemp, "bbsysmon.widthPercent: %d\r\n", BBSysMonWidthPercent);
		//WriteFile(file, szTemp, strlen(szTemp), &retLength, NULL);

		//sprintf(szTemp, "bbsysmon.placement: %s\r\n", BBSysMonPlacement);
		//WriteFile(file, szTemp, strlen(szTemp), &retLength, NULL);

		//(BBSysMonHidden) ? strcpy(temp, "true") : strcpy(temp, "false");
		//sprintf(szTemp, "bbsysmon.hidden: %s\r\n", temp);
		//WriteFile(file, szTemp, strlen(szTemp), &retLength, NULL);
		(BBSysMonPluginToggle) ? strcpy(temp, "true") : strcpy(temp, "false");
		sprintf(szTemp, "bbsysmon.pluginToggle: %s\r\n", temp);
		WriteFile(file, szTemp, strlen(szTemp), &retLength, NULL);

		(wantInSlit) ? strcpy(temp, "true") : strcpy(temp, "false");
		sprintf(szTemp, "bbsysmon.inSlit: %s\r\n", temp);
 		WriteFile(file, szTemp, strlen(szTemp), &retLength, NULL);

		sprintf(szTemp, "bbsysmon.windowStyle: %s\r\n", windowStyle);
		WriteFile(file, szTemp, strlen(szTemp), &retLength, NULL);

		if(justify == DT_LEFT)
			sprintf(szTemp, "bbsysmon.justification: %s\r\n", "left");
		else if(justify == DT_CENTER)
			sprintf(szTemp, "bbsysmon.justification: %s\r\n", "center");
		else
			sprintf(szTemp, "bbsysmon.justification: %s\r\n", "right");
		WriteFile(file, szTemp, strlen(szTemp), &retLength, NULL);

		(autoResize) ? strcpy(temp, "true") : strcpy(temp, "false");
		sprintf(szTemp, "bbsysmon.autoResize: %s\r\n", temp);
 		WriteFile(file, szTemp, strlen(szTemp), &retLength, NULL);

		(padPercentages) ? strcpy(temp, "true") : strcpy(temp, "false");
		sprintf(szTemp, "bbsysmon.padPercentages: %s\r\n", temp);
 		WriteFile(file, szTemp, strlen(szTemp), &retLength, NULL);

		sprintf(szTemp, "bbsysmon.strFormat: %s\r\n", strFormat);
		WriteFile(file, szTemp, strlen(szTemp), &retLength, NULL);

	}
	CloseHandle(file);
	*/
}

//===========================================================================


int WINAPI _GetPlatformId(DWORD *pdwId, DWORD *pdwMajorVer, DWORD *pdwMinorVer)
{
	OSVERSIONINFO  osvinfo;
	ZeroMemory(&osvinfo, sizeof(osvinfo));
	osvinfo.dwOSVersionInfoSize = sizeof (osvinfo);
	GetVersionEx(&osvinfo);
	*pdwId       = osvinfo.dwPlatformId;
	*pdwMajorVer = osvinfo.dwMajorVersion;
	*pdwMinorVer = osvinfo.dwMinorVersion;
	return 0;
}


//=========================================================================


ULONGLONG GetDiskFreeSpaceBytes(LPCTSTR szDrive)
{
	ULARGE_INTEGER fba, tnob, tnofb;
	if(GetDiskFreeSpaceEx(szDrive, &fba, &tnob, &tnofb) != 0)
		return fba.QuadPart / (DWORD)1024 / (DWORD)1024;
	else
		return 0;
}

//=========================================================================

//get one ip from localhost, can change to get more ip's
void mgetIp()
{
	WORD wVersionRequested;
	WSADATA wsaData;
	char name[255];
	PHOSTENT hostinfo;
	wVersionRequested = MAKEWORD( 1, 1 );
	//char *ip;

	if ( WSAStartup( wVersionRequested, &wsaData ) == 0 )
		if( gethostname ( name, sizeof(name)) == 0)
		{
			//printf("Host name: %s\n", name);

			if((hostinfo = gethostbyname(name)) != NULL)
			{
				//this will count through all Ip's found if you use the while loop
				//note that mLocalhost is only a 'char *mLocalhost', you will need to store it in a char **mLocalhost

				//int nCount = 0;
				//while(hostinfo->h_addr_list[nCount])
				//{
				mLocalhost = inet_ntoa(*(struct in_addr *)hostinfo->h_addr_list[0]);
				//}
			}
		}
	//return ip;
}

//=========================================================================

//for drawing the screen in text mode
void drawTextMode(HWND hwnd)
{
	// Create buffer hdc's, bitmaps etc.
	PAINTSTRUCT ps;  RECT r;

	//get screen buffer
    HDC hdc_scrn = BeginPaint(hwnd, &ps);

	//retrieve the coordinates of the window's client area.
	GetClientRect(hwnd, &r);

	//to prevent flicker of the display, we draw to memory first,
    //then put it on screen in one single operation. This is like this:

    //first get a new 'device context'
	HDC hdc = CreateCompatibleDC(NULL);

	//if(firstPass) hdc = CreateCompatibleDC(NULL); //will reset firstPass=false later in one more check
	//else hdc = GetDC(hwnd);
	//else GetDC(hwnd);

	HBITMAP bufbmp = CreateCompatibleBitmap(hdc_scrn, r.right, r.bottom);
    SelectObject(hdc, bufbmp);
	
	if(!inSlit)
	{
		//Make background gradient
		MakeGradient(hdc, r, myStyleItem->type,
					backColor, backColorTo,
					myStyleItem->interlaced,
					myStyleItem->bevelstyle,
					myStyleItem->bevelposition,
					bevelWidth, borderColor,
					borderWidth);
	}
	
	SetBkMode(hdc, TRANSPARENT);
    SetTextColor(hdc, fontColor);

	//Paint second background according to toolbar.label.color: and toolbar.label.colorto:
	if(!labelIsPR || inSlit)
	{
		if(!inSlit)
		{
			r.left = r.left + (bevelWidth + borderWidth);
			r.top = r.top + (bevelWidth + borderWidth);
			r.bottom = (r.bottom - (bevelWidth + borderWidth));
			r.right = (r.right - (bevelWidth + borderWidth));
		}

		//draw second background according to toolbar.label.color: and toolbar.label.colorto:
		MakeGradient(hdc, r, myStyleItem2->type,
					backColor2, backColorTo2,
					myStyleItem2->interlaced,
					myStyleItem2->bevelstyle,
					myStyleItem2->bevelposition,
					bevelWidth, borderColor, 0);

		r.top = r.top + bevelWidth;
		r.left = r.left + bevelWidth + 2;
		r.right = r.right - (bevelWidth + 2);
		r.bottom = r.bottom - bevelWidth;
	}

	//if not parentrelative only pointer according to bevelWidth because it was moved by the second draw allready
	//if (!labelIsPR)
	//{
	//	r.top = r.top + bevelWidth;
	//	r.left = r.left + bevelWidth + 2;
	//	r.right = r.right - (bevelWidth + 2);
	//	r.bottom = r.bottom - bevelWidth;
	//}
	//else the pointer is moved with bevelWidth + borderWidth, with a padding of 2 on the left and right for text spacing
	else
	{
		r.top = r.top + bevelWidth + borderWidth;
		r.left = r.left + bevelWidth + borderWidth + 2;
		r.right = r.right - (bevelWidth + borderWidth + 2);
		r.bottom = r.bottom - (bevelWidth + borderWidth);
	}

	//Create font and draw some text
	HGDIOBJ otherfont = SelectObject(hdc,
					CreateFont(fontSize,
					0, 0, 0, FW_NORMAL,
					false, false, false,
					DEFAULT_CHARSET,
					OUT_DEFAULT_PRECIS,
					CLIP_DEFAULT_PRECIS,
					DEFAULT_QUALITY,
					DEFAULT_PITCH, fontFace));

	int newWidth;
	int newHeight;
	if(autoResize)
	{
		int bLeft;
		int bTop;
		if(labelIsPR || inSlit)
		{
			bLeft = (2 * borderWidth) + (2 * bevelWidth) + 4; 
			bTop = (2 * borderWidth) + (2 * bevelWidth);
		}
		else
		{
			bLeft = (2 * borderWidth) + (4 * bevelWidth) + 4;
			bTop = (2 * borderWidth) + (4 * bevelWidth);
		}
		int numOfLines;
		int mySizeX;
		SIZE sz;
		strcpy(szTemp, getStringInfo(displayString, &mySizeX, &numOfLines));
		GetTextExtentPoint32(hdc, szTemp, mySizeX, &sz);
		newWidth = sz.cx + bLeft;
		newHeight = (sz.cy * numOfLines) + bTop;
	}

	if (!useNewLine)
		DrawText(hdc, displayString, strlen(displayString), &r,
				justify | DT_EXPANDTABS | DT_VCENTER | DT_SINGLELINE);
	else
		DrawText(hdc, displayString, strlen(displayString), &r,
				justify | DT_EXPANDTABS /*| DT_VCENTER DT_SINGLELINE*/);

	DeleteObject(SelectObject(hdc, otherfont));	

	// Finally, copy from the paint buffer to the window...
	BitBlt(hdc_scrn, 0, 0, width, height, hdc, 0, 0, SRCCOPY);

	// Remember to delete all objects!
	DeleteObject(SelectObject(hdc, bufbmp));//gdi: first select and delete the bmp
	DeleteDC(hdc);         //gdi: now delete the dc

	EndPaint(hwnd, &ps);
	
	if(autoResize)
	{
		if(resizeMyWindow(newWidth, newHeight))
		InvalidateRect(hwndBBSysMon, NULL, true);
	}
}

//=========================================================================
//set screen position based on a placement choice
/*void setScreenPosition()
{
	if (strcmp(BBSysMonPlacement, "\0")!=0)
	{
		ScreenWidth = GetSystemMetrics(SM_CXSCREEN);
		ScreenHeight = GetSystemMetrics(SM_CYSCREEN);
		if (strcmp(BBSysMonPlacement, "TopLeft")==0)
		{
			xpos = 0;
			ypos = 0;
		}
		else if (strcmp(BBSysMonPlacement, "TopRight")==0)
		{
			xpos = ScreenWidth - width;
			ypos = 0;
		}
		else if (strcmp(BBSysMonPlacement, "BottomLeft")==0)
		{
			xpos = 0;
			ypos = ScreenHeight - height;
		}
		else if (strcmp(BBSysMonPlacement, "BottomCenter")==0)
		{
			xpos = (ScreenWidth - width) / 2;
			ypos = ScreenHeight - height;
		}
		else if (strcmp(BBSysMonPlacement, "BottomRight")==0)
		{
			xpos = ScreenWidth - width;
			ypos = ScreenHeight - height;
		}
		else
		{
			xpos = (ScreenWidth - width) / 2;
			ypos = 0;
		}
	}
}
*/

//=========================================================================

//so there you just use BBSLWA like normal SLWA
//(c)grischka
BOOL WINAPI BBSetLayeredWindowAttributes(HWND hwnd, COLORREF crKey, BYTE bAlpha, DWORD dwFlags)
{
	static BOOL (WINAPI *pSLWA)(HWND, COLORREF, BYTE, DWORD);
	static int f=0;
	for (;;) {
		if (2==f)   return pSLWA(hwnd, crKey, bAlpha, dwFlags);
		// if it's not there, just do nothing and report success
		if (f)      return TRUE;
		*(FARPROC*)&pSLWA = GetProcAddress(GetModuleHandle("USER32"), "SetLayeredWindowAttributes");
		f = pSLWA ? 2 : 1;
	}
}

//=========================================================================

//prosess bbsysmon.strFormat: from the RC file.  And set up a string list
void processInputString()
{
	strcpy(szTemp, strFormat);	//seems strFormat gets changed when it is strtok()ed.  So we let it mess with a temp copy.
	char* pch;					//Our token pointer

	int trim = 0;				//Some identifiers are different sizes, so we need a trim condition.

	//special case:  when a token is on the left side of the % sign,
	//which can only happen at the beginning of the input string.
	pch = strtok(szTemp, "%");		//get first token for loop
	if(strFormat[0] != '%')
	{
		currentPos = currentPos + strlen(pch);
		//printf(pch);
		tokenString[newArrayPos] =  new char[strlen(pch) + 1];
		strcpy(tokenString[newArrayPos], pch);
		newArrayPos++;
		numOfRegStrings++;
		pch = strtok (NULL, "%");		//get the next token for the loop
	}
	
	//Now we treat every token as if they are on the right side of the percent sign
	//There is always a token on the right, sometimes it is only the identifier though.  :)
	while (pch != NULL)
	{
		if(strFormat[currentPos + 1] == 'C')			//CPU Counter Found
		{
			if(cpuCntrPtrPos == -1)
			{
				//printf("Cfound->");
				numOfCounters++;
				cpuCntrPtrPos = newArrayPos;
				tokenString[newArrayPos] =  new char[MAXCPUCNTRSIZE];
				strcpy(tokenString[newArrayPos], "cpu");	//this will not need to be copied, actually make equal to \0
				newArrayPos++;									//the counters will be updated to the dynamic string in another function
			}											//just make it the size of the MAXCPUCOUNTERSIZE for example
			trim = 1;
		}
		else if(strFormat[currentPos + 1] == 'D')	//Directory found, only if a dir A-Z if specified.  Case sensitive.
		{
			if((strFormat[currentPos + 2] >= 65) && (strFormat[currentPos + 2] <= 90))	//A-Z or 65 to 90.  Brad's [Mortar] handy dandy ascii trick
			{
				if(dirCntrPtrPos[strFormat[currentPos + 2] - 65] == -1)
				{
					//printf("Dfound->");
					numOfCounters++;
					dirCntrPtrPos[strFormat[currentPos + 2] - 65] = newArrayPos;		//Some more ascii fun, look closely.  (hint..What is the value of strFormat[currentPos + 2] here?)
					tokenString[newArrayPos] =  new char[MAXDRIVECOUNTERSIZE];
					strcpy(tokenString[newArrayPos], "20 secs");
					newArrayPos++;
					numOfDriveCounters++;
				}
				trim = 2;
			}
			else
				trim =1;
		}
		else if(strFormat[currentPos + 1] == 'N')	//Name of user Counter Found
		{
			if(nameCntrPtrPos == -1)
			{
				//printf("Nfound->");
				numOfCounters++;
				nameCntrPtrPos = newArrayPos;
				tokenString[newArrayPos] =  new char[strlen(userName) + 1];		//set max size of user name found
				strcpy(tokenString[newArrayPos], userName);
				newArrayPos++;
			}
			trim = 1;
		}
		else if(strFormat[currentPos + 1] == 'P')	//IP Counter Found
		{
			if(ipCntrPtrPos == -1)
			{
				//printf("Pfound->");
				numOfCounters++;
				ipCntrPtrPos = newArrayPos;
				tokenString[newArrayPos] =  new char[strlen(mLocalhost) + 1];	//set size to localhost found
				strcpy(tokenString[newArrayPos], mLocalhost);
				newArrayPos++;
			}
			trim = 1;
		}
		else if(strFormat[currentPos + 1] == 'R')	//RAM Counter Found
		{
			if(ramCntrPtrPos == -1)
			{
				//printf("Rfound->");
				numOfCounters++;
				ramCntrPtrPos = newArrayPos;
				tokenString[newArrayPos] =  new char[MAXCPUCNTRSIZE];
				strcpy(tokenString[newArrayPos], "ram");
				newArrayPos++;
			}
			trim = 1;
		}
		else if(strFormat[currentPos + 1] == 'S')	//swap Counter Found
		{
			if(swapCntrPtrPos == -1)
			{
				//printf("Sfound->");
				numOfCounters++;
				swapCntrPtrPos = newArrayPos;
				tokenString[newArrayPos] =  new char[MAXCPUCNTRSIZE];
				strcpy(tokenString[newArrayPos], "swap");
				newArrayPos++;
			}
			trim = 1;
		}
		else if(strFormat[currentPos + 1] == 'U')	//Uptime Counter Found
		{
			if((strFormat[currentPos + 2] == 'W') || (strFormat[currentPos + 2] == 'D') || (strFormat[currentPos + 2] == 'H'))
			{
				if((uptimeCntrPos[0] == -1) && (uptimeCntrPos[1] == -1) && (uptimeCntrPos[2] == -1))
				{
					//printf("Ufound->");
					numOfCounters++;
					if(strFormat[currentPos + 2] == 'W')
						uptimeCntrPos[0] = newArrayPos;	//weeks is master counter
					else if(strFormat[currentPos + 2] == 'D')
						uptimeCntrPos[1] = newArrayPos; //days is master counter
					else
						uptimeCntrPos[2] = newArrayPos; //hours is master counter

					tokenString[newArrayPos] =  new char[MAXUPTIMECNTRSIZE];
					strcpy(tokenString[newArrayPos], "uptime");
					newArrayPos++;
				}
				trim = 2;
			}
			else
				trim = 1;
		}
		else if(strFormat[currentPos + 1] == 'W')	//Workstation Counter Found
		{
			if(workstationCntrPtrPos == -1)
			{
				//printf("Wfound->");
				numOfCounters++;
				workstationCntrPtrPos = newArrayPos;
				tokenString[newArrayPos] =  new char[strlen(compName) + 1];
				strcpy(tokenString[newArrayPos], compName);
				newArrayPos++;
			}
			trim = 1;
		}


		if (trim != 0)
			{
			if((strlen(pch) - trim) > 0 )	//if there are more characters and an identifier, trim and save the string
				{
					//printf(pch);
					tokenString[newArrayPos] =  new char[(strlen(pch) + 1)];
					strcpy(tokenString[newArrayPos], pch);
					for(int i = 0; i < (signed)(strlen(pch) - trim); i++)	//get the clipper and start trimming, or shift the characters left
						tokenString[newArrayPos][i] = tokenString[newArrayPos][i + trim];
					tokenString[newArrayPos][strlen(pch) - trim] = NULL;	//make sure there is a NULL at the end
					newArrayPos++;
					numOfRegStrings++;
				}
			//else							//this is only needed for displaying a lonely identifier, will not need a new arrayposition here.
				//printf(pch);				//else save nothing
		}
		else								//save the string even if identifier is not found in the token
		{
			//printf(pch);
			tokenString[newArrayPos] =  new char[strlen(pch) + 1];
			strcpy(tokenString[newArrayPos], pch);
			newArrayPos++;
			numOfRegStrings++;
		}

		currentPos++;					//add 1 because a % sign was found, and it is removed
		trim = 0;						//reset the trim size back to 0
		currentPos = currentPos + strlen(pch);	//keep counting along with the token ponter
		pch = strtok (NULL, "%");		//get next token
	}
}

//=========================================================================

//deal with tabs and new lines.  fix them for windows'isms
void processExtendedChar()
{
	char* pch;
	int i = 0;
	useNewLine = false;	//currently false

	while(tokenString[i] != NULL)
	{
		currentPos = 0;
		sprintf(szTemp, "%s", tokenString[i]);
		pch = strtok(szTemp, "\\");		//get first token for loop
		if(tokenString[i][0] != '\\')
		{
			currentPos = currentPos + strlen(pch);
			pch = strtok (NULL, "\\");		//get the next token for the loop
		}
		while(pch != NULL)
		{
			if(tokenString[i][currentPos + 1] == '\\')
			{
				currentPos++;
				while((currentPos < (signed)(strlen(tokenString[i]) - 1)) && (tokenString[i][currentPos + 1] == '\\'))
					currentPos++;
			}
			
			if(tokenString[i][currentPos + 1] == 't')
			{
				tokenString[i][currentPos] = '\t';
				tokenString[i][currentPos + 1] = ' ';
			}
			else if(tokenString[i][currentPos + 1] == 'n')
			{
				tokenString[i][currentPos] = '\r';
				tokenString[i][currentPos + 1] = '\n';
				useNewLine = true;
			}

			currentPos++;
			currentPos = currentPos + strlen(pch);
			pch = strtok (NULL, "\\");
		}
		i++;
	}
}

//=========================================================================

//Clean our parser
void cleanParser()
{
	for(int i = 0; i < newArrayPos; i++)
	{
		delete [] tokenString[i];
		tokenString[i] = NULL;	//not needed cause initParser does it, but is always a good idea
	}
	initParser(); //do after so newArrayPos can be used to delete the pointers
}

//=========================================================================

//initialize the input string parser
void initParser()
{
	int i;
	for(i = 0; i < 100; i++)	//initialize the pointer array
		tokenString[i] = NULL;
	currentPos = 0;			//for counting along with the token pointer of strtok()
	newArrayPos = 0;		//pointer position in an array to a new dynamically allocated string.  Also acts a a counter.
	
	numOfCounters = 0;		//Count number of counters.  Always handy to have for the future
	numOfRegStrings = 0;	//Count regular strings, not counting token strings that only have an identifier character.
	numOfDriveCounters = 0; //initialize a drive Letter counter
	cpuCntrPtrPos = -1;		//initilize all counter positions to -1.
	swapCntrPtrPos = -1;
	ramCntrPtrPos = -1;		//Quick note to self: May have to pad the dynString and make it the size MaxPossibleSizeOfCpuCounter 
	nameCntrPtrPos = -1;	//which is 4(5 including the '\0' terminator) i.e. %100. That is just the cpu counter max size.
	workstationCntrPtrPos = -1;
	ipCntrPtrPos = -1;

	//int dirCntrPtrPos[26];
	for(i = 0; i < 26; i++)		//initialize all the drive positions to -1.
		dirCntrPtrPos[i] = -1;
	//int uptimeCntrPos[3];
	for(i = 0; i < 3; i++)		//initialize all uptime positions to -1. pos[0] = weeks, pos[1] = days, pos[2] = hours
		uptimeCntrPos[i] = -1;

	strcpy(displayString, "\0");
}

//=========================================================================

//Assemble the display string from the tokenString
void makeDisplayString()
{
	strcpy(displayString, "\0");
	for(int i = 0; i < newArrayPos; i++)
	{
		
		sprintf(szTemp, "%s", tokenString[i]);
		strcat(displayString, szTemp);
	}
}

//=========================================================================

//get the counter strings that don't change
void getStaticCounters()
{
	// Get the name of the computer. 
	compName = tchBuffer; 
	GetComputerName(compName, &cchBuff);

	//Get User name
	userName = tchBuffer2;
	GetUserName(userName, &cchBuff2);

	//Get first IP of localhost
	mgetIp();
}

//=========================================================================

//set drive space values
void setDriveSpaces()
{
	int d = numOfDriveCounters;			//loop through the whole alphabet or as many drive counters found
	for(int c = 0; (c < 26) && (d > 0); c++)
	{
		if(dirCntrPtrPos[c] != -1)
		{
			sprintf(szTemp, "%c:\\", c + 65);
			drSpaceBytes = GetDiskFreeSpaceBytes(szTemp);
			sprintf(tokenString[dirCntrPtrPos[c]], "%dMB", drSpaceBytes);
			d--;
		}
	}
}

//=========================================================================

//set uptime values
void setUptime()
{
	//double seconds; 
	/*LARGE_INTEGER frequency, counter;
	if(QueryPerformanceFrequency(&frequency) == 0) 
		ticker = GetTickCount() / 60000; //Fallback 
	else 
	{ 
		QueryPerformanceCounter(&counter); 
		ticker = (DWORD)(counter.HighPart / frequency.HighPart) / 60000; 
	}*/

	DWORD ticker = GetTickCount() / 1000;
	__int64 c,f;
	QueryPerformanceCounter((_LARGE_INTEGER*)&c);
	QueryPerformanceFrequency((_LARGE_INTEGER*)&f);
	if ((c > 0) && (f > 0))	ticker = (DWORD)ceil((double)(c/f));

	if((uptimeCntrPos[0] != -1) || (uptimeCntrPos[1] != -1) || (uptimeCntrPos[2] != -1))
	{
		if(uptimeCntrPos[0] != -1)
		{
			sprintf(tokenString[uptimeCntrPos[0]], "%dw %dd %dh %dm",
			(ticker / 604800),
			((ticker / 86400) % 7),
			((ticker / 3600) % 24),
			((ticker / 60) % 60));								
		}
		else if(uptimeCntrPos[1] != -1)
		{
			sprintf(tokenString[uptimeCntrPos[1]], "%dd %dh %dm",
			(ticker / 86400),
			((ticker / 3600) % 24),
			((ticker / 60) % 60));
		}
		else
		{
			sprintf(tokenString[uptimeCntrPos[2]], "%dh %dm",
			(ticker / 3600),
			((ticker / 60) % 60));
		}
	}


	//this should be put in it's own timer that only updates every minute
	//getting the uptime should be put into struct
	//since months change length, weeks is the largest counter
	/*
	if((uptimeCntrPos[0] != -1) || (uptimeCntrPos[1] != -1) || (uptimeCntrPos[2] != -1))
	{
		if(uptimeCntrPos[0] != -1)
		{
			sprintf(tokenString[uptimeCntrPos[0]], "%dw %dd %dh %dm",
			(ticker / 10080),
			((ticker / 1440) % 7),
			((ticker / 60) % 24),
			(ticker % 60));								
		}
		else if(uptimeCntrPos[1] != -1)
		{
			sprintf(tokenString[uptimeCntrPos[1]], "%dd %dh %dm",
			(ticker / 1440),
			((ticker / 60) % 24),
			(ticker % 60));	
		}
		else
		{
			sprintf(tokenString[uptimeCntrPos[2]], "%dh %dm",
			(ticker / 60),
			(ticker % 60));
		}
	}
	*/
	/*ticker = GetTickCount();
	if((uptimeCntrPos[0] != -1) || (uptimeCntrPos[1] != -1) || (uptimeCntrPos[2] != -1))
	{
		if(uptimeCntrPos[0] != -1)
		{
			sprintf(tokenString[uptimeCntrPos[0]], "%dw %dd %dh %dm",
			(ticker / 604800000),
			(ticker / 86400000 % 7),
			(ticker / 3600000 % 24),
			((ticker / 60000) % 60));								
		}
		else if(uptimeCntrPos[1] != -1)
		{
			sprintf(tokenString[uptimeCntrPos[1]], "%dd %dh %dm",
			(ticker / 86400000),
			(ticker / 3600000 % 24),
			((ticker / 60000) % 60));
		}
		else
		{
			sprintf(tokenString[uptimeCntrPos[2]], "%dh %dm",
			(ticker / 3600000),
			((ticker / 60000) % 60));
		}
	}*/
}

//=========================================================================

//set main timer values
void setMainTimerValues()
{
	m_pStatsObj->GetData (iCPUPercent, iSwapMemPercent, iPhysMemPercent);
	//check for 0- or 100+ return, it happens when ok buttons are pressed.  Guess windows is holding that value
	if(iCPUPercent < 0 || iCPUPercent > 100) iCPUPercent = 0;
	//if(iSwapMemPercent < 0 || iSwapMemPercent > 100) iSwapMemPercent = 0;
	//if(iPhysMemPercent < 0 || iPhysMemPercent > 100) iPhysMemPercent = 0;

	if(padPercentages)
	{
		if(cpuCntrPtrPos != -1)
			sprintf(tokenString[cpuCntrPtrPos], "%.2d%%", iCPUPercent);
		if(ramCntrPtrPos != -1)
			sprintf(tokenString[ramCntrPtrPos], "%.2d%%", iPhysMemPercent);
		if(swapCntrPtrPos != -1)
			sprintf(tokenString[swapCntrPtrPos], "%.2d%%", iSwapMemPercent);
	}
	else
	{
		if(cpuCntrPtrPos != -1)
			sprintf(tokenString[cpuCntrPtrPos], "%d%%", iCPUPercent);
		if(ramCntrPtrPos != -1)
			sprintf(tokenString[ramCntrPtrPos], "%d%%", iPhysMemPercent);
		if(swapCntrPtrPos != -1)
			sprintf(tokenString[swapCntrPtrPos], "%d%%", iSwapMemPercent);
	}
}

bool resizeMyWindow(int newWidth, int newHeight)
{
	if((width < newWidth) || (height < newHeight))
	{
		if(width < newWidth)
			width = newWidth;
		if(height < newHeight)
			height = newHeight;
	}
	else
		return false;

	if(inSlit)
	{
		SetWindowPos(hwndBBSysMon, 0, 0, 0, width, height, SWP_NOACTIVATE | SWP_NOZORDER | SWP_NOMOVE);
		SendMessage(hSlit, SLIT_UPDATE, NULL, NULL);
		return true;
	}
	else
	{
		if((xpos + width) > ScreenWidth)
			xpos = ScreenWidth - width;
		if((ypos + height) > ScreenHeight)
			ypos = ScreenHeight - height;

		SetWindowPos(hwndBBSysMon, 0, xpos, ypos, width, height, SWP_NOACTIVATE | SWP_NOZORDER);
		return true;
	}
}

char* getStringInfo(char* strptr, int* xsize, int* ysize)
{
	char* pch;
	char* maxString = "";
	unsigned int maxLength = 0;
	int numOfLines = 0;
	strcpy(szTemp, strptr);
	
	pch = strtok(szTemp, "\r");
	while(pch != NULL)
	{
		if(maxLength < strlen(pch))
		{
			maxLength = strlen(pch);
			maxString = pch;
		}
		numOfLines++;
		pch = strtok (NULL, "\r");
	}
	*xsize = maxLength;
	*ysize = numOfLines;
	return maxString;
}